/*****************************************************************************
 * BitStream.cpp: Bitstream Read/Write Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "StdAfx.h"
#include "bitstream.h"

BitStream::BitStream(void)
{
	BUFFER_SIZE = 2048;
	pBuffer = NULL;
}

BitStream::~BitStream(void)
{
}

unsigned char BitStream::Mask[8] = 
	{ 0x01, 0x03, 0x07, 0x0F, 
	  0x1F, 0x3F, 0x7F, 0xFF };

// Go to next byte
// Leave this one unread/unwritten to the end
int BitStream::SeekNextByte()
{
	bitPos = -1;

	return TRUE;
}

//	Read one bit from bitstream
int BitStream::ReadBit()
{
	static const unsigned int mask[8] = { 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80 };

	if (bitPos == -1)
	{
		pBuffer++;
		bitPos = 7;
	}

	return ((*pBuffer & mask[bitPos--]) != 0);
}


//  *** last thorough tested valid version ***
//	Read one bit from bitstream
/*int BitStream::ReadBit()
{
	if (bitPos == -1)
	{
		pBuffer++;
		bitPos = 7;
		if(pBuffer - pOrgBuffer >= BUFFER_SIZE) 
		{
			return -1; 
		}
	}
	int mask = 1 << bitPos;
	int bit = *pBuffer & mask;
	bitPos--;
	
	return (bit != 0);
}*/


// ********************************
//	Read some bits from the stream
//	count = number of bit to read
//	beware: 0 <= count <= 63
// ********************************
/*__int64 BitStream::ReadBits(int count)
{
	__int64 v = 0;
	int bit = 0;

	for (int b = 0; b < count; b++)
	{
		bit = ReadBit();
		if(bit == -1) return -1;
		v = (v << 1) | bit;
	}

	return v;
}*/

__int64 BitStream::ReadBits(int count)
{
	if(count == 0) return 0;

	if (bitPos == -1)
	{
		pBuffer++;
		bitPos = 7;
	}

	// see if the bits requested are in a whole byte
	if(count - 1 <= bitPos)
	{
		int rest = count - 1;
		int bits = (*pBuffer >> (bitPos - rest)) & Mask[rest];
		bitPos -= rest + 1;
		return bits;
	}

	int v = 0;
	
	while(count > 0)
	{
		if (bitPos == -1)
		{
			pBuffer++;
			bitPos = 7;
		}

/*		// bits are on the LSB boundary
		if(count > bitPos)
		{
			v = *pBuffer & Mask[bitPos];
			count -= bitPos + 1; 
			bitPos = -1;
		}
		else
		{*/
			int rest = min(count, bitPos + 1);
			v = (v << rest) | ((*pBuffer >> (bitPos + 1 - rest)) & Mask[rest - 1]);
			bitPos -= rest;
			count -= rest;
/*			return v;
		}*/

		// see if we can get whole bytes
		while(count >= 8)
		{
			pBuffer++;
			v = (v << 8) | *pBuffer;
			count -= 8;
		}
	}

	return v;
}

// last thoroughly tested version
/*__int64 BitStream::ReadBits(int count)
{

	if(count == 0) return 0;

	if (bitPos == -1)
	{
		pBuffer++;
		bitPos = 7;
		if(pBuffer - pOrgBuffer >= BUFFER_SIZE) 
		{
			return -1; 
		}
	}

	// see if the bits requested are in a whole byte
	if(count - 1 <= bitPos)
	{
		int rest = count - 1;
		int bits = *pBuffer & (Mask[rest] << (bitPos - rest));
		int v = bits >> (bitPos - rest);
		bitPos -= rest + 1;
		return v;
	}

	int v = 0;
	
	while(count > 0)
	{
		if (bitPos == -1)
		{
			pBuffer++;
			bitPos = 7;
			if(pBuffer - pOrgBuffer >= BUFFER_SIZE) 
			{
				return -1; 
			}
		}

//		// bits are on the LSB boundary
//		if(count > bitPos)
//		{
//			v = *pBuffer & Mask[bitPos];
//			count -= bitPos + 1; 
//			bitPos = -1;
//		}
//		else
//		{
			int rest = min(count, bitPos + 1);
			int bits = *pBuffer & (Mask[rest - 1] << (bitPos + 1 - rest));
			v = (v << rest) | (bits >> (bitPos + 1 - rest));
			bitPos -= rest;
			count -= rest;
//			return v;
//		}

		// see if we can get whole bytes
		while(count >= 8)
		{
			pBuffer++;
			if(pBuffer - pOrgBuffer >= BUFFER_SIZE) 
			{
				return -1; 
			}
			v = (v << 8) | *pBuffer;
			count -= 8;
		}
	}

	return v;
}*/

// ********************************
//	Write <count> bits from <value>
//	beware: 0 <= count <= 63
// ********************************
/*int BitStream::WriteBits(__int64 value, int count)
{
/*	for(int i = 0; i < 64; i++)
	{
		if(i >= count) value &= ~(__int64)((__int64)1 << i);
	}
		if (bitPos == -1)
		{
			pBuffer++;
			bitPos = 7;
			if(pBuffer - pOrgBuffer >= BUFFER_SIZE) 
			{
				return -1; 
			}
		}
	while(count > 0)
	{
		_ASSERTX(bitPos != -1);
		int rest = min(count, bitPos + 1);

		*pBuffer &= ~(Mask[rest - 1] << (bitPos + 1 - rest));
		int tval = value >> (count - rest);
		*pBuffer |= (tval << (bitPos + 1 - rest));

		bitPos -= rest;
		count -= rest; 

		if (bitPos == -1)
		{
			pBuffer++;
			bitPos = 7;
			if(pBuffer - pOrgBuffer >= BUFFER_SIZE) 
			{
				return -1; 
			}

			// see if we can write whole bytes
			while(count >= 8)
			{
				*pBuffer = value >> (count - 8);
				count -= 8;

				pBuffer++;
				if(pBuffer - pOrgBuffer >= BUFFER_SIZE) 
				{
					return -1; 
				}
			}
		}

	}

	return 0;
}*/
/******/
int BitStream::WriteBits(__int64 value, int count)
{
	while(count > 0)
	{
		int rest = min(count, bitPos + 1);

		int x = Mask[rest - 1] << (bitPos + 1 - rest);
		// clear bit first
		*pBuffer &= ~x;
		// set bit
		*pBuffer |= ((value >> (count - rest)) << (bitPos + 1 - rest)) & x;

		bitPos -= rest;
		count -= rest;

		if (bitPos == -1)
		{
			pBuffer++;
			bitPos = 7;

			// see if we can write whole bytes
			while(count >= 8)
			{
				*pBuffer = value >> (count - 8);
				count -= 8;

				pBuffer++;
			}
		}
	}

	return 0;
}

// last thoroughly tested version
/*int BitStream::WriteBits(__int64 value, int count)
{
	while(count > 0)
	{
		int rest = min(count, bitPos + 1);

		int x = Mask[rest - 1] << (bitPos + 1 - rest);
		// clear bit first
		*pBuffer &= ~x;
		// set bit
		*pBuffer |= ((value >> (count - rest)) << (bitPos + 1 - rest)) & x;

		bitPos -= rest;
		count -= rest;

		if (bitPos == -1)
		{
			pBuffer++;
			bitPos = 7;
			if(pBuffer - pOrgBuffer >= BUFFER_SIZE) 
			{
				return -1; 
			}

			// see if we can write whole bytes
			while(count >= 8)
			{
				*pBuffer = value >> (count - 8);
				count -= 8;

				pBuffer++;
				if(pBuffer - pOrgBuffer >= BUFFER_SIZE) 
				{
					return -1; 
				}
			}
		}
	}

	return 0;
}
*/
/*int BitStream::WriteBits(__int64 value, int count)
{
	for (int i = 0; i < count; i++)
	{
		__int64 mask = 0;

		if (((value >> (count - 1 - i)) & 1) != 0)
		{
			mask = 1 << bitPos;
		}
		// clear bit first
		*pBuffer &= ~(1 << bitPos);
		// set bit
		*pBuffer |= mask;

		bitPos--;
		if (bitPos == -1)
		{
			pBuffer++;
			bitPos = 7;
			if(pBuffer - pOrgBuffer >= BUFFER_SIZE) 
			{
				return -1; 
			}
		}
	}

	return 0;
}*/
/*int BitStream::WriteBits(__int64 value, int count)
{
	for (int i = count - 1; i >= 0; i--)
	{
		__int64 mask = 0;

		if ((value & (1 << i)) != 0)
		{
			mask = 1 << bitPos;
		}
		// clear bit first
		*pBuffer &= ~(1 << bitPos);
		// set bit
		*pBuffer |= mask;

		bitPos--;
		if (bitPos == -1)
		{
			pBuffer++;
			bitPos = 7;
			if(pBuffer - pOrgBuffer >= BUFFER_SIZE) 
			{
				return -1; 
			}
		}
	}

	return 0;
}*/

/*int BitStream::WriteBits(__int64 value, int count)
{
	for (int i = count - 1; i >= 0; i--)
	{
		__int64 mask = 1 << i;
		if(WriteBit((value & mask) != 0) == -1) return -1;
	}

	return 0;
}*/

//	Write One bit to disk
int BitStream::WriteBit(int value)
{
	int mask = 0;
	if (value == 1)
	{
		mask = 1 << bitPos;
	}
	// clear bit first
	*pBuffer &= ~(1 << bitPos);
	// set bit
	*pBuffer |= mask;

	bitPos--;
	if (bitPos == -1)
	{
		pBuffer++;
		bitPos = 7;
		if(pBuffer - pOrgBuffer >= BUFFER_SIZE) 
		{
			return -1; 
		}
	}

	return 0;
}

int BitStream::Seek(int count)
{
	pBuffer += count / 8;
	bitPos -= count % 8;

	if(bitPos < 0)
	{
		bitPos += 8;
		pBuffer++;
	}

	return 0;
}

// ********************************
//	from ISO/IEC 11172:
//	The function nextbits () permits comparison of a bit string 
//	with the next bits to be decoded in the bit stream.
// ********************************
int BitStream::TestNextBits(__int64 value, int count)
{
	int l = 0;
	
//	if((((7 - bitPos) + count) / 8 + (pBuffer - pOrgBuffer)) >= BUFFER_SIZE) return -1; 

	// save the bitstream state
	int prevbitpos = bitPos;
	unsigned char *prevbufferpos = pBuffer;
	__int64 next_bits;

	next_bits = ReadBits(count);

	// restore bistream state
	bitPos = prevbitpos;
	pBuffer = prevbufferpos;
	
	// do the comparison
		 if (next_bits == value) return 1;
	else if (next_bits == -1)	 return -1;
	else						 return 0;
}

// ********************************
//	from ISO/IEC 11172:
//	The function MPEG2_GetNextStartCode () searches
//  for a start code in a possibly '0' byte padded MPEG 2 stream
// ********************************
int BitStream::MPEG2_GetNextStartCode()
{
	int val;

	do
	{
		val = TestNextBits(0x000001, 24);
		if(val == 0) Seek(8);
		if(val == -1) return -1;
	} while(val != 1);
	val = ReadBits(32);

	return val;
}

// attach a buffer to the object
void BitStream::AttachBuffer(void *buffer, int length)
{
	bitPos = 7;
	pBuffer = (unsigned char*) buffer;
	pOrgBuffer = (unsigned char*) buffer;
	BUFFER_SIZE = length;
}
