/*****************************************************************************
 * Block.cpp: Read and Write VLC Block Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#pragma once

#include "VLC.h"

typedef struct tagBLOCK_ENTRY
{
	int run;
	int level;
} BLOCK_ENTRY;

#define LUMA	0
#define CHROMA	1

class Block
{
public:
	Block(void);
	~Block(void);

	virtual void SetType(int type);
	virtual int AttachBuffer(unsigned char *buffer, int length, int bit_startpos);
	virtual int WriteDataBuffer(unsigned char *buffer, int bit_startpos);

	virtual int GetDCCoef(int prev_val);

	// parameters that need to be set before buffer attachment
	int intra;
	int intraVLCFormat;
	int quantizerScaleCode;
	int qScaleType;
	int absNumPicture;
	int absNumMacroBlock;
	int absNumBlock;
	int demoMode;

	static const VLC_SIMPLEENTRY DCT_DC_Size_LumaVLC[12];		// DCT DC Size Luminance Table   - ISO 13818-2 Table B-12 (VLC part)
	static const VLC_SIMPLEENTRY DCT_DC_Size_ChromaVLC[12];		// DCT DC Size Chrominance Table - ISO 13818-2 Table B-13 (VLC part)

	static const char DCT_DC_Size_Luma[12];						// DCT DC Size Luminance Table   - ISO 13818-2 Table B-12 (number part)
	static const char DCT_DC_Size_Chroma[12];					// DCT DC Size Chrominance Table - ISO 13818-2 Table B-13 (number part)

	static const VLC_ENTRY DCT_CoefVLC0[128];					// DCT Coefficient Table Zero - ISO 13818-2 Table B-14 (VLC part)
	static const VLC_ENTRY DCT_CoefVLC1[128];					// DCT Coefficient Table One  - ISO 13818-2 Table B-15 (VLC part)

	static const BLOCK_ENTRY DCT_Coef0[128];					// DCT Coefficient Table Zero - ISO 13818-2 Table B-14 (settings part)
	static const BLOCK_ENTRY DCT_Coef1[128];					// DCT Coefficient Table One  - ISO 13818-2 Table B-15 (settings part)

	static const char DCT_CoefIDX0[12];							// DCT Coefficient Table Zero Indices - ISO 13818-2 Table B-14
	static const char DCT_CoefIDX0m[12];						// DCT Coefficient Table Zero Indices - ISO 13818-2 Table B-14 (NOTE 3-4 modified versioN)
	static const char DCT_CoefIDX1[12];							// DCT Coefficient Table One  Indices - ISO 13818-2 Table B-15

	static const unsigned char QuantizerScaleMap0[32];			// Quantizer Scale Map (derived from quantizer_scale_code and q_scale_type) - ISO 13818-2 Table 7-6 (q_scale_type = 0)
	static const unsigned char QuantizerScaleMap1[32];			// Quantizer Scale Map (derived from quantizer_scale_code and q_scale_type) - ISO 13818-2 Table 7-6 (q_scale_type = 1)

	static const int DCT_Coef0INV[34][40];						// DCT Inverse Coefficient Table Zero [run + 2][level - 1] -> index for DCT_CoefVLC0 - from ISO 13818-2 Table B-14
	static const int DCT_Coef1INV[34][40];						// DCT Inverse Coefficient Table One  [run + 2][level - 1] -> index for DCT_CoefVLC1 - from ISO 13818-2 Table B-15

	int blockEOF;

	short          *bl_coefBuffer;
	unsigned char  *bl_runBuffer;

protected:
	unsigned char *pBuffer;
	int bufferSize;

	int type;
	static VLC bstream;

//	short Coef[64];
//	unsigned char rCoef[64];
	int dct_dc_size, dct_dc_size_vlc_idx;

	int ReadBlock(int begin_idx);

	// debug purposes only
	int testBitSize;

};
