/*****************************************************************************
 * IfoEdit.cpp: Class to Edit DVD Ifo files
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "StdAfx.h"
#include "ifoedit.h"

CIFOEdit::CIFOEdit(void)
{
}

CIFOEdit::~CIFOEdit(void)
{
}

CIFOEdit::CIFOEdit(const char *srcFile)
{
	OpenFile(srcFile);
}

void CIFOEdit::Close()
{
	CloseFile();
	ZeroMemory(m_VTSI_MAT.VTSTT_audio_attributes,sizeof(m_VTSI_MAT.VTSTT_audio_attributes));
	ZeroMemory(m_VTSI_MAT.VTSTT_subtitle_attributes,sizeof(m_VTSI_MAT.VTSTT_subtitle_attributes));
	m_VTS_PGCITI.PGCs.clear();
	m_VTSM_PGCI_UT.vtsm_lu.clear();
}

bool CIFOEdit::Open(const char *destFile)
{
	bool ret = OpenFile(destFile);
	checkVTS();

	if(m_isVTS)
	{
		// VTS
		load_VTSI_MAT();
		load_VTS_PTT_SRPT();
		load_VTS_PGCITI();
		load_VTSM_PGCI_UT();
		load_VTS_TMAPTI();
		load_VTSM_C_ADT();
		load_VTSM_VOBU_ADMAP();
		load_VTS_C_ADT();
		load_VTS_VOBU_ADMAP();
	}
	else
	{
		// VMG
		load_VMG_MAT();
		load_VMG_TT_SRPT();
		load_VMGM_PGCI_UT();
		load_VMG_PTL_MAIT();
		load_VMG_VTS_ATRT();
		load_VMGM_C_ADT();
		load_VMGM_VOBU_ADMAP();
	}

	return ret;
}

void CIFOEdit::checkVTS()
{
	unsigned char tmp[13];
	GetMEM(0,12,tmp);
	tmp[12]='\0';
	if(!strcmp((const char*)tmp,"DVDVIDEO-VTS")) m_isVTS = true;
	else if (!strcmp((const char*)tmp,"DVDVIDEO-VMG")) m_isVTS = false;
	else MessageBox(NULL,"Unknown file format!","E R R O R",MB_OK|MB_ICONERROR|MB_SETFOREGROUND);
}

void CIFOEdit::_VTSI_MAT(bool save)
{
	// VTSI_MAT globals
	TransferMEM(0,12,m_VTSI_MAT.VTS_indetifier,save);

	m_VTSI_MAT.last_VTS_sector =
		TransferINT32(0x0c,save,m_VTSI_MAT.last_VTS_sector);
	m_VTSI_MAT.last_VTSI_sector =
		TransferINT32(0x1c,save,m_VTSI_MAT.last_VTSI_sector);

	m_VTSI_MAT.version_number[0] =
		TransferBYTE(0x20,save,m_VTSI_MAT.version_number[0]);
	m_VTSI_MAT.version_number[1] =
		TransferBYTE(0x21,save,m_VTSI_MAT.version_number[1]);

	m_VTSI_MAT.VTS_category =
		TransferINT32(0x22,save,m_VTSI_MAT.VTS_category);
	m_VTSI_MAT.end_of_VTSI_MAT =
		TransferINT32(0x80,save,m_VTSI_MAT.end_of_VTSI_MAT);

	m_VTSI_MAT.offset_VTSM_VOBS =
		TransferINT32(0xc0,save,m_VTSI_MAT.offset_VTSM_VOBS);
	m_VTSI_MAT.offset_VTSTT_VOBS =
		TransferINT32(0xc4,save,m_VTSI_MAT.offset_VTSTT_VOBS);
	m_VTSI_MAT.offset_VTS_PTT_SRPT =
		TransferINT32(0xc8,save,m_VTSI_MAT.offset_VTS_PTT_SRPT);
	m_VTSI_MAT.offset_VTS_PGCIT =
		TransferINT32(0xcc,save,m_VTSI_MAT.offset_VTS_PGCIT);
	m_VTSI_MAT.offset_VTSM_PGCI_UT =
		TransferINT32(0xd0,save,m_VTSI_MAT.offset_VTSM_PGCI_UT);
	m_VTSI_MAT.offset_VTS_TMAPT =
		TransferINT32(0xd4,save,m_VTSI_MAT.offset_VTS_TMAPT);
	m_VTSI_MAT.offset_VTSM_C_ADT =
		TransferINT32(0xd8,save,m_VTSI_MAT.offset_VTSM_C_ADT);
	m_VTSI_MAT.offset_VTSM_VOBU_ADMAP =
		TransferINT32(0xdc,save,m_VTSI_MAT.offset_VTSM_VOBU_ADMAP);
	m_VTSI_MAT.offset_VTS_C_ADT =
		TransferINT32(0xe0,save,m_VTSI_MAT.offset_VTS_C_ADT);
	m_VTSI_MAT.offset_VTS_VOBU_ADMAP =
		TransferINT32(0xe4,save,m_VTSI_MAT.offset_VTS_VOBU_ADMAP);

	// VTSM
	TransferMEM(0x100,sizeof(m_VTSI_MAT.VTSM_video_attribute),
						(BYTE*)&m_VTSI_MAT.VTSM_video_attribute,save);


	m_VTSI_MAT.nb_VTSM_audio_attributes =
		TransferBYTE(0x103,save,m_VTSI_MAT.nb_VTSM_audio_attributes);
	TransferMEM(0x104,sizeof(m_VTSI_MAT.VTSM_audio_attributes),
						(BYTE*)&m_VTSI_MAT.VTSM_audio_attributes,save);
	m_VTSI_MAT.nb_VTSM_subt_attribute =
		TransferBYTE(0x155,save,m_VTSI_MAT.nb_VTSM_subt_attribute);
	TransferMEM(0x156,sizeof(m_VTSI_MAT.VTSM_subtitle_attribute),
						(BYTE*)&m_VTSI_MAT.VTSM_subtitle_attribute,save);

	// VTSTT
	TransferMEM(0x200,sizeof(m_VTSI_MAT.VTSTT_video_attribute),
						(BYTE*)&m_VTSI_MAT.VTSTT_video_attribute,save);
	m_VTSI_MAT.nb_VTSTT_audio_attributes =
		TransferBYTE(0x203,save,m_VTSI_MAT.nb_VTSTT_audio_attributes);
	TransferMEM(0x204,sizeof(m_VTSI_MAT.VTSTT_audio_attributes),
						(BYTE*)&m_VTSI_MAT.VTSTT_audio_attributes,save);
	m_VTSI_MAT.nb_VTSTT_subt_attribute =
	     TransferBYTE(0x255,save,m_VTSI_MAT.nb_VTSTT_subt_attribute);
	TransferMEM(0x256,sizeof(m_VTSI_MAT.VTSTT_subtitle_attributes),
						(BYTE*)&m_VTSI_MAT.VTSTT_subtitle_attributes,save);

}

void CIFOEdit::Save(const char *destFile)
{
	if(m_isVTS)
	{
		// VTS
		save_VTSI_MAT();
		save_VTS_PTT_SRPT();
		save_VTS_PGCITI();
		save_VTSM_PGCI_UT();
		save_VTS_TMAPTI();
		save_VTSM_C_ADT();
		save_VTSM_VOBU_ADMAP();
		save_VTS_C_ADT();
		save_VTS_VOBU_ADMAP();
	}
	else
	{
		// VMG
		save_VMG_MAT();
		save_VMG_TT_SRPT();
		save_VMGM_PGCI_UT();
		save_VMG_PTL_MAIT();
		save_VMG_VTS_ATRT();
		save_VMGM_C_ADT();
		save_VMGM_VOBU_ADMAP();
	}
	SaveFile(destFile);
}

VTSI_MAT_DATA CIFOEdit::Get_VTSI_MAT()
{
	return m_VTSI_MAT;
}

void CIFOEdit::Set_VTSI_MAT(VTSI_MAT_DATA data, bool save)
{
	m_VTSI_MAT = data;
	if(save) save_VTSI_MAT();
}


void CIFOEdit::_VTS_PGCITI(bool save)
{
	if(!m_VTSI_MAT.offset_VTS_PGCIT) return;
	int offset = m_VTSI_MAT.offset_VTS_PGCIT*SECTOR_SIZE;
	int pgciti_off=offset;
	m_VTS_PGCITI.nb_PGCs = TransferSHORT(offset,save,m_VTS_PGCITI.nb_PGCs);
	m_VTS_PGCITI.VTS_PGCI_SRP_end = TransferINT32(offset + 4,save,
												m_VTS_PGCITI.VTS_PGCI_SRP_end);
	m_VTS_PGCITI.PGC_category = TransferINT32(offset + 8,save,m_VTS_PGCITI.PGC_category);
	m_VTS_PGCITI.offset_VTS_PGC = TransferINT32(offset+12,save,m_VTS_PGCITI.offset_VTS_PGC);

	// read the PGCs
	// starting from offset_VTS_PGCIT + m_VTS_PGCITI.offset_VTS_PGC
	offset += m_VTS_PGCITI.offset_VTS_PGC;
	int i=0;
	while(offset<m_VTS_PGCITI.VTS_PGCI_SRP_end+pgciti_off)
	{
		VTS_PGC pgc;
		if(save) pgc = m_VTS_PGCITI.PGCs[i];
		i++;
		TransPGC(&pgc,offset,save);
		if(!save) m_VTS_PGCITI.PGCs.push_back(pgc);
	}
}

VTS_PGCITI CIFOEdit::Get_VTS_PGCITI()
{
	return m_VTS_PGCITI;
}

void CIFOEdit::Set_VTS_PGCITI(VTS_PGCITI data, bool save)
{
	m_VTS_PGCITI = data;
	if(save)
		save_VTS_PGCITI();
}

void CIFOEdit::_VTS_PTT_SRPT(bool save)
{
	if(!m_VTSI_MAT.offset_VTS_PTT_SRPT) return;
	int offset = m_VTSI_MAT.offset_VTS_PTT_SRPT*SECTOR_SIZE;
	m_VTS_PTT_SRPT.nbTitles = TransferSHORT(offset,save,m_VTS_PTT_SRPT.nbTitles);
	m_VTS_PTT_SRPT.end_address = TransferINT32(offset+0x04,save,m_VTS_PTT_SRPT.end_address);
	m_VTS_PTT_SRPT.offset_PTT = TransferINT32(offset+0x08,save,m_VTS_PTT_SRPT.offset_PTT);
	offset += m_VTS_PTT_SRPT.offset_PTT;
	int i=0;
	while(offset<m_VTS_PTT_SRPT.end_address+m_VTSI_MAT.offset_VTS_PTT_SRPT*SECTOR_SIZE)
	{
		VTS_PTT ptt;
		if(save) ptt = m_VTS_PTT_SRPT.vts_ptt[i];
		else ZeroMemory(&ptt,sizeof(ptt));
		i++;

		ptt.PGCN = TransferSHORT(offset,save,ptt.PGCN);
		ptt.PGN = TransferSHORT(offset+2,save,ptt.PGN);

		if(!save) m_VTS_PTT_SRPT.vts_ptt.push_back(ptt);
		offset += 4;
	}
}

VTS_PTT_SRPT CIFOEdit::Get_VTS_PTT_SRPT()
{
	return m_VTS_PTT_SRPT;
}

void CIFOEdit::Set_VTS_PTT_SRPT(VTS_PTT_SRPT data, bool save)
{
	m_VTS_PTT_SRPT = data;
	if(save)
		save_VTS_PTT_SRPT();
}


void CIFOEdit::_VTSM_PGCI_UT(bool save)
{
	if(!m_VTSI_MAT.offset_VTSM_PGCI_UT) return;
	int offset = m_VTSI_MAT.offset_VTSM_PGCI_UT*SECTOR_SIZE;
	int VTSM_PGCI_UT_off = offset;
	m_VTSM_PGCI_UT.nbUnits = TransferSHORT(offset,save,m_VTSM_PGCI_UT.nbUnits);
	m_VTSM_PGCI_UT.end_adress = TransferINT32(offset+0x04,save,m_VTSM_PGCI_UT.end_adress);
	offset+=0x08;
	for(int i=0;i<m_VTSM_PGCI_UT.nbUnits;i++)
	{
		VTSM_LU_INFO lui;
		if(save) lui = m_VTSM_PGCI_UT.vtsm_lu_infos[i];

		TransferMEM(offset,2,lui.lang_code,save);
		lui.lang_code_ext = TransferBYTE(offset+0x02,save,lui.lang_code_ext);
		lui.existence_flags = TransferBYTE(offset+0x03,save,lui.existence_flags);
		lui.offset_VTSM_LU = TransferINT32(offset+0x04,save,lui.offset_VTSM_LU);
		if(!save) m_VTSM_PGCI_UT.vtsm_lu_infos.push_back(lui);
		offset+=0x08;
	}
	for(int i=0;i<m_VTSM_PGCI_UT.nbUnits;i++)
	{
		offset = m_VTSM_PGCI_UT.vtsm_lu_infos[i].offset_VTSM_LU+VTSM_PGCI_UT_off;
		int VTSM_LU_off = offset;
		VTSM_LU lu;
		if(save) lu = m_VTSM_PGCI_UT.vtsm_lu[i];
		lu.nbMenus = TransferSHORT(offset,save,lu.nbMenus);
		lu.end_adress = TransferINT32(offset+0x04,save,lu.end_adress);
		offset += 8;
		int ix=0;
		for(int ix=0;ix<lu.nbMenus;ix++)
		{
			PGC_MENU cat;
			if(save) cat = lu.Menus[i];
			else ZeroMemory(&cat,sizeof(cat));
			TransferMEM(offset,4,(BYTE*)cat.category,save);
			cat.offset_PGCI = TransferINT32(offset+0x04,save,cat.offset_PGCI);
			offset+=8;
			if(!save) lu.Menus.push_back(cat);

		}

	//	offset = VMGM_LU_off + lu.offset_VMGM_PGC;
		int VTSM_PGC_off = offset;
		for(int ig=0;ig<lu.nbMenus;ig++)
		{
			VTS_PGC pgc;
			if(save) pgc = lu.vtsm_pgc[ig];
			offset = lu.Menus[ig].offset_PGCI+VTSM_LU_off;
			TransPGC(&pgc,offset,save);
			if(!save) lu.vtsm_pgc.push_back(pgc);
		}
		if(!save) m_VTSM_PGCI_UT.vtsm_lu.push_back(lu);
	}
}

VTSM_PGCI_UT CIFOEdit::Get_VTSM_PGCI_UT()
{
	return m_VTSM_PGCI_UT;
}

void CIFOEdit::Set_VTSM_PGCI_UT(VTSM_PGCI_UT data, bool save)
{
	m_VTSM_PGCI_UT = data;
	if(save)
		save_VTSM_PGCI_UT();
}

void CIFOEdit::TransPGC(VTS_PGC *pgc, int &offset, bool save)
{
	int pgc_off = offset;
	// PGC header
	pgc->nb_programs = TransferBYTE(offset+0x02,save,pgc->nb_programs);
	pgc->nb_cells = TransferBYTE(offset+0x03,save,pgc->nb_cells);
	TransferMEM(offset+0x04,4,(BYTE*)&pgc->playback_time,save);
	pgc->prohibited_ops = TransferINT32(offset+0x08,save,pgc->prohibited_ops);
	TransferMEM(offset+0x0c,sizeof(pgc->audio),(BYTE*)pgc->audio,save);
	TransferMEM(offset+0x1c,sizeof(pgc->subpicture),(BYTE*)pgc->subpicture,save);
	pgc->next_PGCN = TransferSHORT(offset+0x9c,save,pgc->next_PGCN);
	pgc->prev_PGCN = TransferSHORT(offset+0x9e,save,pgc->prev_PGCN);
	pgc->goup_PGCN = TransferSHORT(offset+0xa0,save,pgc->goup_PGCN);
	pgc->PGC_still_time = TransferBYTE(offset+0xa2,save,pgc->PGC_still_time);
	pgc->playback_mode = TransferBYTE(offset+0xa3,save,pgc->playback_mode);
	TransferMEM(offset+0xa4,sizeof(pgc->palette),(BYTE*)pgc->palette,save);
	pgc->offset_commands = TransferSHORT(offset+0xe4,save,pgc->offset_commands);
	pgc->offset_program_map = TransferSHORT(offset+0xe6,save,pgc->offset_program_map);
	pgc->offset_playback_info = TransferSHORT(offset+0xe8,save,pgc->offset_playback_info);
	pgc->offset_position_info = TransferSHORT(offset+0xea,save,pgc->offset_position_info);
	offset += 0xEC;
	// command table
	// -= to be reinserted =-
/*	if(pgc->offset_commands)
	{
		offset = pgc_off + pgc->offset_commands;
		int command_offset = offset;
		pgc->command_table.nbPreCommands = TransferSHORT(offset,save,pgc->command_table.nbPreCommands);
		pgc->command_table.nbPostCommands = TransferSHORT(offset+0x02,save,pgc->command_table.nbPostCommands);
		pgc->command_table.nbCellCommands = TransferSHORT(offset+0x04,save,pgc->command_table.nbCellCommands);
		pgc->command_table.end_address = TransferSHORT(offset+0x06,save,pgc->command_table.end_address);
		offset += 0x08;
		for(int i=0;i<pgc->command_table.nbPreCommands;i++)
		{ // read/write the pre-commands
			CHAR_8 c;
			if(save) memcpy(c.c,pgc->command_table.pre_commands[i].c,8);
			TransferMEM(offset,8,(BYTE*)c.c,save);
			if(!save) pgc->command_table.pre_commands.push_back(c);
			offset += 8;
		}
		for(int i=0;i<pgc->command_table.nbPostCommands;i++)
		{ // read/write the post-commands
			CHAR_8 c;
			if(save) memcpy(c.c,pgc->command_table.post_commands[i].c,8);
			TransferMEM(offset,8,(BYTE*)c.c,save);
			if(!save) pgc->command_table.post_commands.push_back(c);
			offset += 8;
		}
		for(int i=0;i<pgc->command_table.nbCellCommands;i++)
		{ // read/write the cell-commands
			CHAR_8 c;
			if(save) memcpy(c.c,pgc->command_table.cell_commands[i].c,8);
			TransferMEM(offset,8,(BYTE*)c.c,save);
			if(!save) pgc->command_table.cell_commands.push_back(c);
			offset += 8;
		}
	}*/


	// program map
	if(pgc->offset_program_map)
	{
		offset = pgc_off + pgc->offset_program_map;
		for(int i=0;i<pgc->nb_cells;i++)
		{
			unsigned char uc=0;
			if(save) uc = pgc->program_map.entry_cell_number[i];
			uc = TransferBYTE(offset,save,uc);
			offset++;
			if(!save) pgc->program_map.entry_cell_number.push_back(uc);
		}
	}


	// cell playback information table entry
	struct {
		char angle				:1;
		char SCR_discontinuity  :1;
		char interleaved		:1;
		char seamless_playback	:1;
		char block_type			:2;
		char cell_type			:2;
		char					:6;
		char restricted			:1;
		char					:1;
	} cell_category;

	if(pgc->offset_playback_info)
	{
		offset = pgc_off + pgc->offset_playback_info;
		for(int i=0;i<pgc->nb_cells;i++)
		{
			PGC_CELL_PLAYBACK_INFO cell_playback_info;
			if(save) cell_playback_info = pgc->cell_playback_info[i];

			if(save)
			{
				cell_category.angle = cell_playback_info.angle;
				cell_category.SCR_discontinuity = cell_playback_info.SCR_discontinuity;
				cell_category.interleaved = cell_playback_info.interleaved;
				cell_category.seamless_playback = cell_playback_info.seamless_playback;
				cell_category.block_type = cell_playback_info.block_type;
				cell_category.cell_type = cell_playback_info.cell_type;
				cell_category.restricted = cell_playback_info.restricted;
			}
			TransferMEM(offset,2,(BYTE*)&cell_category,save);
			if(!save)
			{
				cell_playback_info.angle = cell_category.angle;
				cell_playback_info.SCR_discontinuity=cell_category.SCR_discontinuity;
				cell_playback_info.interleaved=cell_category.interleaved;
				cell_playback_info.seamless_playback=cell_category.seamless_playback;
				cell_playback_info.block_type=cell_category.block_type;
				cell_playback_info.cell_type=cell_category.cell_type;
				cell_playback_info.restricted=cell_category.restricted;
			}
			offset += 2;
			cell_playback_info.cell_still_time = TransferBYTE(offset,save,cell_playback_info.cell_still_time);
			offset++;
			cell_playback_info.cell_command = TransferBYTE(offset,save,cell_playback_info.cell_command);
			offset++;
			TransferMEM(offset,4,(BYTE*)&cell_playback_info.cell_playback_time,save);offset+=4;
			cell_playback_info.first_VOBU_start = TransferINT32(offset,save,cell_playback_info.first_VOBU_start);
			offset+=4;
			cell_playback_info.first_ILVU_end = TransferINT32(offset,save,cell_playback_info.first_ILVU_end);
			offset+=4;
			cell_playback_info.last_VOBU_start = TransferINT32(offset,save,cell_playback_info.last_VOBU_start);
			offset+=4;
			cell_playback_info.last_VOBU_end = TransferINT32(offset,save,cell_playback_info.last_VOBU_end);
			offset+=4;
			if(!save) pgc->cell_playback_info.push_back(cell_playback_info);
		}
	}

	// cell position information table entry
	if(pgc->offset_position_info)
	{
		offset = pgc_off+pgc->offset_position_info;
		for(int i=0;i<pgc->nb_cells;i++)
		{
			PGC_CELL_POSITION_INFO ci;
			if(save) ci = pgc->cell_position_infos[i];
			else ZeroMemory(&ci,sizeof(ci));
			ci.vob_id = TransferSHORT(offset,save,ci.vob_id);
			ci.reserved = TransferBYTE(offset+2,save,ci.reserved);
			ci.cell_id = TransferBYTE(offset+3,save,ci.cell_id);
			offset += 4;
			if(!save) pgc->cell_position_infos.push_back(ci);
		}
	}
}

void CIFOEdit::_VTS_TMAPTI(bool save)
{
	if(!m_VTSI_MAT.offset_VTS_TMAPT) return;
	int offset = m_VTSI_MAT.offset_VTS_TMAPT*SECTOR_SIZE;

	m_VTS_TMAPTI.nbPGCs = TransferSHORT(offset,save,m_VTS_TMAPTI.nbPGCs);
	m_VTS_TMAPTI.end_adress = TransferINT32(offset+0x04,save,m_VTS_TMAPTI.end_adress);
	m_VTS_TMAPTI.offset_VTS_TMAP = TransferINT32(offset+0x08,save,m_VTS_TMAPTI.offset_VTS_TMAP);

	int off_TMAP = offset + m_VTS_TMAPTI.offset_VTS_TMAP;
/*	VTS_TMAP tmap;
	if(save) tmap = m_VTS_TMAPTI.vts_map;*/
	if(!save) m_VTS_TMAPTI.vts_map.entries.empty();
	while(off_TMAP<m_VTS_TMAPTI.end_adress+offset)
	{
		// load/save the entries
//		VTS_TMAP tmap;
		m_VTS_TMAPTI.vts_map.time_unit = TransferBYTE(off_TMAP,save,m_VTS_TMAPTI.vts_map.time_unit);
		m_VTS_TMAPTI.vts_map.nbEntries = TransferSHORT(off_TMAP+0x02,save,m_VTS_TMAPTI.vts_map.nbEntries);
		off_TMAP += 0x04;
		for(int i=0;i<m_VTS_TMAPTI.vts_map.nbEntries;i++)
		{
			if(save)
				PutINT32(off_TMAP,m_VTS_TMAPTI.vts_map.entries[i]);
			else
				m_VTS_TMAPTI.vts_map.entries.push_back(GetINT32(off_TMAP));
			off_TMAP += 0x04;
		}
	}
//	if(!save) m_VTS_TMAPTI.vts_map = tmap;
}

VTS_TMAPTI CIFOEdit::Get_VTS_TMAPTI()
{
	return m_VTS_TMAPTI;
}

void CIFOEdit::Set_VTS_TMAPTI(VTS_TMAPTI data, bool save)
{
	m_VTS_TMAPTI = data;
	if(save)
		save_VTS_TMAPTI();
}

void CIFOEdit::Trans_C_ADT(VTS_C_ADT *adt, int &offset, bool save)
{

	adt->nbCells = TransferSHORT(offset,save,adt->nbCells);
	adt->end_adress = TransferINT32(offset+4,save,adt->end_adress);

	int entry_off = offset+0x08;

	for(int i=0;i<adt->nbCells;i++)
	{   // load/save the entries
		VTS_C_ADT_ENTRY entry;
		if(save) entry = adt->entries[i];
		else ZeroMemory(&entry,sizeof(entry));
		entry.vobID = TransferSHORT(entry_off,save,entry.vobID);
		entry.cellID = TransferBYTE(entry_off+0x02,save,entry.cellID);
		entry.start_sec_VOB = TransferINT32(entry_off+0x04,save,entry.start_sec_VOB);
		entry.end_sec_VOB = TransferINT32(entry_off+0x08,save,entry.end_sec_VOB);
		entry_off+=12;
		if(!save) adt->entries.push_back(entry);
	}
}

void CIFOEdit::_VTSM_C_ADT(bool save)
{
	if(!m_VTSI_MAT.offset_VTSM_C_ADT) return;
	int offset = m_VTSI_MAT.offset_VTSM_C_ADT*SECTOR_SIZE;
	Trans_C_ADT((VTS_C_ADT*)&m_VTSM_C_ADT,offset,save);
}

VTS_C_ADT CIFOEdit::Get_VTSM_C_ADT()
{
	return m_VTSM_C_ADT;
}

void CIFOEdit::Set_VTSM_C_ADT(VTS_C_ADT data, bool save)
{
	m_VTSM_C_ADT = data;
	if(save)
		save_VTSM_C_ADT();
}

void CIFOEdit::_VTSM_VOBU_ADMAP(bool save)
{
	if(!m_VTSI_MAT.offset_VTSM_VOBU_ADMAP) return;
	int offset = m_VTSI_MAT.offset_VTSM_VOBU_ADMAP*SECTOR_SIZE;
	Trans_VOBU_ADMAP((VTS_VOBU_ADMAP *)&m_VTSM_VOBU_ADMAP,offset,save);
}

VTS_VOBU_ADMAP CIFOEdit::Get_VTSM_VOBU_ADMAP()
{
	return m_VTSM_VOBU_ADMAP;
}

void CIFOEdit::Set_VTSM_VOBU_ADMAP(VTS_VOBU_ADMAP data, bool save)
{
	m_VTSM_VOBU_ADMAP = data;
	if(save)
		save_VTSM_VOBU_ADMAP();
}

void CIFOEdit::_VTS_C_ADT(bool save)
{
	int offset = m_VTSI_MAT.offset_VTS_C_ADT*SECTOR_SIZE;
	Trans_C_ADT((VTS_C_ADT*)&m_VTS_C_ADT,offset,save);
}

VTS_C_ADT CIFOEdit::Get_VTS_C_ADT()
{
	return m_VTS_C_ADT;
}

void CIFOEdit::Set_VTS_C_ADT(VTS_C_ADT data, bool save)
{
	m_VTS_C_ADT = data;
	if(save)
		save_VTS_C_ADT();
}

void CIFOEdit::Trans_VOBU_ADMAP(VTS_VOBU_ADMAP *admap, int &offset, bool save)
{
	admap->end_adress = TransferINT32(offset,save,admap->end_adress);
	admap->star_sec_first_VOBU = TransferINT32(offset,save,admap->star_sec_first_VOBU);
}

void CIFOEdit::_VTS_VOBU_ADMAP(bool save)
{
	int offset = m_VTSI_MAT.offset_VTS_VOBU_ADMAP*SECTOR_SIZE;
	Trans_VOBU_ADMAP((VTS_VOBU_ADMAP *)&m_VTS_VOBU_ADMAP,offset,save);
}

VTS_VOBU_ADMAP CIFOEdit::Get_VTS_VOBU_ADMAP()
{
	return m_VTS_VOBU_ADMAP;
}

void CIFOEdit::Set_VTS_VOBU_ADMAP(VTS_VOBU_ADMAP data, bool save)
{
	m_VTS_VOBU_ADMAP = data;
	if(save)
		save_VTS_VOBU_ADMAP();
}

void CIFOEdit::_VMG_MAT(bool save)
{
	// VMG_MAT globals
	TransferMEM(0,12,m_VMG_MAT.VMG_indetifier,save);

	m_VMG_MAT.last_VMG_sector =
		TransferINT32(0x0c,save,m_VMG_MAT.last_VMG_sector);
	m_VMG_MAT.last_IFO_sector =
		TransferINT32(0x1c,save,m_VMG_MAT.last_IFO_sector);

	m_VMG_MAT.version_number[0] =
		TransferBYTE(0x20,save,m_VMG_MAT.version_number[0]);
	m_VMG_MAT.version_number[1] =
		TransferBYTE(0x21,save,m_VMG_MAT.version_number[1]);
	m_VMG_MAT.VMG_category =
		TransferINT32(0x22,save,m_VMG_MAT.VMG_category);

	m_VMG_MAT.nbVolumes = TransferSHORT(0x26,save,m_VMG_MAT.nbVolumes);
	m_VMG_MAT.vol_number = TransferSHORT(0x28,save,m_VMG_MAT.vol_number);
	m_VMG_MAT.side_ID = TransferBYTE(0x2A,save,m_VMG_MAT.side_ID);
	m_VMG_MAT.nb_title_sets = TransferSHORT(0x3E,save,m_VMG_MAT.nb_title_sets);
	TransferMEM(0x40,40,m_VMG_MAT.provider_ID,save);
	TransferMEM(0x60,8,m_VMG_MAT.VMG_POS,save);
	m_VMG_MAT.end_of_VMGI_MAT = TransferINT32(0x80,save,m_VMG_MAT.end_of_VMGI_MAT);
	m_VMG_MAT.offset_FP_PGC = TransferINT32(0x84,save,m_VMG_MAT.offset_FP_PGC);

	m_VMG_MAT.offset_VMGM_VOBS =
		TransferINT32(0xc0,save,m_VMG_MAT.offset_VMGM_VOBS);
	m_VMG_MAT.offset_TT_SRPT =
		TransferINT32(0xc4,save,m_VMG_MAT.offset_TT_SRPT);
	m_VMG_MAT.offset_VMGM_PGCI_UT =
		TransferINT32(0xc8,save,m_VMG_MAT.offset_VMGM_PGCI_UT);
	m_VMG_MAT.offset_VMG_PTL_MAIT =
		TransferINT32(0xcc,save,m_VMG_MAT.offset_VMG_PTL_MAIT);
	m_VMG_MAT.offset_VMG_VTS_ATRT =
		TransferINT32(0xd0,save,m_VMG_MAT.offset_VMG_VTS_ATRT);
	m_VMG_MAT.offset_VMG_TXTDT_MG =
		TransferINT32(0xd4,save,m_VMG_MAT.offset_VMG_TXTDT_MG);
	m_VMG_MAT.offset_VMGM_C_ADT =
		TransferINT32(0xd8,save,m_VMG_MAT.offset_VMGM_C_ADT);
	m_VMG_MAT.offset_VMGM_VOBU_ADMAP =
		TransferINT32(0xdc,save,m_VMG_MAT.offset_VMGM_VOBU_ADMAP);

	// VMGM
	TransferMEM(0x100,sizeof(m_VMG_MAT.VMGM_video_attribute),
						(BYTE*)&m_VMG_MAT.VMGM_video_attribute,save);


	m_VMG_MAT.nb_VMGM_audio_attributes =
		TransferBYTE(0x103,save,m_VMG_MAT.nb_VMGM_audio_attributes);
	TransferMEM(0x104,sizeof(m_VMG_MAT.VMGM_audio_attributes),
						(BYTE*)&m_VMG_MAT.VMGM_audio_attributes,save);
	m_VMG_MAT.nb_VMGM_subt_attribute =
		TransferBYTE(0x155,save,m_VMG_MAT.nb_VMGM_subt_attribute);
	TransferMEM(0x156,sizeof(m_VMG_MAT.VMGM_subtitle_attribute),
						(BYTE*)&m_VMG_MAT.VMGM_subtitle_attribute,save);

	int offset = m_VMG_MAT.offset_FP_PGC;
	TransPGC(&m_VMG_MAT.FP_PGC,offset,save);
}

VMG_MAT_DATA CIFOEdit::Get_VMG_MAT()
{
	return m_VMG_MAT;
}

void CIFOEdit::Set_VMG_MAT(VMG_MAT_DATA data, bool save)
{
	m_VMG_MAT = data;
	if(save)
		save_VMG_MAT();
}

void CIFOEdit::_VMG_TT_SRPT(bool save)
{
	int offset = m_VMG_MAT.offset_TT_SRPT*SECTOR_SIZE;
	int TT_SRPT_off = offset;

	m_VMG_TT_SRPT.nbTitles = TransferSHORT(offset,save,m_VMG_TT_SRPT.nbTitles);
	m_VMG_TT_SRPT.end_adress = TransferINT32(offset+0x04,save,m_VMG_TT_SRPT.end_adress);

	offset += 8;
	int i=0;
	while(offset<m_VMG_TT_SRPT.end_adress+TT_SRPT_off)
	{
		VMG_TT_SRPT_ENTRY tt_srpt;
		if(save) tt_srpt = m_VMG_TT_SRPT.entries[i];
		else ZeroMemory(&tt_srpt,sizeof(tt_srpt));
		i++;
		tt_srpt.title_type = TransferBYTE(offset,save,tt_srpt.title_type);
		tt_srpt.nbAngels = TransferBYTE(offset+0x01,save,tt_srpt.nbAngels);
		tt_srpt.nbChapters = TransferSHORT(offset+0x02,save,tt_srpt.nbChapters);
		tt_srpt.par_mng_mask = TransferSHORT(offset+0x04,save,tt_srpt.par_mng_mask);
		tt_srpt.VTS_TTN = TransferBYTE(offset+0x06,save,tt_srpt.VTS_TTN);
		tt_srpt.VTSN = TransferBYTE(offset+0x07,save,tt_srpt.VTSN);
		tt_srpt.start_sec_VTS = TransferINT32(offset+0x08,save,tt_srpt.start_sec_VTS);
		offset += 0x0c;
		if(!save) m_VMG_TT_SRPT.entries.push_back(tt_srpt);
	}
}

VMG_TT_SRPT CIFOEdit::Get_VMG_TT_SRPT()
{
	return m_VMG_TT_SRPT;
}

void CIFOEdit::Set_VMG_TT_SRPT(VMG_TT_SRPT data, bool save)
{
	m_VMG_TT_SRPT = data;
	if(save)
		save_VMG_TT_SRPT();
}

void CIFOEdit::_VMGM_PGCI_UT(bool save)
{
	int offset = m_VMG_MAT.offset_VMGM_PGCI_UT*SECTOR_SIZE;
	int VMGM_PGCI_UT_off = offset;
	m_VMGM_PGCI_UT.nbLangUnits = TransferSHORT(offset,save,m_VMGM_PGCI_UT.nbLangUnits);
	m_VMGM_PGCI_UT.end_adress = TransferINT32(offset+0x04,save,m_VMGM_PGCI_UT.end_adress);
	TransferMEM(offset+0x08,2,m_VMGM_PGCI_UT.lang_code,save);
	m_VMGM_PGCI_UT.lang_ext = TransferBYTE(offset+0x0a,save,m_VMGM_PGCI_UT.lang_ext);
	m_VMGM_PGCI_UT.menu_existence_flag = TransferBYTE(offset+0x0b,save,m_VMGM_PGCI_UT.menu_existence_flag);
	m_VMGM_PGCI_UT.offset_VMGM_LU = TransferINT32(offset+0x0c,save,m_VMGM_PGCI_UT.offset_VMGM_LU);

	offset = VMGM_PGCI_UT_off + m_VMGM_PGCI_UT.offset_VMGM_LU;
	int VMGM_LU_off = offset;
	int i=0;
	// -= to be reinserted =-
/*	while(offset<m_VMGM_PGCI_UT.end_adress+VMGM_PGCI_UT_off)
	{
		VMGM_LU lu;
		if(save) lu = m_VMGM_PGCI_UT.VMGM_LUs[i];
		i++;
		lu.nbMenus = TransferSHORT(offset,save,lu.nbMenus);
		lu.end_adress = TransferINT32(offset+0x04,save,lu.end_adress);
		offset += 8;
		int ix=0;
		for(int ix=0;ix<lu.nbMenus;ix++)
		{
			PGC_MENU cat;
			if(save) cat = lu.Menus[i];
			else ZeroMemory(&cat,sizeof(cat));
			TransferMEM(offset,4,(BYTE*)cat.category,save);
			cat.offset_PGCI = TransferINT32(offset+0x04,save,cat.offset_PGCI);
			offset+=8;
			if(!save) lu.Menus.push_back(cat);

		}

	//	offset = VMGM_LU_off + lu.offset_VMGM_PGC;
		int VMGM_PGC_off = offset;
		for(int ig=0;ig<lu.nbMenus;ig++)
		{
			VTS_PGC pgc;
			if(save) pgc = lu.PGCs[ig];
			offset = lu.Menus[ig].offset_PGCI+VMGM_LU_off;
			TransPGC(&pgc,offset,save);
			if(!save) lu.PGCs.push_back(pgc);
		}
		if(!save) m_VMGM_PGCI_UT.VMGM_LUs.push_back(lu);
	}*/
}

VMGM_PGCI_UT CIFOEdit::Get_VMGM_PGCI_UT()
{
	return m_VMGM_PGCI_UT;
}

void CIFOEdit::Set_VMGM_PGCI_UT(VMGM_PGCI_UT data, bool save)
{
	m_VMGM_PGCI_UT = data;
	if(save)
		save_VMGM_PGCI_UT();
}

void CIFOEdit::_VMG_PTL_MAIT(bool save)
{
	int offset = m_VMG_MAT.offset_VMG_PTL_MAIT*SECTOR_SIZE;
	if(offset==0) return;
	int VMG_PTL_MAIT_off = offset;
	m_VMG_PTL_MAIT.nbCountries = TransferSHORT(offset,save,m_VMG_PTL_MAIT.nbCountries);
	m_VMG_PTL_MAIT.nbTitleSets = TransferSHORT(offset+2,save,m_VMG_PTL_MAIT.nbTitleSets);
	m_VMG_PTL_MAIT.end_PTL_MAIT = TransferSHORT(offset+4,save,m_VMG_PTL_MAIT.end_PTL_MAIT);
	for(int n=0,offset=8; n<m_VMG_PTL_MAIT.nbCountries;n++,offset+=8)
	{
		VMG_PTL_MAIT_CONTRY_CODE code;
		if(save) code = m_VMG_PTL_MAIT.country_codes[n];
		else ZeroMemory(&code,sizeof(code));
		code.country_code = TransferSHORT(offset,save,code.country_code);
		code.offset_PTL_MAIT = TransferSHORT(offset+4,save,code.offset_PTL_MAIT);
		if(!save) m_VMG_PTL_MAIT.country_codes.push_back(code);
	}

	for(int i=0; i<m_VMG_PTL_MAIT.nbCountries;i++)
	{
		offset = VMG_PTL_MAIT_off + m_VMG_PTL_MAIT.country_codes[i].offset_PTL_MAIT;
		for(int n=1;n<=8;n++)
		{
			for(int m=0; m<m_VMG_PTL_MAIT.nbTitleSets+1;m++)
			{
				unsigned short s;
				if(save) s = m_VMG_PTL_MAIT.PTL_MAITs[n].par_masks[m];
				else s=0;
				TransferMEM(offset,2,(BYTE*)&s,save);
				offset += 2;
				if(!save) m_VMG_PTL_MAIT.PTL_MAITs[n].par_masks.push_back(s);
			}
		}
	}
}

VMG_PTL_MAIT CIFOEdit::Get_VMG_PTL_MAIT()
{
	return m_VMG_PTL_MAIT;
}

void CIFOEdit::Set_VMG_PTL_MAIT(VMG_PTL_MAIT data, bool save)
{
	m_VMG_PTL_MAIT = data;
	if(save)
		save_VMG_PTL_MAIT();
}

void CIFOEdit::_VMG_VTS_ATRT(bool save)
{
	int offset = m_VMG_MAT.offset_VMG_VTS_ATRT*SECTOR_SIZE;
	if(offset==0) return;
	int VMG_VTS_ATRT_off = offset;
	m_VMG_VTS_ATRT.nbTitleSets = TransferSHORT(offset,save,m_VMG_VTS_ATRT.nbTitleSets);
	m_VMG_VTS_ATRT.VTS_ATRT_end = TransferINT32(offset+4,save,m_VMG_VTS_ATRT.VTS_ATRT_end);
	for(int n=0,offset=VMG_VTS_ATRT_off+8;n<m_VMG_VTS_ATRT.nbTitleSets;n++,offset+=4)
	{
		unsigned int i=0;
		if(save) i =  m_VMG_VTS_ATRT.VTS_ATRT_offsets[n];
		i = TransferINT32(offset,save,i);
		if(!save) m_VMG_VTS_ATRT.VTS_ATRT_offsets.push_back(i);
	}
	for(int n=0;n<m_VMG_VTS_ATRT.nbTitleSets;n++)
	{
		VTS_ATRT atrt;
		offset = m_VMG_VTS_ATRT.VTS_ATRT_offsets[n]+VMG_VTS_ATRT_off;
		atrt.end_address =0;atrt.VTS_CAT=0;
		if(save) atrt = m_VMG_VTS_ATRT.VTS_ATRTs[n];

		atrt.end_address = TransferINT32(offset,save,atrt.end_address);
		atrt.VTS_CAT = TransferINT32(offset+4,save,atrt.VTS_CAT);

		// VTSM
		TransferMEM(offset+8,sizeof(m_VTSI_MAT.VTSM_video_attribute),
						(BYTE*)&m_VTSI_MAT.VTSM_video_attribute,save);

		m_VTSI_MAT.nb_VTSM_audio_attributes =
			TransferBYTE(offset+11,save,m_VTSI_MAT.nb_VTSM_audio_attributes);
		TransferMEM(offset+12,sizeof(m_VTSI_MAT.VTSM_audio_attributes),
							(BYTE*)&m_VTSI_MAT.VTSM_audio_attributes,save);
		m_VTSI_MAT.nb_VTSM_subt_attribute =
			TransferBYTE(offset+93,save,m_VTSI_MAT.nb_VTSM_subt_attribute);
		TransferMEM(offset+94,sizeof(m_VTSI_MAT.VTSM_subtitle_attribute),
							(BYTE*)&m_VTSI_MAT.VTSM_subtitle_attribute,save);

		// VTSTT
		TransferMEM(offset+264,sizeof(m_VTSI_MAT.VTSTT_video_attribute),
							(BYTE*)&m_VTSI_MAT.VTSTT_video_attribute,save);
		m_VTSI_MAT.nb_VTSTT_audio_attributes =
			TransferBYTE(offset+267,save,m_VTSI_MAT.nb_VTSTT_audio_attributes);
		TransferMEM(offset+268,sizeof(m_VTSI_MAT.VTSTT_audio_attributes),
							(BYTE*)&m_VTSI_MAT.VTSTT_audio_attributes,save);
		m_VTSI_MAT.nb_VTSTT_subt_attribute =
			TransferBYTE(offset+349,save,m_VTSI_MAT.nb_VTSTT_subt_attribute);
		TransferMEM(offset+350,sizeof(m_VTSI_MAT.VTSTT_subtitle_attributes),
							(BYTE*)&m_VTSI_MAT.VTSTT_subtitle_attributes,save);


		if(!save) m_VMG_VTS_ATRT.VTS_ATRTs.push_back(atrt);
	}



}

VMG_VTS_ATRT CIFOEdit::Get_VMG_VTS_ATRT()
{
	return m_VMG_VTS_ATRT;
}

void CIFOEdit::Set_VMG_VTS_ATRT(VMG_VTS_ATRT data, bool save)
{
	m_VMG_VTS_ATRT = data;
	if(save)
		save_VMG_VTS_ATRT();
}

void CIFOEdit::_VMGM_C_ADT(bool save)
{
	int offset = m_VMG_MAT.offset_VMGM_C_ADT*SECTOR_SIZE;
	if(offset==0)return;
	Trans_C_ADT((VTS_C_ADT*)&m_VMGM_C_ADT,offset,save);
}

VMGM_C_ADT CIFOEdit::Get_VMGM_C_ADT()
{
	return m_VMGM_C_ADT;
}

void CIFOEdit::Set_VMGM_C_ADT(VMGM_C_ADT data, bool save)
{
	m_VMGM_C_ADT = data;
	if(save)
		save_VMGM_C_ADT();
}

void CIFOEdit::_VMGM_VOBU_ADMAP(bool save)
{
	int offset = m_VMG_MAT.offset_VMGM_VOBU_ADMAP*SECTOR_SIZE;
	if(offset==0)return;
	Trans_VOBU_ADMAP((VTS_VOBU_ADMAP *)&m_VMGM_VOBU_ADMAP,offset,save);
}

VMGM_VOBU_ADMAP CIFOEdit::Get_VMGM_VOBU_ADMAP()
{
	return m_VMGM_VOBU_ADMAP;
}

void CIFOEdit::Set_VMGM_VOBU_ADMAP(VMGM_VOBU_ADMAP data, bool save)
{
	m_VMGM_VOBU_ADMAP = data;
	if(save)
		save_VMGM_VOBU_ADMAP();
}

