/*****************************************************************************
 * IfoEdit.h: Class to Edit DVD Ifo files
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#pragma once

#include "ifofile.h"
#include "ifoedit_def.h"
#include <vector>

class CIFOEdit : public CIFOFile
{
	/**********************************************************/
	/*------------------------ VMG Files ---------------------*/
	/**********************************************************/
	VMG_MAT_DATA m_VMG_MAT;
	VMG_TT_SRPT m_VMG_TT_SRPT;
	VMGM_PGCI_UT m_VMGM_PGCI_UT;
	VMG_PTL_MAIT m_VMG_PTL_MAIT;
	VMG_VTS_ATRT m_VMG_VTS_ATRT;
	VMGM_C_ADT m_VMGM_C_ADT;
	VMGM_VOBU_ADMAP m_VMGM_VOBU_ADMAP;

	/**********************************************************/
	/*------------------------ VTS Files ---------------------*/
	/**********************************************************/
    VTSI_MAT_DATA m_VTSI_MAT;
	VTS_PTT_SRPT m_VTS_PTT_SRPT;
	VTS_PGCITI m_VTS_PGCITI;
	VTSM_PGCI_UT m_VTSM_PGCI_UT;
	VTS_TMAPTI m_VTS_TMAPTI;
	VTS_C_ADT m_VTSM_C_ADT;
	VTS_VOBU_ADMAP m_VTSM_VOBU_ADMAP;
	VTS_C_ADT m_VTS_C_ADT;
	VTS_VOBU_ADMAP m_VTS_VOBU_ADMAP;
	/*--------------------------------------------------------*/

	char *m_strDest;
	bool m_isVTS;

	void checkVTS();

protected:
	void TransPGC(VTS_PGC *pgc, int &offset, bool save=0); // load/save a PGC table
	void Trans_C_ADT(VTS_C_ADT *adt, int &offset, bool save=0); //load/save a C_ADT
	void Trans_VOBU_ADMAP(VTS_VOBU_ADMAP *admap, int &offset, bool save=0);
	/**********************************************************/
	/*------------------------ VMG Files ---------------------*/
	/**********************************************************/
	// VMG_MAT_DATA
	void _VMG_MAT(bool save=0);
	void load_VMG_MAT() { // load the VMG_MAT_DATA
		_VMG_MAT(); };
	void save_VMG_MAT() { // save the VMG_MAT_DATA
		_VMG_MAT(1); };
	// VMG_PTT_SRPT
	void _VMG_TT_SRPT(bool save=0);
	void load_VMG_TT_SRPT() { // load the VMG_PTT_SRPT
		_VMG_TT_SRPT(); };
	void save_VMG_TT_SRPT() { // save the VMG_PTT_SRPT
		_VMG_TT_SRPT(1); };
	// VMGM_PGCI_UT
	void _VMGM_PGCI_UT(bool save=0);
	void load_VMGM_PGCI_UT() { // load the VMGM_PGCI_UT
		_VMGM_PGCI_UT(); };
	void save_VMGM_PGCI_UT() { // save the VMGM_PGCI_UT
		_VMGM_PGCI_UT(1); };
	// VMG_PTL_MAIT
	void _VMG_PTL_MAIT(bool save=0);
	void load_VMG_PTL_MAIT() { // load the VMG_PTL_MAIT
		_VMG_PTL_MAIT(); };
	void save_VMG_PTL_MAIT() { // save the VMG_PTL_MAIT
		_VMG_PTL_MAIT(1); };
	// VMG_VTS_ATRT
	void _VMG_VTS_ATRT(bool save=0);
	void load_VMG_VTS_ATRT() { // load the VMG_VTS_ATRT;
		_VMG_VTS_ATRT(); };
	void save_VMG_VTS_ATRT() { // save the VMG_VTS_ATRT
		_VMG_VTS_ATRT(1); };
	// VMGM_C_ADT
	void _VMGM_C_ADT(bool save=0);
	void load_VMGM_C_ADT() { // load the VMGM_C_ADT
		_VMGM_C_ADT(); };
	void save_VMGM_C_ADT() { // save the VMGM_C_ADT
		_VMGM_C_ADT(1); };
	// VMGM_VOBU_ADMAP
	void _VMGM_VOBU_ADMAP(bool save=0);
	void load_VMGM_VOBU_ADMAP() { // load tha VMGM_VOBU_ADMAP
		_VMGM_VOBU_ADMAP(); };
	void save_VMGM_VOBU_ADMAP() { // save the VMGM_VOBU_ADMAP
		_VMGM_VOBU_ADMAP(); };

	/**********************************************************/
	/*------------------------ VTS Files ---------------------*/
	/**********************************************************/

	// VTSI_MAT
	void _VTSI_MAT(bool save=0);
	void load_VTSI_MAT() { // load the VTSI_MAT
		_VTSI_MAT(); };
	void save_VTSI_MAT() { // save the VTSI_MAT
		_VTSI_MAT(1); };
	// VTS_PTT_SRPT
	void _VTS_PTT_SRPT(bool save=0);
	void load_VTS_PTT_SRPT() { // load the VTS_PTT_SRPT
		_VTS_PTT_SRPT(); };
	void save_VTS_PTT_SRPT() { // save the VTS_PTT_SRPT
		_VTS_PTT_SRPT(1); };
	// VTS_PGCITI
	void _VTS_PGCITI(bool save=0);
	void load_VTS_PGCITI() { // load the VTS_PGCITI
		_VTS_PGCITI(); };
	void save_VTS_PGCITI() { // save the VTS_PGCITI
		_VTS_PGCITI(1); };
	// VTSM_PGCI_UT
	void _VTSM_PGCI_UT(bool save=0);
	void load_VTSM_PGCI_UT() { // load the VTSM_PGCI_UT
		_VTSM_PGCI_UT(); };
	void save_VTSM_PGCI_UT() { // save the VTSM_PGCI_UT
		_VTSM_PGCI_UT(1); };
	// VTS_TMAPTI
	void _VTS_TMAPTI(bool save=0);
	void load_VTS_TMAPTI() { // load the VTS_TMAPTI
		_VTS_TMAPTI(); };
	void save_VTS_TMAPTI() { // save the VTS_TMAPTI
		_VTS_TMAPTI(1); };
	// VTSM_C_ADT
	void _VTSM_C_ADT(bool save=0);
	void load_VTSM_C_ADT() { // load the VTS_C_ADT
		_VTSM_C_ADT(); };
	void save_VTSM_C_ADT() { // save the VTS_C_ADT
		_VTSM_C_ADT(1); };
	// VTSM_VOBU_ADMAP
	void _VTSM_VOBU_ADMAP(bool save=0);
	void load_VTSM_VOBU_ADMAP() { // load the VTS_VOBU_ADMAP
		_VTSM_VOBU_ADMAP(); };
	void save_VTSM_VOBU_ADMAP() { // save the VTS_VOBU_ADMAP
		_VTSM_VOBU_ADMAP(1); };
	// VTS_C_ADT
	void _VTS_C_ADT(bool save=0);
	void load_VTS_C_ADT() { // load the VTS_C_ADT
		_VTS_C_ADT(); };
	void save_VTS_C_ADT() { // save the VTS_C_ADT
		_VTS_C_ADT(1); };
	// VTS_VOBU_ADMAP
	void _VTS_VOBU_ADMAP(bool save=0);
	void load_VTS_VOBU_ADMAP() { // load the VTS_VOBU_ADMAP
		_VTS_VOBU_ADMAP(); };
	void save_VTS_VOBU_ADMAP() { // save the VTS_VOBU_ADMAP
		_VTS_VOBU_ADMAP(1); };

	/*--------------------------------------------------------*/


public:
	/**********************************************************/
	/*------------------------- Globals ----------------------*/
	/**********************************************************/
	CIFOEdit(void);
	CIFOEdit(const char *srcFile);
	~CIFOEdit(void);
	bool Open(const char *srcFile);
	void Close();
	void Save(const char *destFile);
	bool isVTS(){return m_isVTS; };

	/**********************************************************/
	/*------------------------ VMG Files ---------------------*/
	/**********************************************************/
	VMG_MAT_DATA Get_VMG_MAT();
	void Set_VMG_MAT(VMG_MAT_DATA data, bool save=1);

	VMG_TT_SRPT Get_VMG_TT_SRPT();
	void Set_VMG_TT_SRPT(VMG_TT_SRPT data, bool save=1);

	VMGM_PGCI_UT Get_VMGM_PGCI_UT();
	void Set_VMGM_PGCI_UT(VMGM_PGCI_UT data, bool save);

	VMG_PTL_MAIT Get_VMG_PTL_MAIT();
	void Set_VMG_PTL_MAIT(VMG_PTL_MAIT data, bool save);

	VMG_VTS_ATRT Get_VMG_VTS_ATRT();
	void Set_VMG_VTS_ATRT(VMG_VTS_ATRT data, bool save);

	VMGM_C_ADT Get_VMGM_C_ADT();
	void Set_VMGM_C_ADT(VMGM_C_ADT data, bool save);

	VMGM_VOBU_ADMAP Get_VMGM_VOBU_ADMAP();
	void Set_VMGM_VOBU_ADMAP(VMGM_VOBU_ADMAP data, bool save);

	/**********************************************************/
	/*------------------------ VTS Files ---------------------*/
	/**********************************************************/
	VTSI_MAT_DATA Get_VTSI_MAT();
	void Set_VTSI_MAT(VTSI_MAT_DATA data, bool save=1);

	VTS_PTT_SRPT Get_VTS_PTT_SRPT();
	void Set_VTS_PTT_SRPT(VTS_PTT_SRPT data, bool save=1);

	VTS_PGCITI Get_VTS_PGCITI();
	void Set_VTS_PGCITI(VTS_PGCITI data, bool save=1);

	VTSM_PGCI_UT Get_VTSM_PGCI_UT();
	void Set_VTSM_PGCI_UT(VTSM_PGCI_UT data, bool save=1);

	VTS_TMAPTI Get_VTS_TMAPTI();
	void Set_VTS_TMAPTI(VTS_TMAPTI data, bool save=1);

	VTS_C_ADT Get_VTSM_C_ADT();
	void Set_VTSM_C_ADT(VTS_C_ADT data, bool save=1);

	VTS_VOBU_ADMAP Get_VTSM_VOBU_ADMAP();
	void Set_VTSM_VOBU_ADMAP(VTS_VOBU_ADMAP data, bool save=1);

	VTS_C_ADT Get_VTS_C_ADT();
	void Set_VTS_C_ADT(VTS_C_ADT data, bool save=1);

	VTS_VOBU_ADMAP Get_VTS_VOBU_ADMAP();
	void Set_VTS_VOBU_ADMAP(VTS_VOBU_ADMAP data, bool save=1);
	/**********************************************************/
	/*--------------------------------------------------------*/
	/**********************************************************/
};

