/*****************************************************************************
 * IOLayer.h: Custom IOLayer Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#pragma once

typedef struct tagIO_BUFFER
{
	int maxCurrSector;
	unsigned char *pBuffer;
} IO_BUFFER;

// IO LAYER
// ********
//  + lowest layer
//  + does buffered asynchronous disk reading and writing 
//  + smallest buffer unit is a sector (usually 2048 bytes)
class IOLayer
{
public:
	IOLayer(void);
	~IOLayer(void);

	virtual int Open(unsigned char *file, int rwFlag);
	virtual void Close();

	virtual int Read(void *buffer);
	virtual int Read(void *buffer, __int64 offset);
	virtual int Write(void *buffer);

	virtual void SetBufferSize(int numSectors);
	virtual void SetSectorSize(int numBytes);

	__int64 kBThroughput;

	// constants
	int SECTOR_SIZE;
	int MAX_SECTORS;
	int MAX_BUFFERS;
	int SLEEP_TIME_MS;

	IO_BUFFER **ioElement;
	CRITICAL_SECTION csection;
	int rwFlag;
	unsigned int threadHandle;
	int stopFlag;

	// file params
	__int64 FileSize, CurrPos, currOffset;
	HANDLE hFile;

	// internal functions
	virtual int internalRead(IO_BUFFER *ioElement);
	virtual int internalWrite(IO_BUFFER *ioElement);

protected:
	// occupancy
	int currSector;
	int opened;
};
