/*****************************************************************************
 * MacroBlock.cpp: Read and Write Macroblock Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "StdAfx.h"
#include "macroblock.h"

MacroBlock::MacroBlock(void)
{
	block[0 ].SetType(LUMA);
	block[1 ].SetType(LUMA);
	block[2 ].SetType(LUMA);
	block[3 ].SetType(LUMA);

	block[4 ].SetType(CHROMA);
	block[5 ].SetType(CHROMA);
	block[6 ].SetType(CHROMA);
	block[7 ].SetType(CHROMA);
	block[8 ].SetType(CHROMA);
	block[9 ].SetType(CHROMA);
	block[10].SetType(CHROMA);
	block[11].SetType(CHROMA);
}

MacroBlock::~MacroBlock(void)
{
}

VLC MacroBlock::bstream;
VLC MacroBlock::tBitStream;

// I-Frame Macroblock-Type Table - ISO 13818-2 Table B-2 (settings part)
const MBTYPE MacroBlock::IFrame[2] =
{ { 0, 0, 0, 0, 1 },
  { 1, 0, 0, 0, 1 } };

// I-Frame Macroblock-Type Table - ISO 13818-2 Table B-2 (VLC part)
const VLC_ENTRY MacroBlock::IFrameVLC[2] =
{ { 0, 1, 1 },
  { 1, 1, 1 } };

// I-Frame Macroblock-Type Table Indices - ISO 13818-2 Table B-2
const char MacroBlock::IFrameIDX[2] = { 0, 1 };

// P-Frame Macroblock-Type Table - ISO 13818-2 Table B-3 (settings part)
const MBTYPE MacroBlock::PFrame[7] =
{ { 0, 1, 0, 1, 0 },
  { 0, 0, 0, 1, 0 },
  { 0, 1, 0, 0, 0 },
  { 0, 0, 0, 0, 1 },
  { 1, 1, 0, 1, 0 },
  { 1, 0, 0, 1, 0 },
  { 1, 0, 0, 0, 1 } };

// P-Frame Macroblock-Type Table - ISO 13818-2 Table B-3 (VLC part)
const VLC_ENTRY MacroBlock::PFrameVLC[7] =
{ { 0, 1, 1 },
  { 1, 1, 1 },
  { 2, 1, 1 },
  { 3, 2, 3 },
  { 3, 2, 2 },
  { 4, 1, 1 },
  { 5, 1, 1 } };

// P-Frame Macroblock-Type Table Indices - ISO 13818-2 Table B-3
const char MacroBlock::PFrameIDX[6] = { 0, 1, 2, 3, 5, 6 };

// B-Frame Macroblock-Type Table - ISO 13818-2 Table B-4 (settings part)
const MBTYPE MacroBlock::BFrame[11] =
{ { 0, 1, 1, 0, 0 },
  { 0, 1, 1, 1, 0 },
  { 0, 0, 1, 0, 0 },
  { 0, 0, 1, 1, 0 },
  { 0, 1, 0, 0, 0 },
  { 0, 1, 0, 1, 0 },
  { 0, 0, 0, 0, 1 },
  { 1, 1, 1, 1, 0 },
  { 1, 1, 0, 1, 0 },
  { 1, 0, 1, 1, 0 },
  { 1, 0, 0, 0, 1 } };

// B-Frame Macroblock-Type Table - ISO 13818-2 Table B-4 (VLC part)
const VLC_ENTRY MacroBlock::BFrameVLC[11] =
{ { 0, 2, 2 },
  { 0, 2, 3 },
  { 1, 2, 2 },
  { 1, 2, 3 },
  { 2, 2, 2 },
  { 2, 2, 3 },
  { 3, 2, 3 },
  { 3, 2, 2 },
  { 4, 2, 3 },
  { 4, 2, 2 },
  { 5, 1, 1 } };

// B-Frame Macroblock-Type Table Indices - ISO 13818-2 Table B-4
const char MacroBlock::BFrameIDX[6] = { 0, 2, 4, 6, 8, 10 };

// Macroblock Addressing Table - ISO 13818-2 Table B-1 (VLC part)
const VLC_ENTRY MacroBlock::MBAddressVLC[34] =
{ { 0, 1, 1  },
  { 1, 2, 3  },
  { 1, 2, 2  },
  { 2, 2, 3  },
  { 2, 2, 2  },
  { 3, 2, 3  },
  { 3, 2, 2  },
  { 4, 3, 7  },
  { 4, 3, 6  },
  { 4, 4, 11 },
  { 4, 4, 10 },
  { 4, 4, 9  },
  { 4, 4, 8  },
  { 5, 3, 7  },
  { 5, 3, 6  },
  { 5, 5, 23 },
  { 5, 5, 22 },
  { 5, 5, 21 },
  { 5, 5, 20 },
  { 5, 5, 19 },
  { 5, 5, 18 },
  { 5, 6, 35 },
  { 5, 6, 34 },
  { 5, 6, 33 },
  { 5, 6, 32 },
  { 6, 5, 31 },
  { 6, 5, 30 },
  { 6, 5, 29 },
  { 6, 5, 28 },
  { 6, 5, 27 },
  { 6, 5, 26 },
  { 6, 5, 25 },
  { 6, 5, 24 },
  { 7, 4, 8  } };

// Macroblock Addressing Table Indices - ISO 13818-2 Table B-1
const char MacroBlock::MBAddressIDX[8] = { 0, 1, 3, 5, 7, 13, 25, 33 };

// Macroblock Pattern Table - ISO 13818-2 Table B-9 (VLC part)
const VLC_ENTRY MacroBlock::MBPatternVLC[64] =
{ { 0, 3, 7  },
  { 0, 4, 13 },
  { 0, 4, 12 },
  { 0, 4, 11 },
  { 0, 4, 10 },
  { 0, 5, 19 },
  { 0, 5, 18 },
  { 0, 5, 17 },
  { 0, 5, 16 },
  { 1, 4, 15 },
  { 1, 4, 14 },
  { 1, 4, 13 },
  { 1, 4, 12 },
  { 1, 4, 11 },
  { 1, 4, 10 },
  { 1, 4, 9  },
  { 1, 4, 8  },
  { 2, 4, 15 },
  { 2, 4, 14 },
  { 2, 4, 13 },
  { 2, 4, 12 },
  { 2, 5, 23 },
  { 2, 5, 22 },
  { 2, 5, 21 },
  { 2, 5, 20 },
  { 2, 5, 19 },
  { 2, 5, 18 },
  { 2, 5, 17 },
  { 2, 5, 16 },
  { 3, 5, 31 },
  { 3, 5, 30 },
  { 3, 5, 29 },
  { 3, 5, 28 },
  { 3, 5, 27 },
  { 3, 5, 26 },
  { 3, 5, 25 },
  { 3, 5, 24 },
  { 3, 5, 23 },
  { 3, 5, 22 },
  { 3, 5, 21 },
  { 3, 5, 20 },
  { 3, 5, 19 },
  { 3, 5, 18 },
  { 3, 5, 17 },
  { 3, 5, 16 },
  { 4, 4, 15 },
  { 4, 4, 14 },
  { 4, 4, 13 },
  { 4, 4, 12 },
  { 4, 4, 11 },
  { 4, 4, 10 },
  { 4, 4, 9  },
  { 4, 4, 8  },
  { 5, 3, 7  },
  { 5, 3, 6  },
  { 5, 3, 5  },
  { 5, 3, 4  },
  { 6, 3, 7  },
  { 6, 3, 6  },
  { 6, 3, 5  },
  { 6, 3, 4  },
  { 7, 2, 3  },
  { 7, 2, 2  },
  { 8, 1, 1  } };

// Macroblock Pattern Table Indices - ISO 13818-2 Table B-9
const char MacroBlock::MBPatternIDX[9] = { 0, 9, 17, 29, 45, 53, 57, 61, 63 };

// Macroblock Pattern Table - ISO 13818-2 Table B-9 (settings part)
const char MacroBlock::MBPattern[64] =
{
	60,
	4,
	8,
	16,
	32,
	12,
	48,
	20,
	40,
	28,
	44,
	52,
	56,
	1,
	61,
	2,
	62,
	24,
	36,
	3,
	63,
	5,
	9,
	17,
	33,
	6,
	10,
	18,
	34,
	7,
	11,
	19,
	35,
	13,
	49,
	21,
	41,
	14,
	50,
	22,
	42,
	15,
	51,
	23,
	43,
	25,
	37,
	26,
	38,
	29,
	45,
	53,
	57,
	30,
	46,
	54,
	58,
	31,
	47,
	55,
	59,
	27,
	39,
	0
};

// Inverse Macroblock Pattern Table - Inverse ISO 13818-2 Table B-9 (settings part)
const char MacroBlock::MBPatternINV[64] =
{
	63,
	13,
	15,
	19,
	1,
	21,
	25,
	29,
	2,
	22,
	26,
	30,
	5,
	33,
	37,
	41,
	3,
	23,
	27,
	31,
	7,
	35,
	39,
	43,
	17,
	45,
	47,
	61,
	9,
	49,
	53,
	57,
	4,
	24,
	28,
	32,
	18,
	46,
	48,
	62,
	8,
	36,
	40,
	44,
	10,
	50,
	54,
	58,
	6,
	34,
	38,
	42,
	11,
	51,
	55,
	59,
	12,
	52,
	56,
	60,
	0,
	14,
	16,
	20
};

// Motion Vector Code Table - ISO 13818-2 Table B-10 (VLC part)
const VLC_ENTRY MacroBlock::MVCodeVLC[17] = 
{ { 0, 1, 1  },
  { 1, 1, 1  },
  { 2, 1, 1  },
  { 3, 1, 1  },
  { 4, 2, 3  },
  { 4, 3, 5  },
  { 4, 3, 4  },
  { 5, 2, 3  },
  { 5, 4, 11 },
  { 5, 4, 10 },
  { 5, 4, 9  },
  { 5, 5, 17 },
  { 5, 5, 16 },
  { 6, 4, 15 },
  { 6, 4, 14 },
  { 6, 4, 13 },
  { 6, 4, 12 } };

// Motion Vector Code Table Indices - ISO 13818-2 Table B-10
const char MacroBlock::MVCodeIDX[7] = { 0, 1, 2, 3, 4, 7, 13 };


int MacroBlock::AttachBuffer(unsigned char *buffer, int length, int bit_startpos)
{

	// init parameters
	pBuffer = buffer;
	bufferSize = length;

	// attach a bitstream to the buffer
	bstream.AttachBuffer(pBuffer, bufferSize + (bit_startpos + 7) / 8);
	bstream.Seek(bit_startpos);

	int val, bitSize = 0;

	preQScaleStore[0] = 0;
	preQScaleStore[1] = 0;
	preQScaleStore[2] = 0;
	preQScaleStore[3] = 0;

	postQScaleStore[0] = 0;
	postQScaleStore[1] = 0;
	postQScaleStore[2] = 0;
	postQScaleStore[3] = 0;

	// attach a bistream to the storage
	tBitStream.AttachBuffer(preQScaleStore, 16);
	preQScaleStoreBitSize = 0;
	
	// read macroblock address
	MBAddressInc = 0;
	MBAddressIncSize = 0;

	bstream.table = (VLC_ENTRY *) MBAddressVLC;
	bstream.indices = (unsigned char *) MBAddressIDX;
	do
	{
		val = bstream.ReadVLC();
		MBAddressIncSize += MBAddressVLC[val].digits + MBAddressVLC[val].zeroes;
		if(val == 33) MBAddressInc += val;
		else	  	  MBAddressInc += val + 1;
	} while(val == 33);
	absNumMacroBlock += MBAddressInc;

	// read macroblock type
	switch(pictureType)
	{
		case I_PICTURE:
			tBitStream.table = (VLC_ENTRY *) IFrameVLC;
			tBitStream.indices = (unsigned char *) IFrameIDX;
			bstream.table = (VLC_ENTRY *) IFrameVLC;
			bstream.indices = (unsigned char *) IFrameIDX;
			MBType = (MBTYPE *) IFrame;
			break;

		case P_PICTURE:
			tBitStream.table = (VLC_ENTRY *) PFrameVLC;
			tBitStream.indices = (unsigned char *) PFrameIDX;
			bstream.table = (VLC_ENTRY *) PFrameVLC;
			bstream.indices = (unsigned char *) PFrameIDX;
			MBType = (MBTYPE *) PFrame;
			break;

		case B_PICTURE:
			tBitStream.table = (VLC_ENTRY *) BFrameVLC;
			tBitStream.indices = (unsigned char *) BFrameIDX;
			bstream.table = (VLC_ENTRY *) BFrameVLC;
			bstream.indices = (unsigned char *) BFrameIDX;
			MBType = (MBTYPE *) BFrame;
			break;
	}

	MBTypeIdx = bstream.ReadVLC();
	MBTypeSize = bstream.table[MBTypeIdx].digits + bstream.table[MBTypeIdx].zeroes;
	MBType += MBTypeIdx;

	// do some debugging
	_ASSERTX(MBTypeIdx >= 0);
#ifdef _DEBUG
	switch(pictureType)
	{
		case I_PICTURE: _ASSERTX(MBTypeIdx < 2);  break;
		case P_PICTURE: _ASSERTX(MBTypeIdx < 7);  break;
		case B_PICTURE: _ASSERTX(MBTypeIdx < 11); break;
	}
#endif

	// parameters for motion vectors
	int dmv, motion_vector_count, mv_format;

	// detect parameters for motion vectors ***************
	if(MBType->macroblock_motion_forward || MBType->macroblock_motion_backward)
	{
		if(picture_structure == FRAME_PICTURE)
		{
			if(frame_pred_frame_dct == 0) 
			{
				int frame_motion_type = bstream.ReadBits(2);
				tBitStream.WriteBits(frame_motion_type, 2);
				preQScaleStoreBitSize += 2;
				
				switch(frame_motion_type)
				{
					case 1: motion_vector_count = 2; mv_format = FIELD_BASED; dmv = 0; break;
					case 2: motion_vector_count = 1; mv_format = FRAME_BASED; dmv = 0; break;
					case 3: motion_vector_count = 1; mv_format = FIELD_BASED; dmv = 1; break;
					// no other option accepted
					default: _ASSERTX(FALSE); break;
				}
			}
			else
			{
						    motion_vector_count = 1; mv_format = FRAME_BASED; dmv = 0;
			}
		}
		else
		{
			int field_motion_type = bstream.ReadBits(2);
			tBitStream.WriteBits(field_motion_type, 2);
			preQScaleStoreBitSize += 2;

			switch(field_motion_type)
			{
				case 1: motion_vector_count = 1; mv_format = FIELD_BASED; dmv = 0; break;
				case 2: motion_vector_count = 2; mv_format = FIELD_BASED; dmv = 0; break;
				case 3: motion_vector_count = 1; mv_format = FIELD_BASED; dmv = 1; break;
				// no other option accepted
				default: _ASSERTX(FALSE); break;
			}
		}
	}
	// concealment motion vectors have standard parameters not coded in the stream ***************
	if(MBType->macroblock_intra && concealment_motion_vectors)
	{
		if(picture_structure == FRAME_PICTURE) { motion_vector_count = 1; mv_format = FRAME_BASED; dmv = 0; }
		else								   { motion_vector_count = 1; mv_format = FIELD_BASED; dmv = 0; }
	}

	dct_typeSize = 0;
	// simple copy stream ***************
	if((picture_structure == FRAME_PICTURE) && (frame_pred_frame_dct == 0) && (MBType->macroblock_intra || MBType->macroblock_pattern))
	{
		dct_type = bstream.ReadBits(1);
		dct_typeSize = 1;
	}

	// read quantizer scale
	if(MBType->macroblock_quant)
	{
		quantizerScaleCode = bstream.ReadBits(5);
		bitSize += 5;
	}
	
	// attach a bistream to the next storage
	tBitStream.AttachBuffer(postQScaleStore, 16);
	postQScaleStoreBitSize = 0;

	// simple copy stream ***************
	if(MBType->macroblock_motion_forward || (MBType->macroblock_intra && concealment_motion_vectors))
	{
		postQScaleStoreBitSize += ReadMotionData(motion_vector_count, mv_format, dmv, MV_FORWARD);
	}

	// simple copy stream ***************
	if(MBType->macroblock_motion_backward)
	{
		postQScaleStoreBitSize += ReadMotionData(motion_vector_count, mv_format, dmv, MV_BACKWARD);
	}

	// simple copy stream ***************
	if(MBType->macroblock_intra && concealment_motion_vectors)
	{
		tBitStream.WriteBit(bstream.ReadBit());
		postQScaleStoreBitSize += 1;
	}

	int coded_block_pattern;
	int i;

	int maxblock = -1;
	if(chroma_format == CHROMA_420) maxblock = 6;
	if(chroma_format == CHROMA_422) maxblock = 8;
	if(chroma_format == CHROMA_444) maxblock = 12;

	// initialise block pattern
	for(i = 0; i < 12; i++)
	{
		if(MBType->macroblock_intra && i < maxblock)
		{
			pattern_code[i] = 1;
		}
		else
		{
			pattern_code[i] = 0;
		}
	}

	// read coded block pattern
	if(MBType->macroblock_pattern)
	{
		bitSize += ReadCodedBlockPattern();	
	}

	bitSize += MBAddressIncSize + MBTypeSize + preQScaleStoreBitSize + dct_typeSize + postQScaleStoreBitSize;

	bl_numCoef = 0;
	absNumBlock = 0;
	// read block data
    for(i = 0; i < 12; i++)
	{
		if(pattern_code[i])
		{
			block[i].intra = MBType->macroblock_intra;
			block[i].intraVLCFormat = intraVLCFormat;
			block[i].quantizerScaleCode = quantizerScaleCode;
			block[i].qScaleType = qScaleType;
			block[i].absNumPicture = absNumPicture;
			block[i].absNumMacroBlock = absNumMacroBlock;
			block[i].absNumBlock = absNumBlock;
			block[i].demoMode = demoMode;
			
			block[i].bl_coefBuffer = &bl_coefBuffer[bl_numCoef];
			block[i].bl_runBuffer = &bl_runBuffer[bl_numCoef];

			bitSize += block[i].AttachBuffer(pBuffer, bufferSize - bitSize / 8, bitSize + bit_startpos);
			absNumBlock++;
			bl_numCoef += block[i].blockEOF;
		}
	}
	
	// collect DC coefficients from I pictures for visualization purposes
	int u, v;
	if(chroma_format == CHROMA_420) { u = 5; v = 6;  }
	if(chroma_format == CHROMA_422) { u = 6; v = 8;  }
	if(chroma_format == CHROMA_444) { u = 8; v = 12; }

	if(pictureType == I_PICTURE)
	{
	    for(i = 0; i < 12; i++)
		{
			if(pattern_code[i])
			{
					 if(i < 4) { DCCoef[i] = block[i].GetDCCoef(YUVpred[0]); YUVpred[0] = DCCoef[i]; }
				else if(i < u) { DCCoef[i] = block[i].GetDCCoef(YUVpred[1]); YUVpred[1] = DCCoef[i]; }
				else if(i < v) { DCCoef[i] = block[i].GetDCCoef(YUVpred[2]); YUVpred[2] = DCCoef[i]; }
			}
		}
	}

	orgQuantizerScaleCode = quantizerScaleCode;
	testBitSize = bitSize;

	return bitSize;
}

// sets the YUV predictors
void MacroBlock::SetDCYUVpred(int *YUVpred)
{
	this->YUVpred[0] = YUVpred[0];
	this->YUVpred[1] = YUVpred[1];
	this->YUVpred[2] = YUVpred[2];
}

// gets the YUV predictors
void MacroBlock::GetDCYUVpred(int *YUVpred)
{
	YUVpred[0] = this->YUVpred[0];
	YUVpred[1] = this->YUVpred[1];
	YUVpred[2] = this->YUVpred[2];
}

// gets predicted values
void MacroBlock::GetDCCoef(int *DCCoef)
{
	CopyMemory(DCCoef, this->DCCoef, 12 * sizeof(int));
}

int MacroBlock::WriteDataBuffer(unsigned char *buffer, int bit_startpos, int irreductible)
{

	int i;
	int doSkip = 0;
/*	if(!irreductible /*&& !MBType->macroblock_quant* && pictureType == B_PICTURE)
	{
		MBTYPE *tMBType = MBType;
		int valid = ValidateMacroBlock(irreductible);
		if(valid != 0)
		{
			doSkip = 1;
			for(i = 0; i < 12; i++) pattern_code[i] = 0;
		}
		else
		{
			MBType = tMBType;
		}
	}*/

	// attach a bitstream to the buffer (deactivate limitation as we do not use the same buffer as for reading)
	bstream.AttachBuffer(buffer, 0x7FFFFFFF);
	bstream.Seek(bit_startpos);

	// write macroblock address increments
	bstream.table = (VLC_ENTRY *) MBAddressVLC;
	bstream.indices = (unsigned char *) MBAddressIDX;
	MBAddressIncSize = 0;
	int val = MBAddressInc;
	do
	{
		if(val > 33) { MBAddressIncSize += bstream.WriteVLC(33);      val -= 33; }
		else		 { MBAddressIncSize += bstream.WriteVLC(val - 1); val = 0;   }
	} while(val > 0);

	// write macroblock type
	int length = MBAddressIncSize + preQScaleStoreBitSize + postQScaleStoreBitSize;

	// first store blocks in temporary location
	int alignStartPos = bit_startpos + length + 64;
	int block_length = 0;
	int sumBlockSize = 0;

	// repeat until something remains for irreductible blocks
	int valid = 0;
	// if we can skip the content of the blocks
	if(doSkip == 0)
	{
		int t_quantizerScaleCode = quantizerScaleCode; // save quantizer scale code (may only be changed temporarily)
		int minimal_mode = 0;	// the minimal mode flag is used when the quantizer is artificially lowered in order to
			                    // preserve blocks. if minimal mode is on the smallest possible block is formed
								// WARNING: there may still be the case that a single block overflows, so some blocks may need
								//          an own minimal mode
		do
		{
			block_length = 0;
			sumBlockSize = 0;

			for(i = 0; i < 12; i++) test_pattern_code[i] = pattern_code[i];

			// write every block
			for(i = 0; i < 12; i++)
			{
				if(test_pattern_code[i])
				{
					int bitSize = 0;
					// if either minimal mode is off or minimal mode is on but block is still empty
					if(minimal_mode == 0 || block_length == 0)
					{
						block[i].quantizerScaleCode = quantizerScaleCode;
						bitSize = block[i].WriteDataBuffer(buffer, alignStartPos + block_length);
						sumBlockSize += bitSize;
						block_length += bitSize;
					}
					// change pattern code if block vanishes
					if(bitSize == 0) test_pattern_code[i] = 0;
				}
			}
			if(block_length == 0) valid = ValidateMacroBlock(irreductible);
			if(block_length == 0 && irreductible && valid == 0) 
			{
				_ASSERT(quantizerScaleCode > orgQuantizerScaleCode); 
				quantizerScaleCode = orgQuantizerScaleCode;//max(0, quantizerScaleCode - 1);
				SetCodedQuantizer();
			}
			minimal_mode = 1; // next time we do the loop get into minimal mode
		} while(block_length == 0 && irreductible && valid == 0);

		// restore quantizer scale code
		quantizerScaleCode = t_quantizerScaleCode;

		for(i = 0; i < 12; i++) pattern_code[i] = test_pattern_code[i];

		length += block_length;
	}
	// if not, we are sure the block is valid
	else
	{
		valid = 1;
	}

//	int valid = 0;
//	if(block_length == 0) valid = UnSetCodedQuantizerVal();

	switch(pictureType)
	{
		case I_PICTURE: bstream.table = (VLC_ENTRY *) IFrameVLC; bstream.indices = (unsigned char *) IFrameIDX;	break;
		case P_PICTURE:	bstream.table = (VLC_ENTRY *) PFrameVLC; bstream.indices = (unsigned char *) PFrameIDX;	break;
		case B_PICTURE:	bstream.table = (VLC_ENTRY *) BFrameVLC; bstream.indices = (unsigned char *) BFrameIDX;	break;
	}

	length += MBTypeSize = bstream.WriteVLC(MBTypeIdx);

	// write first store
	tBitStream.AttachBuffer(preQScaleStore, 16);
	for(i = 0; i < preQScaleStoreBitSize / 32; i++)
	{
		bstream.WriteBits(tBitStream.ReadBits(32), 32);
	}
	bstream.WriteBits(tBitStream.ReadBits(preQScaleStoreBitSize % 32), preQScaleStoreBitSize % 32);

	// write dct_type
	dct_typeSize = 0;
	if((picture_structure == FRAME_PICTURE) && (frame_pred_frame_dct == 0) && (MBType->macroblock_intra || MBType->macroblock_pattern))
	{
		bstream.WriteBits(dct_type, 1);
		dct_typeSize = 1;
		length++;
	}

	// write quantizer scale
	if(MBType->macroblock_quant)
	{
		bstream.WriteBits(quantizerScaleCode, 5);
		length += 5;
	}

	// write second store
	tBitStream.AttachBuffer(postQScaleStore, 16);
	for(i = 0; i < postQScaleStoreBitSize / 32; i++)
	{
		bstream.WriteBits(tBitStream.ReadBits(32), 32);
	}
	bstream.WriteBits(tBitStream.ReadBits(postQScaleStoreBitSize % 32), postQScaleStoreBitSize % 32);

	// write coded block pattern
	if(MBType->macroblock_pattern)
	{
		length += WriteCodedBlockPattern();	
	}

	// move blocks to final location in stream
	tBitStream.AttachBuffer(buffer, 0x7FFFFFFF);//bufferSize + (alignStartPos + 7) / 8);
	tBitStream.Seek(alignStartPos);
	for(i = 0; i < block_length / 32; i++)
	{
		bstream.WriteBits(tBitStream.ReadBits(32), 32);
	}	
	bstream.WriteBits(tBitStream.ReadBits(block_length % 32), block_length % 32);

// debug
//	_ASSERTX(length == testBitSize);
//	_ASSERTX(length <= testBitSize);

/*#ifdef _DEBUG
	BitStream srcBStream;
			  srcBStream.AttachBuffer(pBuffer, 2000000);
			  srcBStream.Seek(bit_startpos);
	BitStream destBStream;
			  destBStream.AttachBuffer(buffer, 2000000);
			  destBStream.Seek(bit_startpos);

    int val;
	for(int i = 0; i < length / 16; i++)
	{
		val = srcBStream.ReadBits(16);
		_ASSERTX(destBStream.ReadBits(16) == val);
	}

	val = srcBStream.ReadBits(length % 16);
	_ASSERTX(destBStream.ReadBits(length % 16) == val);

#endif*/

	// if this macroblock carries not relevant data
	if(sumBlockSize == 0) return length + MACROBLOCK_EMPTY + (valid == 0 ? MACROBLOCK_INVALID : 0);
	else				  return length;
//	return bufferSize;
//	return 0;
}

int MacroBlock::ReadMotionData(int motion_vector_count, int mv_format, int dmv, int mv_dir)
{
	int bitSize = 0;

	// set motion vector VLC table
	bstream.table = (VLC_ENTRY *) MVCodeVLC;
	bstream.indices = (unsigned char *) MVCodeIDX;
	tBitStream.table = (VLC_ENTRY *) MVCodeVLC;
	tBitStream.indices = (unsigned char *) MVCodeIDX;

	if(motion_vector_count == 1)
	{
		if((mv_format == FIELD_BASED) && (dmv != 1))
		{
			tBitStream.WriteBits(bstream.ReadBits(1), 1);
			bitSize += 1;
		}
		bitSize += ReadMotionVector(dmv, mv_dir, 0);
		bitSize += ReadMotionVector(dmv, mv_dir, 1);
	}
	else
	{
		tBitStream.WriteBits(bstream.ReadBits(1), 1);
		bitSize += 1;
		bitSize += ReadMotionVector(dmv, mv_dir, 0);
		bitSize += ReadMotionVector(dmv, mv_dir, 1);

		tBitStream.WriteBits(bstream.ReadBits(1), 1);
		bitSize += 1;
		bitSize += ReadMotionVector(dmv, mv_dir, 0);
		bitSize += ReadMotionVector(dmv, mv_dir, 1);
	}

	return bitSize;
}

int MacroBlock::ReadMotionVector(int dmv, int mv_dir, int num)
{
	int bitSize = 0;
	int val;

	// read motion code
	val = bstream.ReadVLC();
	if(val != 0) 
	{
		// write motion code
		bitSize += tBitStream.WriteVLC(val);
		// read+write sign
		tBitStream.WriteBits(bstream.ReadBits(1), 1);
		bitSize++;

		// read+write motion residual
		_ASSERTX(f_code[mv_dir][num] > 0);								// zero is forbidden
		_ASSERTX(f_code[mv_dir][num] < 10 || f_code[mv_dir][num] == 15); // 10 through 14 are reserved
		if(f_code[mv_dir][num] != 1)
		{
			int r_size = f_code[mv_dir][num] - 1;
			tBitStream.WriteBits(bstream.ReadBits(r_size), r_size);
			bitSize += r_size;
		}
	}
	else
	{
		// write motion code
		bitSize += tBitStream.WriteVLC(val);
	}

	// read dmv data
	if(dmv == 1)
	{
		// can only be 0, 10, or 11
		val = bstream.ReadBits(1);
		tBitStream.WriteBits(val, 1);
		bitSize++;
		if(val != 0)
		{
			tBitStream.WriteBits(bstream.ReadBits(1), 1);
			bitSize++;
		}
	}

	return bitSize;
}

int MacroBlock::ReadCodedBlockPattern()
{
	int bitSize = 0;
	int val, cbp, cbpx;

	// set motion vector VLC table
	bstream.table = (VLC_ENTRY *) MBPatternVLC;
	bstream.indices = (unsigned char *) MBPatternIDX;
	tBitStream.table = (VLC_ENTRY *) MBPatternVLC;
	tBitStream.indices = (unsigned char *) MBPatternIDX;

	val = bstream.ReadVLC();
	bitSize += tBitStream.WriteVLC(val);

	int i;
	cbp = MBPattern[val];
	// generate pattern code
	for(i = 0; i < 6; i++)
	{
		if(cbp & (1 << (5 - i))) 
		{
			pattern_code[i] = 1;
		}
	}
	
	if(chroma_format == CHROMA_422)
	{
		cbpx = bstream.ReadBits(2);
		tBitStream.WriteBits(cbpx, 2);
		bitSize += 2;

		// extend pattern code
		for(i = 6; i < 8; i++)
		{
			if(cbpx & (1 << (7 - i))) 
			{
				pattern_code[i] = 1;
			}
		}
	}
	if(chroma_format == CHROMA_444)
	{
		cbpx = bstream.ReadBits(6);
		tBitStream.WriteBits(cbpx, 6);
		bitSize += 6;

		// extend pattern code
		for(i = 6; i < 12; i++)
		{
			if(cbpx & (1 << (11 - i))) 
			{
				pattern_code [i]= 1;
			}
		}
	}

	return bitSize;
}

int MacroBlock::WriteCodedBlockPattern()
{
	int bitSize = 0;
	int val, cbp, cbpx;

	// set motion vector VLC table
	bstream.table = (VLC_ENTRY *) MBPatternVLC;
	bstream.indices = (unsigned char *) MBPatternIDX;

	int i;
	cbp = 0;
	// generate pattern code
	for(i = 0; i < 6; i++)
	{
		if(pattern_code[i] == 1)
		{
			cbp |= (1 << (5 - i));
		}
	}

//	_ASSERTX(cbp > 0);

	val = MBPatternINV[cbp];
	bitSize += bstream.WriteVLC(val);

	if(chroma_format == CHROMA_422)
	{
		cbpx = 0;
		// extend pattern code
		for(i = 6; i < 8; i++)
		{
			if(pattern_code[i] == 1)
			{
				cbpx |= (1 << (7 - i));
			}
		}

		bstream.WriteBits(cbpx, 2);
		bitSize += 2;
	}
	if(chroma_format == CHROMA_444)
	{
		cbpx = 0;
		// extend pattern code
		for(i = 6; i < 12; i++)
		{
			if(pattern_code[i] == 1)
			{
				cbpx |= (1 << (11 - i));
			}
		}

		bstream.WriteBits(cbpx, 6);
		bitSize += 6;
	}

	return bitSize;
}

// adds to the macroblock address increment (skipped macroblocks) the given value
void MacroBlock::AddSkippedCount(int skippedCnt)
{
	MBAddressInc += skippedCnt;
}

// returns the macroblock address increment - the number of (skipped macroblocks)
int MacroBlock::GetSkippedCount()
{
	return MBAddressInc;
}

// returns whether the macroblock is intra coded
int MacroBlock::IsIntra()
{
	return MBType->macroblock_intra;
}

// returns whether the macroblock has coded its own quantization value
int MacroBlock::IsCodedQuantizerVal()
{
	return MBType->macroblock_quant;
}

int MacroBlock::ValidateMacroBlock(int irreductible)
{
	int maxTable;
	MBTYPE *testMBType;

	switch(pictureType)
	{
		case I_PICTURE:	maxTable = 2;  testMBType = (MBTYPE *) IFrame; break;
		case P_PICTURE:	maxTable = 7;  testMBType = (MBTYPE *) PFrame; break;
		case B_PICTURE:	maxTable = 11; testMBType = (MBTYPE *) BFrame; break;
	}

	int found = 0;
	for(int i = 0; i < maxTable; i++)
	{
		if((testMBType[i].macroblock_intra           == MBType->macroblock_intra) &&
			(testMBType[i].macroblock_motion_backward == MBType->macroblock_motion_backward) &&
			(testMBType[i].macroblock_motion_forward  == MBType->macroblock_motion_forward) &&
			(testMBType[i].macroblock_pattern         == 0) &&
			(testMBType[i].macroblock_quant           == 0))
		{
			found = 1;
			MBTypeIdx = i;
			break;
		}
	}
/*	if(irreductible && found == 0)
	{
		for(int i = 0; i < maxTable; i++)
		{
			if((testMBType[i].macroblock_intra           == MBType->macroblock_intra) &&
				(testMBType[i].macroblock_motion_backward == MBType->macroblock_motion_backward) &&
				(testMBType[i].macroblock_motion_forward  == MBType->macroblock_motion_forward) &&
				(testMBType[i].macroblock_pattern         == 1) &&
				(testMBType[i].macroblock_quant           == 1))
			{
//				found = 1;
				MBTypeIdx = i;
				break;
			}
		}
	}*/
//	_ASSERT(found == 1);
	// only remake block if it has no quantizer
	MBType = &testMBType[MBTypeIdx];

	return found;
}

// configures the block so that a quantizer is coded
int MacroBlock::SetCodedQuantizer()
{
	int maxTable;
	MBTYPE *testMBType;

	switch(pictureType)
	{
		case I_PICTURE:	maxTable = 2;  testMBType = (MBTYPE *) IFrame; break;
		case P_PICTURE:	maxTable = 7;  testMBType = (MBTYPE *) PFrame; break;
		case B_PICTURE:	maxTable = 11; testMBType = (MBTYPE *) BFrame; break;
	}

	int found = 0;
	for(int i = 0; i < maxTable; i++)
	{
		if((testMBType[i].macroblock_intra           == MBType->macroblock_intra) &&
			(testMBType[i].macroblock_motion_backward == MBType->macroblock_motion_backward) &&
			(testMBType[i].macroblock_motion_forward  == MBType->macroblock_motion_forward) &&
			(testMBType[i].macroblock_pattern         == MBType->macroblock_pattern) &&
			(testMBType[i].macroblock_quant           == 1))
		{
			found = 1;
			MBTypeIdx = i;
			break;
		}
	}
	if(found == 1) MBType = &testMBType[MBTypeIdx];

	return found;
}
