/*****************************************************************************
 * Multiplexer.h: Multiplexer Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#pragma once

#include "PESPack.h"
#include "NAVPacket.h"
#include "VideoSequence.h"
#include "IOLayer.h"
#include "IFO.h"

#define CATEGORY_NORMAL				0
#define CATEGORY_SEQUENCE_HEADER	1

typedef struct tagSTREAM_IGNORETABLE 
{
	int nextPGCMark;
	int numStream;
	int *streamID;
	int *subStreamID;
} STREAM_IGNORETABLE;

typedef struct tagPICTURE_INFO
{
	int index;
	int numPacks;
	int packIndex[512];
	int packFillBytes[512];
} PICTURE_INFO;

class Multiplexer
{
public:
	Multiplexer(void);
	~Multiplexer(void);

	virtual int Init(char *srcPath, char *targetPath, int VTSnum, int VOBnum, IFO *pIFO, int tsSrcReadSECT, int tsTargetWrittenSECT);
	virtual void SetQuality(int quality);
	virtual void SetRemovePUOs(bool removePUOs);
	virtual int GetRemovePUOs();
	virtual void SetIgnoreTable(STREAM_IGNORETABLE *IgnoreTable);
	virtual int EnableCSS(int enable);
	virtual void SetCSS(Machinist *CSS);
	virtual void SetDemoMode(bool demoMode);
	virtual int  GetDemoMode();
	virtual int ProcessVOBU();
	virtual int SearchEncryption(IOLayer *ReadIO);
	virtual void ForceTargetSize(int forceTargetSize);
	virtual void SetAlert(TransferAlert *alert);										 // sets a pointer to an alert object
																						 // the alert object can be changed at will before and after setting it in transfer
	virtual void Close();	// closes IO streams and updates statistics

	VideoSequence VideoSeq;
	int pictureCount;
	float compressionRatio;

	IOLayer ReadIO, WriteIO;
	char actualFileName[256];

protected:
	int SECTOR_SIZE, MAX_SECTORS;

	char *srcPath, *targetPath;
	int VTSnum, VOBnum;
	int removePUOs;
	int quality;

	unsigned char *pSeqBuffer;
	unsigned char *pData;
	PESPack **pPackBuffer, **pPackBufferFinalized;
	NAVPacket pNAV[3];

	unsigned char overShootBuffer[2048];
	int overShootSize;
	int overShootStreamID;

	int seqStreamID;
	int seqSize, dataSize;
	int packCount, packetsWriteWait;
	int ignoreCounter;

	IFO *pIFO;
	int tsSrcReadSECT;
	int tsTargetWrittenSECT;
	int tsSrcReadSECTVOB;
	int tsTargetWrittenSECTVOB;
	int currTransformSECTVOB;
	int currTransformSECTIFO;
	int isMenuVOB;

	STREAM_IGNORETABLE *IgnoreTable;

	virtual int ReadNextPack();
	virtual int SearchLastSequenceHeader(PESPack *pPackBuffer);
	virtual int InitCSS(int VTSnum, int VOBnum);
	virtual int ReadSequence();
	virtual int WriteSequence();

	int numPicInfo;
	int overShootNumPicInfo;
	PICTURE_INFO picInfo[64];
	int picSize[64];
	int actNumPicInfo;

	unsigned char savedByte[3];
	int savedPackIndex;

	int demoMode;
	HANDLE hHyperSpeedBMP;

	TransferAlert *alert;

	Machinist *CSS;
	HCSS hCSS;
};
