/*****************************************************************************
 * NAVPacket.cpp: Navigation Packet manipulating Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "StdAfx.h"
#include "navpacket.h"

NAVPacket::NAVPacket(void)
{
	hasDSI = 0;
	nothingAttached = 1;
}

NAVPacket::~NAVPacket(void)
{
}

unsigned char ESDSI[8] = { 0x00, 0x01, 0xCA, 0xD5, 0x08, 0x69, 0xE3, 0x11 };

int NAVPacket::offDSI = 0x3DA;

#define intCast(x)  ((NAVBuffer[x]     << 24) + \
                     (NAVBuffer[x + 1] << 16) + \
				     (NAVBuffer[x + 2] <<  8) + \
					  NAVBuffer[x + 3])

#define sintCast(x) ((NAVBuffer[x]     <<  8) + \
					  NAVBuffer[x + 1])

#define byteCast(x) NAVBuffer[x]

#define intCastW(x, y)  { NAVBuffer[x]     = (y >> 24) & 0xFF; \
						  NAVBuffer[x + 1] = (y >> 16) & 0xFF; \
						  NAVBuffer[x + 2] = (y >>  8) & 0xFF; \
						  NAVBuffer[x + 3] = y; }

#define sintCastW(x, y) { NAVBuffer[x]     = (y >>  8) & 0xFF; \
						  NAVBuffer[x + 1] = y; }

#define byteCastW(x, y) { NAVBuffer[x] = y; }

int NAVPacket::AttachPESPacket(PESPacket *pPESPacket)
{
	this->pPESPacket = pPESPacket;
	NAVBuffer = pPESPacket->payLoad;

	// check whether this is a NAVPacket
	if(pPESPacket->streamID != 0xBF) { nothingAttached = 1; return -1; }

	if(intCast(offDSI - 0x07) == 0x000001BF && byteCast(offDSI - 0x07 + 0x06) == 0x01) 
	{
		hasDSI = 1;
	}
	else 
	{
		hasDSI = 0;
	}

	if(pPESPacket->subStreamID == 0x00) 
	{ 
		nothingAttached = 0; 
		if(hasDSI) 
		{
			for(int i = 0; i < 8; i++) byteCastW(offDSI + 0x222 + i, ESDSI[i]);
		}
		return 0; 
	}

	return -1;
}

void NAVPacket::RemoveProhibitedUserOps()
{
	if(nothingAttached) return;

	intCastW(0x008, 0);
}

void NAVPacket::SetLogicalBlockNumber(int LBN)
{
	if(nothingAttached) return;

	intCastW(0x000, LBN);
	if(hasDSI) intCastW(offDSI + 0x004, LBN);
}

void NAVPacket::SetEndSECT(int SECT)
{
	if(nothingAttached) return;

	if(hasDSI) intCastW(offDSI + 0x008, SECT);
}

void NAVPacket::SetFrameEndSECT(int SECT, int index)
{
	if(nothingAttached) return;

	if(hasDSI && index < 3) intCastW(offDSI + 0x00C + index * 4, SECT);
}
