/*****************************************************************************
 * PESPack.cpp: Read and Write PESPack Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "StdAfx.h"
#include "pespack.h"

PESPack::PESPack(void)
{
	CSS = NULL;
	hCSS = NULL;
}

PESPack::PESPack(IOLayer *io)
{
	CSS = NULL;
	hCSS = NULL;
	Read(io);
}

PESPack::~PESPack(void)
{
}

// read a packet from the input layer
int PESPack::Read(IOLayer *io)
{
	// read a sector and attach a bitstream reader to it
	if((fileOffset = io->Read(pBuffer)) == -1) return -1;
/*#ifdef _DEBUG
	CheckConformance();
#endif*/
	bstream.AttachBuffer(pBuffer, SECTOR_SIZE);

	// **** ---> see ISO 13818-1 Table 2-33 (Program Stream Pack Header) <--- ***

	int val;
	// jump pack start code
	val = bstream.ReadBits(32);
	_ASSERT(val == 0x000001BA);

	// if this is a PESPack -> descramble packet if necessary
	if(CSS != NULL && hCSS != NULL) 
	{ 
		if(CSS->CSSDecrypt(pBuffer, fileOffset, hCSS) == -2) return -2;
	}

	// jump header
	bstream.Seek(77);
	// read number of stuffing bytes
	int stuffing = bstream.ReadBits(3);	_ASSERT(stuffing >= 0);
	pPESPacket = &pBuffer[14 + stuffing];
	int ret = Packet.AttachBuffer(pPESPacket, SECTOR_SIZE - 14 - stuffing);

	return ret;
}

// write the packet to the output layer
int PESPack::Write(IOLayer *io)
{
/*#ifdef _DEBUG
	CheckConformance();
#endif*/
	return io->Write(pBuffer);
}

int PESPack::AddStuffingBytes(int bytes)
{
/*	bstream.AttachBuffer(pBuffer, SECTOR_SIZE);
	bstream.Seek(32 + 77);	// jump startcode and header
	int stuffing = bstream.ReadBits(3);	
	int added = 0;

	if(stuffing < 5)
	{
		added = min(bytes, 5 - stuffing);
		MoveMemory(&pBuffer[14 + stuffing + added], &pBuffer[14 + stuffing], (SECTOR_SIZE - 14 - stuffing));
		bstream.AttachBuffer(pBuffer, SECTOR_SIZE);
		bstream.Seek(32 + 77);	// jump startcode and header
		bstream.WriteBits(stuffing + added, 3);
	}
	
	return added;*/
	return 0;
}

int PESPack::DelStuffingBytes(int bytes)
{
/*	bstream.AttachBuffer(pBuffer, SECTOR_SIZE);
	bstream.Seek(32 + 77);	// jump startcode and header
	int stuffing = bstream.ReadBits(3);	
	int subbed = 0;

	if(stuffing > 0)
	{
		subbed = min(bytes, stuffing);
		MoveMemory(&pBuffer[14 + stuffing - subbed], &pBuffer[14 + stuffing], (SECTOR_SIZE - 14 - stuffing));
		bstream.AttachBuffer(pBuffer, SECTOR_SIZE);
		bstream.Seek(32 + 77);	// jump startcode and header
		bstream.WriteBits(stuffing - subbed, 3);
	}
	
	return subbed;*/
	return 0;
}

#ifdef _DEBUG
void PESPack::CheckConformance()
{
	// debug correctness of pack
	bstream.AttachBuffer(pBuffer, SECTOR_SIZE);
	int val;
	val = bstream.ReadBits(32);	_ASSERT(val == 0x000001BA);		// read pack start code
	val = bstream.ReadBits(2);	_ASSERT(val == 1);				// read mask bits
	val = bstream.ReadBits(3);									// read scr
	val = bstream.ReadBits(1);	_ASSERT(val == 1);				// read mask bits
	val = bstream.ReadBits(15);									// read scr
	val = bstream.ReadBits(1);	_ASSERT(val == 1);				// read mask bits
	val = bstream.ReadBits(15);									// read scr
	val = bstream.ReadBits(1);	_ASSERT(val == 1);				// read mask bits
	val = bstream.ReadBits(9);									// read scr
	val = bstream.ReadBits(1);	_ASSERT(val == 1);				// read mask bits
	val = bstream.ReadBits(22);	_ASSERT(val != 0);				// read programm mux rate
	val = bstream.ReadBits(2);	_ASSERT(val == 3);				// read mask bits
	val = bstream.ReadBits(5);									// read reserved bits
	val = bstream.ReadBits(3);									// read pack stuffing length
	int i;
	for(i = 0; i < val; i++)
	{
		val = bstream.ReadBits(8); _ASSERT(val == 0xFF);		// read pack stuffing bits
	}
	val = bstream.ReadBits(24);	_ASSERT(val == 0x000001);		// read packet start code
	val = bstream.ReadBits(8);									// read stream id
	// if video stream
	if(val >= 0xE0 && val <= 0xEF)
	{
		val = bstream.ReadBits(16);									// read packet length
		int packet_length = val;
		val = bstream.ReadBits(2);	_ASSERT(val == 2);				// read mask bits
		val = bstream.ReadBits(2);/*_ASSERT(val == 0);*/			// read scrambling control
		val = bstream.ReadBits(1);									// read priority control
		val = bstream.ReadBits(1); 	_ASSERT(val == 0);				// read data alignment indicator
		val = bstream.ReadBits(1); 									// read copyright
		val = bstream.ReadBits(1); 									// read original or copy
		val = bstream.ReadBits(2);  _ASSERT(val != 1);				// read PTS DTS flags
		int PTS_DTS = val;
		val = bstream.ReadBits(1);									// read escr flag
		int ESCR = val;
		val = bstream.ReadBits(1); 									// read es rate flag
		int ES_rate = val;
		val = bstream.ReadBits(1); 	_ASSERT(val == 0);				// read DSM trick mode flag
		val = bstream.ReadBits(1); 									// read additional copy info flag
		int add_copy_info = val;
		val = bstream.ReadBits(1); 									// read PES CRC flag
		int PES_CRC = val;
		val = bstream.ReadBits(1); 									// read PES extension flag
		int PES_extension = val;
		val = bstream.ReadBits(8); 									// read PES header legnth
		int length = val;
		int actlength = 0;
		if(PTS_DTS == 2)
		{
			val = bstream.ReadBits(4); 	_ASSERT(val == 2);			// read mask bits
			val = bstream.ReadBits(3);								// read pts
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(15);								// read pts
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(15);								// read pts
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			actlength += 40;
		}
		if(PTS_DTS == 3)
		{
			val = bstream.ReadBits(4); 	_ASSERT(val == 3);			// read mask bits
			val = bstream.ReadBits(3);								// read pts
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(15);								// read pts
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(15);								// read pts
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(4); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(3);								// read dts
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(15);								// read dts
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(15);								// read dts
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			actlength += 80;
		}
		if(ESCR == 1)
		{
			val = bstream.ReadBits(2); 	_ASSERT(val == 0);			// read reserved
			val = bstream.ReadBits(3);								// read escr
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(15);								// read escr
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(15);								// read escr
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(9);								// read escr
			val = bstream.ReadBits(1);	_ASSERT(val == 1);			// read mask bits
			actlength += 48;
		}
		if(ES_rate == 1)
		{
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(22);								// read es rate
			val = bstream.ReadBits(1);	_ASSERT(val == 1);			// read mask bits
			actlength += 24;
		}
		if(add_copy_info == 1)
		{
			val = bstream.ReadBits(1); 	_ASSERT(val == 1);			// read mask bits
			val = bstream.ReadBits(7);								// read additional copy info
			actlength += 8;
		}
/*		if(PES_CRC == 1)
		{
			val = bstream.ReadBits(16);								// read additional copy info
			actlength += 16;
		}*/

	}
}
#endif