/*****************************************************************************
 * Slice.cpp: Slice Handling Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "StdAfx.h"
#include "slice.h"

int Slice::YUVreset[4] = { 128, 256, 512, 1024 };

Slice::Slice(void)
{
	MB = new MacroBlock *[1024];
	MBCount = 0;
	skipped = 0;

	bl_coefBufferMBlocks = 0;
}

Slice::~Slice(void)
{
	for(int i = 0; i < MBCount; i++)
	{
		delete MB[i];
	}
	delete MB;

	if(bl_coefBufferMBlocks != 0)
	{
		delete bl_coefBuffer;
		delete bl_runBuffer;
	}
}

// attach a buffer to the object and parse it
int Slice::AttachBuffer(unsigned char *buffer, int length, unsigned char *wbuffer)
{

	// init parameters
	pBuffer = buffer;
	bufferSize = length;

	skipped = 0;

	// attach a bitstream to the buffer
	bstream.AttachBuffer(pBuffer, bufferSize + 3);

	int val;
	// read slice start code
	val = bstream.ReadBits(32);
	_ASSERTX(val >= 0x00000101 && val <= 0x000001AF);

	// read quantizerScaleCode;
	quantizerScaleCode = bstream.ReadBits(5);
	mb_quantizerScaleCode = quantizerScaleCode;

	val = bstream.ReadBit();
	_ASSERTX(val == 0);

	// initialize predictors for I pictures
	if(pictureType == I_PICTURE)
	{
		YUVpred[0] = YUVreset[intraDCPrecision];
		YUVpred[1] = YUVreset[intraDCPrecision];
		YUVpred[2] = YUVreset[intraDCPrecision];
	}

	// read macroblocks
	currMB = 0;
	int currNumCoefMB = 0;
	int bitPos = 38;
	do {
		// dynamically add a macroblock if needed
		if(currMB + 1 > MBCount) 
		{
			MB[currMB] = new MacroBlock();
			MBCount++;
			_ASSERTX(MBCount < 1024);

			// reallocate block buffers
			if(bl_coefBufferMBlocks < MBCount)
			{
				if(bl_coefBufferMBlocks == 0)
				{
					bl_coefBufferMBlocks = 45;

					bl_coefBuffer = new short[bl_coefBufferMBlocks * 12 * 64];
					bl_runBuffer  = new unsigned char[bl_coefBufferMBlocks * 12 * 64];
				}
				else
				{
					bl_coefBufferMBlocks *= 2;
				
					short *tbl_coefBuffer = new short[bl_coefBufferMBlocks * 12 * 64];
					unsigned char *tbl_runBuffer  = new unsigned char[bl_coefBufferMBlocks * 12 * 64];

					CopyMemory(tbl_coefBuffer, bl_coefBuffer, bl_coefBufferMBlocks * 12 * 64 * sizeof(short));
					CopyMemory(tbl_runBuffer, bl_runBuffer, bl_coefBufferMBlocks * 12 * 64 * sizeof(unsigned char));

					delete bl_coefBuffer;
					delete bl_runBuffer;
	
					bl_coefBuffer = tbl_coefBuffer;
					bl_runBuffer = tbl_runBuffer;
				}
			}
		}
		SetMacroBlockData(MB[currMB]);

		MB[currMB]->bl_coefBuffer = &bl_coefBuffer[currNumCoefMB];
		MB[currMB]->bl_runBuffer = &bl_runBuffer[currNumCoefMB];

		// read the macroblock
		MB[currMB]->SetDCYUVpred(YUVpred);	// set predictors
		int bitSize = MB[currMB]->AttachBuffer(pBuffer, bufferSize - bitPos / 8, bitPos);
		MB[currMB]->GetDCYUVpred(YUVpred);	// get predictors

		currNumCoefMB += MB[currMB]->bl_numCoef;
		mb_quantizerScaleCode = MB[currMB]->quantizerScaleCode;
		absNumMacroBlock = MB[currMB]->absNumMacroBlock;
		bitPos += bitSize;
		bstream.Seek(bitSize);
//		_RPT1(_CRT_WARN, "Macroblock %d\n", currMB);
		currMB++;
//		/********remove this******->***/break;
		_ASSERTX(bstream.TestNextBits(0, 23) != -1);
	} while(bstream.TestNextBits(0, 23) == 0);

	testBitSize = bitPos;

	BitRateControl->ProcessSlice(this);

	return WriteDataBuffer(wbuffer);

}

int Slice::WriteDataBuffer(unsigned char *buffer)
{
	// move slice start code
	CopyMemory(buffer, pBuffer, 4);
	
	// attach a bitstream to the buffer (deactivate limitation as we do not use the same buffer as for reading)
	bstream.AttachBuffer(buffer + 4, 0x7FFFFFFF);//bufferSize - 4);
	
	// write quantizer scale code
	bstream.WriteBits(quantizerScaleCode, 5);

	// write unecessary value
	bstream.WriteBits(0, 1);

	int length = 38;

	// write every macroblock
	int skippedCnt = 0;
	int codedQuantizer = 0;
	int codedQuantizerScaleCode = 0;
	int oldCodedQuantizerScaleCode = quantizerScaleCode;
	int firstQuantizer = -1;
	for(int i = 0; i < currMB; i++)
	{
		if(oldCodedQuantizerScaleCode != MB[i]->quantizerScaleCode)
		{
			MB[i]->SetCodedQuantizer();
		}

		// if the macroblock comes after some skipped ones
		if(skippedCnt != 0)	    MB[i]->AddSkippedCount(skippedCnt);
		int hasCodedQuantizer = MB[i]->IsCodedQuantizerVal();
		int couldCodeQuantizer = 0;
/*		if(codedQuantizer != 0 && hasCodedQuantizer == 0) 
		{
			couldCodeQuantizer = MB[i]->SetCodedQuantizerVal(codedQuantizerScaleCode);
			// if could not code quantizer, then take the old one
			if(couldCodeQuantizer == 0)
			{
				MB[i]->quantizerScaleCode = oldCodedQuantizerScaleCode;
			}
		}
		else */

/*		// if the macroblock has no coded quantizer scale code
		if(hasCodedQuantizer == 0)
		{
			// WARNING: the max condition sets the greatest of the quantizers
			//          this will make the coefs less important but still occupying the same space
			//          a better solution should be found here
			MB[i]->quantizerScaleCode = max(MB[i]->quantizerScaleCode, oldCodedQuantizerScaleCode);
		}*/

		if(i == 0)
		{ 
			codedQuantizer = 1;
			codedQuantizerScaleCode = MB[i]->quantizerScaleCode;
		}
//		MB[i]->AddSkippedCount(1);
//		MB[i]->SetCodedQuantizerVal(MB[i]->quantizerScaleCode);

		// irreductible macroblocks
		int irreductible = 0;
		// - if it is the first or the last macroblock in a slice
	    if((i == 0) || (i == (currMB - 1))) irreductible = 1;
		// - if the former macroblock was intra coded
		if(i > 0) { if(MB[i - 1]->IsIntra() == 1) irreductible = 1; }
		// - if this macroblock is intra coded
		if(MB[i]->IsIntra() == 1) irreductible = 1;

		int status = MB[i]->WriteDataBuffer(buffer, length, irreductible);

/*		if((status & MACROBLOCK_EMPTY) == MACROBLOCK_EMPTY)
		{
			// if it has a coded quantizer value then save it
			if(hasCodedQuantizer == 1)
			{
				codedQuantizer = 1;
				codedQuantizerScaleCode = MB[i]->quantizerScaleCode;
			}					
		}
		else
		{
			codedQuantizer = 0;
		}
*/

		// 0, skip block
		// 1, take block
		int action = 1;

		// if macroblock is empty
		if((status & MACROBLOCK_EMPTY) == MACROBLOCK_EMPTY)
		{
			// if it has a coded quantizer value then save it
			if(hasCodedQuantizer == 1)
			{
				codedQuantizer = 1;
				codedQuantizerScaleCode = MB[i]->quantizerScaleCode;
			}

			// if this block is invalid and it is not the first or the last macroblock in the slice and it is then skip it
			if(((status & MACROBLOCK_INVALID) == MACROBLOCK_INVALID) && 
			   (i > 0) && (i < (currMB - 1)))
			{
				// if last block was not an intra block 
				if(MB[i - 1]->IsIntra() == 0)
				{
					skippedCnt = MB[i]->GetSkippedCount();
					action = 0; // skip block
				}
				// the else means an invalid macroblock we do not know how to handle because before there was an intra macroblock - should not be possible
				else { _ASSERT(0); length += status & 0x0FFFFFFF; skippedCnt = 0; if(couldCodeQuantizer) { codedQuantizer = 0; } };
			}
			// the else means that either the macroblock is valid (or that it is invalid and we do not know how to handle - should not be possible)
			else { length += status & 0x0FFFFFFF; skippedCnt = 0; if(couldCodeQuantizer) { codedQuantizer = 0; } }
		}
		else     { length += status & 0x0FFFFFFF; skippedCnt = 0; if(couldCodeQuantizer) { codedQuantizer = 0; } }
//		length += status & 0x0FFFFFFF;
//		length += status;
//		/********remove this******->***break;

		// if block is taken
		if(action == 1)
		{
			// if macroblock has coded the quantizer succesfully
			if(MB[i]->IsCodedQuantizerVal())
			{
				oldCodedQuantizerScaleCode = MB[i]->quantizerScaleCode;
			}
		}
	}

//	_ASSERTX(length <= testBitSize);

	// if we have to stuff some bits
	if(length % 8 != 0)
	{
		bstream.Seek(length - 38);
		bstream.WriteBits(0, 8 - length % 8);
		length += 8 - length % 8;
	}

/*	_ASSERTX(length / 8 == bufferSize);

#ifdef _DEBUG
	BitStream srcBStream;
			  srcBStream.AttachBuffer(pBuffer, 2000000);
	BitStream destBStream;
			  destBStream.AttachBuffer(buffer, 2000000);

    int val;
	for(int i = 0; i < bufferSize * 8 / 16; i++)
	{
		val = srcBStream.ReadBits(16);
		_ASSERTX(destBStream.ReadBits(16) == val);
	}

	val = srcBStream.ReadBits((bufferSize * 8) % 16);
	_ASSERTX(destBStream.ReadBits((bufferSize * 8) % 16) == val);

#endif*/

	_ASSERT(length % 8 == 0);

	// return rounded up byte length
	return (length + 7) / 8;
//	return length / 8;
//	_ASSERTX((length + 7) / 8 == bufferSize);
//	return bufferSize;
}

int Slice::SetMacroBlockData(MacroBlock *pMB)
{
	pMB->quantizerScaleCode = quantizerScaleCode;

	pMB->f_code[0][0] = f_code[0][0];
	pMB->f_code[0][1] = f_code[0][1];
	pMB->f_code[1][0] = f_code[1][0];
	pMB->f_code[1][1] = f_code[1][1];
	pMB->pictureType = pictureType;
	pMB->intraDCPrecision = intraDCPrecision;
	pMB->picture_structure = picture_structure;
	pMB->frame_pred_frame_dct = frame_pred_frame_dct;
	pMB->concealment_motion_vectors = concealmentMotionVectors;
	pMB->qScaleType = qScaleType;
	pMB->intraVLCFormat = intraVLCFormat;
	pMB->alternateScan = alternateScan;
	pMB->chroma_format = chroma_format;
	pMB->quantizerScaleCode = mb_quantizerScaleCode;
	pMB->absNumPicture = absNumPicture;
	pMB->absNumMacroBlock = absNumMacroBlock;
	pMB->demoMode = demoMode;

	return TRUE;
}