/*****************************************************************************
 * Slice.h: Slice Handling Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#pragma once

#include "BitStream.h"
#include "MacroBlock.h"

// -- attention also defined in picture.h
#define MAX_SLICE_SKIP_PROB  1000

class Picture;
class Slice;

class BitrateControl
{
public:
	BitrateControl(void);
	~BitrateControl(void);

	virtual void SetRatio(__int64 actualSizeBytes, __int64 targetSizeBytes);
	virtual float GetRatio();
	virtual float GetCurrRatio();
	virtual void UpdateStatistics();
	virtual void ProcessPicture(Picture *Frame);
	virtual void ProcessSlice(Slice *pSlice);
//	virtual int GetMBQuantizerScaleCode(int quantizerScaleCode);
	virtual float GetSliceSkipProb();
	virtual float RecalcRatio();
	virtual void SetQuality(int quality);
	virtual void ForceTargetSize(int force);

	__int64 inputBytes;
	__int64 outputBytes;

	__int64 inputBytesSeq;
	__int64 outputBytesSeq;

protected:
	float ratio, control_ratio;
	bool mult_actualized;
	int quality;
	int forceTargetSize;
	float sliceSkipProb;
	int qScaleType;

	__int64 actualSizeBytes;
	__int64 targetSizeBytes;

	static float QUALITY_QUANTIZER[5];

	static const unsigned char QuantizerScaleMap0[31];		// Quantizer Scale Map (derived from quantizer_scale_code and q_scale_type) - ISO 13818-2 Table 7-6 (q_scale_type = 0)
	static const unsigned char QuantizerScaleMap1[31];		// Quantizer Scale Map (derived from quantizer_scale_code and q_scale_type) - ISO 13818-2 Table 7-6 (q_scale_type = 1)
	static const unsigned char QuantizerScaleMap0INV[62];	// Inverse Quantizer Scale Map (derived from quantizer_scale_code and q_scale_type) - ISO 13818-2 Table 7-6 (q_scale_type = 0)
	static const unsigned char QuantizerScaleMap1INV[112];	// Inverse Quantizer Scale Map (derived from quantizer_scale_code and q_scale_type) - ISO 13818-2 Table 7-6 (q_scale_type = 1)

	virtual float CalcMultiplier();
	virtual int DecodeQuantizerScale(int qScaleType, int quantizerScaleCode);
	virtual int EncodeQuantizerScale(int qScaleType, int quantizerScale);
	virtual int DeEncodeQuantizerScale(int qScaleType, int quantizerScaleCode, float multiplier);

};

class Slice
{
public:
	Slice(void);
	~Slice(void);

	virtual int AttachBuffer(unsigned char *buffer, int length, unsigned char *wbuffer);	// attach a buffer to the object and parse it
	virtual int WriteDataBuffer(unsigned char *buffer);

	int quantizerScaleCode;
	int pictureType;
	int intraDCPrecision;
	int picture_structure;
	int frame_pred_frame_dct;
	int concealmentMotionVectors;
	int qScaleType;
	int intraVLCFormat;
	int alternateScan;
	int chroma_format;
	int f_code[2][2];
	int absNumPicture;
	int absNumMacroBlock;
	int demoMode;
	int skipped;

	BitrateControl *BitRateControl;

	MacroBlock **MB;
	int currMB;

protected:
	unsigned char *pBuffer;
	int mb_quantizerScaleCode;
	int bufferSize;

	short          *bl_coefBuffer;
	unsigned char  *bl_runBuffer;
	int			    bl_coefBufferMBlocks; // number of macroblocks the buffer can serve

	BitStream bstream;

	int MBCount;
	int YUVpred[3];
	static int YUVreset[4];

	int SetMacroBlockData(MacroBlock *pMB);

	// debug
	int testBitSize;
};
