/*****************************************************************************
 * Transfer.cpp: Main ShrinkTo5 Engine Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "StdAfx.h"
#include "transfer.h"
#include "Multiplexer.h"
#include "IFO.h"
#include "..\External\Machinist.h"
#include "..\Resource.h"


/*** plain C functions ***/
Transfer DVDTransfer;

extern "C" 
{
	int _stdcall TransferPath(char *targetPath, char *stopFlag) { return DVDTransfer.TransferPath(targetPath, stopFlag); }

	void _stdcall SetQuality(int quality)			{ DVDTransfer.SetQuality(quality); }
	int  _stdcall GetQuality()						{ return DVDTransfer.GetQuality(); }

	void _stdcall SetMovieOnly(bool movieOnly)		{ DVDTransfer.SetMovieOnly(movieOnly); }
	int  _stdcall GetMovieOnly()					{ return DVDTransfer.GetMovieOnly(); }
	
	int  _stdcall SetUseDeCSS(bool useDeCSS)		{ return DVDTransfer.SetUseDeCSS(useDeCSS); }
	int  _stdcall GetUseDeCSS()						{ return DVDTransfer.GetUseDeCSS(); }

	int  _stdcall SetAuthorizeDrive(bool authorizeDrive)	{ return DVDTransfer.SetAuthorizeDrive(authorizeDrive); }
	int  _stdcall GetAuthorizeDrive()						{ return DVDTransfer.GetAuthorizeDrive(); }

	void _stdcall SetDemoMode(bool demoMode)		{ DVDTransfer.SetDemoMode(demoMode); }
	int  _stdcall GetDemoMode()						{ return DVDTransfer.GetDemoMode(); }

	float _stdcall GetCompressionRatio()			{ return DVDTransfer.GetCompressionRatio(); }
	int _stdcall GetReadKB()						{ return DVDTransfer.GetReadKB(); }
	int _stdcall GetWrittenKB()					    { return DVDTransfer.GetWrittenKB(); }
	int _stdcall GetNumFrames()						{ return DVDTransfer.GetNumFrames(); }
	int _stdcall GetReadLimitKB()					{ return DVDTransfer.GetReadLimitKB(); }
	int _stdcall GetReadBufferOccupancyKB()			{ return DVDTransfer.GetReadBufferOccupancyKB();  }
	int _stdcall GetWriteBufferOccupancyKB()		{ return DVDTransfer.GetWriteBufferOccupancyKB(); }

	char * _stdcall GetActualFileName()				{ return DVDTransfer.GetActualFileName(); }
	int _stdcall GetElapsedTimeSec()				{ return DVDTransfer.GetElapsedTimeSec(); }
	int _stdcall GetEstimatedTimeSec()				{ return DVDTransfer.GetEstimatedTimeSec(); }
	int _stdcall GetRemainingTimeSec()				{ return DVDTransfer.GetRemainingTimeSec(); }

	int _stdcall Open(char *srcPath){ return DVDTransfer.Open(srcPath); }
	int _stdcall SetTargetSizeMB(int targetSize)	 { return DVDTransfer.SetTargetSizeMB(targetSize); }
	int _stdcall GetTargetSizeMB()					 { return DVDTransfer.GetTargetSizeMB(); }
	int _stdcall GetSourceSizeMB(bool movieOnly)	 { return DVDTransfer.GetSourceSizeMB(movieOnly); }
	int _stdcall ForceTargetSize(bool force)		 { return DVDTransfer.ForceTargetSize(force); }
	int _stdcall GetNumVTS()						 { return DVDTransfer.GetNumVTS(); }
	int _stdcall GetVTSSize(int VTSIndex)			 { return DVDTransfer.GetVTSSize(VTSIndex); }						

	int _stdcall GetNumVTSLang(int VTSIndex)		 { return DVDTransfer.GetNumVTSLang(VTSIndex); }
	int _stdcall GetVTSLangDesc(int VTSIndex, int langIndex, char *buffer, int szBuffer)	{ return DVDTransfer.GetVTSLangDesc(VTSIndex, langIndex, buffer, szBuffer); }
	int _stdcall GetVTSLangSelected(int VTSIndex, int langIndex)							{ return DVDTransfer.GetVTSLangSelected(VTSIndex, langIndex); }
	int _stdcall GetVTSLangDefault(int VTSIndex, int langIndex)								{ return DVDTransfer.GetVTSLangDefault(VTSIndex, langIndex); }
	int _stdcall SetVTSLangSelected(int VTSIndex, int langIndex, int selected)				{ return DVDTransfer.SetVTSLangSelected(VTSIndex, langIndex, selected); }
	int _stdcall SetVTSLangDefault(int VTSIndex, int langIndex)								{ return DVDTransfer.SetVTSLangDefault(VTSIndex, langIndex); }

	int _stdcall GetNumVTSSub(int VTSIndex)													{ return DVDTransfer.GetNumVTSSub(VTSIndex); }
	int _stdcall GetVTSSubDesc(int VTSIndex, int subIndex, char *buffer, int szBuffer)		{ return DVDTransfer.GetVTSSubDesc(VTSIndex, subIndex, buffer, szBuffer); }
	int _stdcall GetVTSSubSelected(int VTSIndex, int subIndex)								{ return DVDTransfer.GetVTSSubSelected(VTSIndex, subIndex); }
	int _stdcall GetVTSSubDefault(int VTSIndex, int subIndex)								{ return DVDTransfer.GetVTSSubDefault(VTSIndex, subIndex); }
	int _stdcall SetVTSSubSelected(int VTSIndex, int subIndex, int selected)				{ return DVDTransfer.SetVTSSubSelected(VTSIndex, subIndex, selected); }
	int _stdcall SetVTSSubDefault(int VTSIndex, int subIndex)								{ return DVDTransfer.SetVTSSubDefault(VTSIndex, subIndex); }
}

/*** C++ object ***/
Transfer::Transfer(void)
{
	// initialize global statistics behaviour
	globalReadKB = globalWrittenKB = globalNumFrames = -1;
	baseTimeMSec = elapsedTimeMSec = estimatedTimeMSec = remainingTimeMSec = -1;
	opened = 0;
	
	SelectTable = (char ***)new int[100];
	for(int i = 0; i < 100; i++)
	{
		SelectTable[i] = (char **)new int[2];
		for(int j = 0; j < 2; j++)
		{
			SelectTable[i][j] = (char *)new int[32];
			for(int k = 0; k < 32; k++)
			{
				SelectTable[i][j][k] = 0;
			}
		}
	}

	mainIFO = new IFO();
	currIFO = new IFO();
	openIFO = new IFO();

	IgnoreTable = NULL;

	defAud = -1;
	defSub = -1;

	compressionRatio = 1;
	strcpy(actualFileName, "");

	movieOnly = 0;

	useDeCSS = 0;
	authorize = 0;

	demoMode = 0;
	quality = 1;

	targetSizeMB = 4482;
	forceTargetSize = 0;

	CSS = new Machinist();

	alert = NULL;
}

Transfer::~Transfer(void)
{
	for(int i = 0; i < 100; i++)
	{
		for(int j = 0; j < 2; j++)
		{
			delete SelectTable[i][j];
		}
		delete SelectTable[i];
	}
	delete SelectTable;

	STREAM_IGNORETABLE *IgnoreTable = (STREAM_IGNORETABLE *)this->IgnoreTable;
	if(IgnoreTable != NULL) 
	{
		int i = -1;
		do
		{
			i++;
			delete IgnoreTable[i].streamID;
			delete IgnoreTable[i].subStreamID;
		} while(IgnoreTable[i].nextPGCMark != INT_MAX);
		delete IgnoreTable;
	}

	delete (IFO *)this->mainIFO;
	delete (IFO *)this->currIFO;
	delete (IFO *)this->openIFO;

	delete CSS;
}

int Transfer::TransferPath(char *targetPath, char *stopFlag)
{
	char actual_targetPath[256];
	char actual_srcPath[256];

	// if the source was not successfully opened
	if(opened == 0) return -5;

	// set global statistics behaviour
	globalReadKB = globalWrittenKB = globalNumFrames = 0;
	compressionRatio = 1;
	strcpy(actualFileName, "");

	// add backslash to the end of pathname
	strcpy(actual_targetPath, targetPath);
	int len = strlen(actual_targetPath);
	if(actual_targetPath[len - 1] != '\\')
	{
		actual_targetPath[len]	   = '\\';
		actual_targetPath[len + 1] = '\0';
	}

	if(strcmpi(&actual_targetPath[strlen(actual_targetPath) - 9], "VIDEO_TS\\") != 0)
	{
		strcat(actual_targetPath, "VIDEO_TS\\");
		CreateDirectory(actual_targetPath, NULL);
	}

	// add backslash to the end of pathname
	strcpy(actual_srcPath, srcPath);

	int srcReadMB = 0, targetWrittenMB = 0;

	// initialize time variables
	baseTimeMSec = GetTickCount();
	savedEstimatedTimeMSec = 0;

	// transfer VOBs/IFOs
	int result = TransferFiles(actual_srcPath, actual_targetPath, /*&srcReadMB, &targetWrittenMB,*/ sourceSizeMB, targetSizeMB, stopFlag);
	// if video is scrambled
	if(result == -2) return -2;
	// if drive cannot be authorized
	if(result == -6) return -6;

	// if there where no errors then save number of written MB
	targetWrittenMB = result;
	if(*stopFlag != 0 && targetWrittenMB > 0) targetWrittenMB = 0;

	// set global statistics behaviour
	globalReadKB = globalWrittenKB = globalNumFrames = -1;

	// deinitialize time variables
	estimatedTimeMSec = remainingTimeMSec = -1;

	return targetWrittenMB;
}

/// transfer a VOB file
int Transfer::TransferFile(char *srcPath, char *targetPath, int targetSizeMB, int tsSrcReadSECT, int tsTargetWrittenSECT, int VTSnum, int VOBnum, char *stopFlag)
{
	if(*stopFlag == 1) return 0;

	IFO *currIFO = (IFO *)this->currIFO;
	IFO *mainIFO = (IFO *)this->mainIFO;

	int srcSizeMB = GetVOBSizeMB(srcPath, VTSnum, VOBnum);
	// if file is very small, ignore it 
	//   -> set source and target size to 1
	if(srcSizeMB == 0) 
	{
		srcSizeMB = 1;
		targetSizeMB = 1;
	}
	if(targetSizeMB == 0)
	{
		targetSizeMB = 1;
	}

	IFO *pIFO = mainIFO;
	if(currIFO->isVTS() != 0) pIFO = currIFO;

	if(*stopFlag == 0) GenerateIgnoreTable(VTSnum, VOBnum);

	Multiplexer *Muxer = new Multiplexer();
	Muxer->SetAlert(alert);
	Muxer->SetRemovePUOs(movieOnly); // this setting is on for movie only mode
	Muxer->SetQuality(quality);		 // quality setting
	Muxer->Init(srcPath, targetPath, VTSnum, VOBnum, pIFO, tsSrcReadSECT, tsTargetWrittenSECT);
	Muxer->SetCSS((Machinist *)CSS);
    if(Muxer->EnableCSS(useDeCSS) == -2) return -2;
	Muxer->SetIgnoreTable((STREAM_IGNORETABLE *)IgnoreTable);
	Muxer->VideoSeq.BitRateControl.SetRatio(((__int64)srcSizeMB) * 1024 * 1024, ((__int64)targetSizeMB) * 1024 * 1024);
	Muxer->SetDemoMode(demoMode);
	Muxer->ForceTargetSize(forceTargetSize);

	int result;

	// initialize statistics
	localReadKB = localWrittenKB = localNumFrames = 0;

	while((result = Muxer->ProcessVOBU()) >= 0 && *stopFlag == 0)
	{
		// show actual file name
		strcpy(actualFileName, Muxer->actualFileName);

		// actualize statistics
		compressionRatio = Muxer->compressionRatio;
//		readBufferOccupancy  = ReadIO.bufferOccupancy  / 1024;
//		writeBufferOccupancy = WriteIO.bufferOccupancy / 1024;
		if(globalReadKB != -1)	globalReadKB += Muxer->ReadIO.kBThroughput / 1024 - localReadKB;
								 localReadKB  = Muxer->ReadIO.kBThroughput / 1024;
		if(globalWrittenKB != -1) globalWrittenKB += Muxer->WriteIO.kBThroughput / 1024 - localWrittenKB;
								   localWrittenKB  = Muxer->WriteIO.kBThroughput / 1024;
		if(globalNumFrames != -1) globalNumFrames += Muxer->pictureCount - localNumFrames;
								   localNumFrames  = Muxer->pictureCount;

	   #define TIMEESTIMATE_BEGINDELAY      10 // 8 means 8 seconds stabilization reserve for phase 1
	   #define TIMEESTIMATE_BEGINSTABILITY  2  // 1 means 1 second actual data
	   #define TIMEESTIMATE_STABILITY       60 // 60 means one about minute (60 seconds) actual data

		elapsedTimeMSec = GetTickCount() - baseTimeMSec;

		// calculate estimated time (phase 1 at beginning)
		if((GetReadKB() / 1024) <= TIMEESTIMATE_BEGINDELAY)
		{
			if((GetReadKB() / 1024) > 0)
			{
				savedEstimatedTimeMSec += 1.0 / TIMEESTIMATE_BEGINSTABILITY * ((float)(GetReadLimitKB() / 1024) * (float)(GetTickCount() - baseTimeMSec) / ((float)(GetReadKB() / 1024)) - savedEstimatedTimeMSec);
			}
		}

		if((GetReadKB() / 1024) > TIMEESTIMATE_BEGINDELAY)
		{
			// calculate estimated time (phase 2 during transfer)
			if((GetReadKB() / 1024) > 0)
			{
				savedEstimatedTimeMSec += 1.0 / TIMEESTIMATE_STABILITY * ((float)(GetReadLimitKB() / 1024) * (float)(GetTickCount() - baseTimeMSec) / ((float)(GetReadKB() / 1024)) - savedEstimatedTimeMSec);
			}

			if(savedEstimatedTimeMSec < (GetTickCount() - baseTimeMSec)) savedEstimatedTimeMSec = (GetTickCount() - baseTimeMSec);

			estimatedTimeMSec = savedEstimatedTimeMSec;
			remainingTimeMSec = estimatedTimeMSec - elapsedTimeMSec;
		}

	}

	Muxer->Close();
	int actualSizeSECT = Muxer->WriteIO.kBThroughput / 2048;

	// tidy up
	delete Muxer;

	// if packets are scrambled
	if(result == -2) return -2;

	return actualSizeSECT;
}

/// evaluate size of all files in the path
__int64 Transfer::EvaluatePath(char *path)
{
	__int64 actualSize = 0;

	char actual_path[256];
	WIN32_FIND_DATA fileEnum;

	strcpy(actual_path, path);
	strcat(actual_path, "*");

	HANDLE hFileEnum = FindFirstFile(actual_path, &fileEnum);
	if(hFileEnum == INVALID_HANDLE_VALUE) return 0;
	do
	{
		if(strcmp(fileEnum.cFileName, ".")  != 0 && 
		   strcmp(fileEnum.cFileName, "..") != 0)
		{
			strcpy(actual_path, path);
			strcat(actual_path, fileEnum.cFileName);

			int len = strlen(fileEnum.cFileName);		
			if(len == 12)
			{
				if(((strcmpi(&fileEnum.cFileName[len - 4], ".VOB") == 0)  ||
				    (strcmpi(&fileEnum.cFileName[len - 4], ".IFO") == 0)  ||
				    (strcmpi(&fileEnum.cFileName[len - 4], ".BUP") == 0)) &&
					(fileEnum.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != FILE_ATTRIBUTE_DIRECTORY)
				{
					actualSize += fileEnum.nFileSizeLow;
				}
			}
		}
	} while(FindNextFile(hFileEnum, &fileEnum) != 0);

	return actualSize;
}

void Transfer::CopyAllFiles(char *srcPath, char *targetPath)
{
	char actual_srcPath[256];
	char actual_targetPath[256];
	WIN32_FIND_DATA fileEnum;

	strcpy(actual_srcPath, srcPath);
	strcat(actual_srcPath, "*");

	HANDLE hFileEnum = FindFirstFile(actual_srcPath, &fileEnum);
	if(hFileEnum == INVALID_HANDLE_VALUE) return;
	do
	{
		if(strcmp(fileEnum.cFileName, ".")  != 0 && 
		   strcmp(fileEnum.cFileName, "..") != 0)
		{
			strcpy(actual_targetPath, targetPath);
			strcat(actual_targetPath, fileEnum.cFileName);

			strcpy(actual_srcPath, srcPath);
			strcat(actual_srcPath, fileEnum.cFileName);

			localReadKB = localWrittenKB = 0;

			int len = strlen(fileEnum.cFileName);		
			if(len == 12)
			{
				if((/*(strcmpi(&fileEnum.cFileName[len - 4], ".VOB") == 0)  ||*/
				    (strcmpi(&fileEnum.cFileName[len - 4], ".IFO") == 0)  ||
				    (strcmpi(&fileEnum.cFileName[len - 4], ".BUP") == 0)) &&
					(fileEnum.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != FILE_ATTRIBUTE_DIRECTORY)
				{
					strcpy(actualFileName, actual_srcPath);
					CopyFile(actual_srcPath, actual_targetPath, 0);
					if(globalReadKB != -1)	globalReadKB += fileEnum.nFileSizeLow / 1024 - localReadKB;
											 localReadKB  = fileEnum.nFileSizeLow / 1024;
					if(globalWrittenKB != -1) globalWrittenKB += fileEnum.nFileSizeLow / 1024 - localWrittenKB;
											   localWrittenKB  = fileEnum.nFileSizeLow / 1024;
				}
			}
		}
	} while(FindNextFile(hFileEnum, &fileEnum) != 0);
}

int Transfer::TransferFiles(char *srcPath, char *targetPath, /*int *srcReadMB, int *targetWrittenMB, */int srcSizeMB, int targetSizeMB, char *stopFlag)
{
	char actual_srcPath[256];
	char actual_targetPath[256];
//c	WIN32_FIND_DATA fileEnum;
	IFO *currIFO = (IFO *)this->currIFO;
	IFO *mainIFO = (IFO *)this->mainIFO;

	// if authorization is enabled
	if(authorize == 1)
	{
		char root[4];
		root[0] = srcPath[0]; 
		root[1] = srcPath[1]; 
		root[2] = srcPath[2]; 
		root[3] = '\0';
		unsigned int driveType = GetDriveType(root);
		// only if we have a dvd drive
		if(driveType == DRIVE_CDROM)
		{
			int retval = ((Machinist *)CSS)->CSSAuth(srcPath);
		/*	// write down disc key
			char key[32768];
			sprintf(key, "DiscKey=");
			for(int i = 0; i < 2047; i++)
			{
				sprintf(key, "%s%02X:", key, auth->DiscKey[i]);
			}
			sprintf(key, "%s%02X\r\n", key, auth->DiscKey[2047]);

			sprintf(actual_targetPath, "%sEXTRA.TXT", targetPath);

			// open file
			HANDLE hFile = CreateFile(actual_targetPath, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL);
			_ASSERT(hFile > 0);

			int writeSize = 0;
			WriteFile(hFile, key, strlen(key), (LPDWORD)&writeSize, NULL);
			_ASSERT(writeSize == strlen(key));

			// close file
			CloseHandle(hFile);*/
			if(retval != 0) return -6;
		}
	}

	_ASSERT(srcSizeMB > 0);
	_ASSERT(targetSizeMB > 0);

	int targetWrittenSECT = 0;
	int srcReadSECT = 0;
	int tsTargetWrittenSECT = 0;
	int tsSrcReadSECT = 0;

	// if only movie should be copied
	if(movieOnly)
	{
		// create template VIDEO_TS.IFO
		// load template IFO from resource
		HMODULE hMod = GetModuleHandle("ShrinkTo5.dll");
		HRSRC hRs = FindResource(hMod, MAKEINTRESOURCE(IDR_IFO), "IFO");
		_ASSERT(hRs != 0);
		HGLOBAL dummyIFO = LoadResource(hMod, hRs);
		void *pDummyIFO = LockResource(dummyIFO);

		// write template IFO into a temporary file
		char actual_tempPath[256];
		int ret = GetTempFileName(".\\", "MPT", 0, actual_tempPath);
		_ASSERT(ret != 0);

		HANDLE hDummyIFOFile = CreateFile(actual_tempPath, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);
		_ASSERT(hDummyIFOFile > 0);

		int writeSize, resourceSize = SizeofResource(hMod, hRs);
		// write whole file
		WriteFile(hDummyIFOFile, pDummyIFO, resourceSize, (LPDWORD)&writeSize, NULL);
		_ASSERT(writeSize == resourceSize);

		CloseHandle(hDummyIFOFile);

		// load that temporary IFO file in the IFO parser
		mainIFO->Load(actual_tempPath);

		// delete the temporary IFO file
		DeleteFile(actual_tempPath);		

		// update sector readings
		_ASSERT(resourceSize % 2048 == 0);
		sprintf(actual_srcPath,    "%sVIDEO_TS.VOB", srcPath);

		tsSrcReadSECT		+= 2 * GetFileSizeSECT(actual_srcPath); // IFO+BUP size
		tsTargetWrittenSECT += 2 * resourceSize / 2048; // IFO+BUP size

		// write IFO length data and reset title set size counters
		targetWrittenSECT += tsTargetWrittenSECT;
		srcReadSECT += tsSrcReadSECT;

		tsSrcReadSECT		= 0;
		tsTargetWrittenSECT = 0;
	}
	else
	{
		// load main (VMG) IFO
		sprintf(actual_srcPath, "%sVIDEO_TS.IFO", srcPath);
		if(alert != NULL) alert->SendAlert(ALERT_FILENAME_CHANGED, (int)actual_srcPath);

		mainIFO->Load(actual_srcPath);
		currIFO->Load(actual_srcPath);
		tsSrcReadSECT		+= GetFileSizeSECT(actual_srcPath); // IFO size
		tsTargetWrittenSECT += GetFileSizeSECT(actual_srcPath); // IFO size

		mainIFO->SetFirstMenuVOBSector(tsTargetWrittenSECT);

		// process main (VMG) VOB
		sprintf(actual_srcPath,    "%sVIDEO_TS.VOB", srcPath);
		sprintf(actual_targetPath, "%sVIDEO_TS.VOB", targetPath);

		int ret;

		// if VIDEO_TS.VOB exists
		if(GetFileSizeSECT(actual_srcPath) > 0)
		{
			//	mainIFO.SetCurrSectorEntry(tsSrcReadSECT - 1);		// -1 because sector count begins from 0
			mainIFO->SetCurrSectorEntry(0, REFERENCE_IFO);		// -1 because sector count begins from 0
			mainIFO->SetCurrSectorEntry(0, REFERENCE_MENU_VOB);	// -1 because sector count begins from 0

			ret = TransferFile(srcPath, targetPath, GetVOBSizeMB(srcPath, 0, 0) * targetSizeMB / srcSizeMB, tsSrcReadSECT, tsTargetWrittenSECT, 0, 0, stopFlag);
			// if there was an error
			if(ret < 0) return ret;

			tsTargetWrittenSECT += ret;
			tsSrcReadSECT += GetVOBSizeSECT(srcPath, 0, 0);
		}

		sprintf(actual_srcPath, "%sVIDEO_TS.IFO", srcPath);
		tsSrcReadSECT		+= GetFileSizeSECT(actual_srcPath); // BUP size
		tsTargetWrittenSECT += GetFileSizeSECT(actual_srcPath); // BUP size

		// write IFO length data and reset title set size counters
		targetWrittenSECT += tsTargetWrittenSECT;
		srcReadSECT += tsSrcReadSECT;

		mainIFO->SetLastBUPSector(tsTargetWrittenSECT - 1); // -1 because sector count begins from 0
		currIFO->Close();

		tsTargetWrittenSECT = 0;
		tsSrcReadSECT = 0;
	}

	// if user cancelled
	if(*stopFlag == 1) return 0;

	// set how many title sets and VOBs to search
	int minVTSnum = 1, maxVTSnum = 100;
	int minVOBnum = 0, maxVOBnum = 2;

	// for movie only mode - grab the biggest VTS only
	if(movieOnly)
	{
		int movieVTS = GetMovieOnlyVTSNum();
		int movieSize = GetMovieOnlySizeMB();

		minVTSnum = movieVTS;
		maxVTSnum = movieVTS + 1;
		minVOBnum = 1;

		// set the new smaller movie size
		sourceSizeKB = movieSize * 1024;
		sourceSizeMB = movieSize;
	}

	int ret = 0;

/*	minVTSnum = 3;
	maxVTSnum = minVTSnum + 1;
	minVOBnum = 1;
	maxVOBnum = minVOBnum + 1;*/

	// check for titlesets
	for(int i = minVTSnum; i < maxVTSnum; i++)
	{
		// load titleset IFO
		sprintf(actual_srcPath, "%sVTS_%02i_0.IFO", srcPath, i);
		if(alert != NULL) alert->SendAlert(ALERT_FILENAME_CHANGED, (int)actual_srcPath);

		// if file does not exist - break
		if(GetFileSizeSECT(actual_srcPath) == 0) break;

		currIFO->Load(actual_srcPath);
		tsSrcReadSECT		+= GetFileSizeSECT(actual_srcPath); // IFO size
		tsTargetWrittenSECT += GetFileSizeSECT(actual_srcPath); // IFO size

		// only if <not> in movie only mode
		if(!movieOnly) mainIFO->VMG_SetTitleStartSector(i, targetWrittenSECT);

		// process titleset VOBs
		for(int j = minVOBnum; j < maxVOBnum; j++)
		{
			sprintf(actual_srcPath,    "%sVTS_%02i_%1i.VOB", srcPath,    i, j);
			sprintf(actual_targetPath, "%sVTS_%02i_%1i.VOB", targetPath, i, j);

			// if file does not exist - break
			if(GetFileSizeSECT(actual_srcPath) == 0)
			{
				if(j > 0)  break;
				if(j == 0) continue;
			}
			else
			{
				if(j == 1) currIFO->SetFirstTitleVOBSector(tsTargetWrittenSECT);
				if(j == 0) currIFO->SetFirstMenuVOBSector(tsTargetWrittenSECT);
			}

			// initialize the get-next-sector iterator
			currIFO->SetCurrSectorEntry(tsSrcReadSECT/*    - 1*/, REFERENCE_IFO);//tsSrcReadSECT - 1);				// -0 because sector count begins from 0 (but next sector has not yet been added)
				 if(j == 0)	currIFO->SetCurrSectorEntry(0/* - 1*/, REFERENCE_MENU_VOB);//tsSrcReadSECT - 1); // -0 because sector count begins from 0 (but next sector has not yet been added)
			else if(j == 1)	currIFO->SetCurrSectorEntry(0/* - 1*/, REFERENCE_TITLE_VOB);//tsSrcReadSECT - 1); // -0 because sector count begins from 0 (but next sector has not yet been added)
//			else			currIFO->SetCurrSectorEntry(tsSrcReadSECTVOB/* - 1*/, REFERENCE_TITLE_VOB);//tsSrcReadSECT - 1); // -0 because sector count begins from 0 (but next sector has not yet been added)

			// transfer the VOB
			_ASSERT((srcReadSECT + tsSrcReadSECT) > 0);
			_ASSERT((targetWrittenSECT + tsTargetWrittenSECT) > 0);

			// calculate target size for this file
/*			__int64 x = (__int64)srcSizeMB * srcSizeMB * ((targetWrittenSECT + tsTargetWrittenSECT) / 512);
			__int64 curr_targetSize = ((__int64)GetVOBSizeMB(srcPath, i, j) * targetSizeMB) / srcSizeMB;
			if(x != 0) curr_targetSize = ((__int64)GetVOBSizeMB(srcPath, i, j) * ((srcReadSECT + tsSrcReadSECT) / 512) * targetSizeMB * targetSizeMB) / x;
			if(curr_targetSize == 0) curr_targetSize = 1;*/
			__int64 currWrittenMB = (__int64)((targetWrittenSECT + tsTargetWrittenSECT) / 512);
			__int64 currReadMB = (__int64)((srcReadSECT + tsSrcReadSECT) / 512);
			__int64 diffWrittenMB = currReadMB * targetSizeMB / srcSizeMB - currWrittenMB;
			__int64 currVOBSize = GetVOBSizeMB(srcPath, i, j);
			__int64 diffVOBMB = currVOBSize * diffWrittenMB / srcSizeMB;
			__int64 curr_targetSize = (currVOBSize * targetSizeMB) / srcSizeMB + diffVOBMB;
//			if(y != 0) curr_targetSize = ((__int64)GetVOBSizeMB(srcPath, i, j)) * x / y;
			if(curr_targetSize == 0) curr_targetSize = 1;

			// for movie only mode - target Size is always maximum size
			if(movieOnly) curr_targetSize = targetSizeMB;

			ret = TransferFile(srcPath, targetPath, curr_targetSize, tsSrcReadSECT, tsTargetWrittenSECT, i, j, stopFlag);

			if(ret >= 0) tsTargetWrittenSECT += ret;
			else break;

			// if user aborted
			if(*stopFlag != 0) return 0;

			tsSrcReadSECT += GetVOBSizeSECT(srcPath, i, j);
		}

		// if stream is scrambled
		if(ret == -2) return -2;
		// if drive cannot be authorized
		if(ret == -6) return -6;

		// for movie only mode - copy audio and video attributes into the video manager ifo and
		//                       search and select movie to play
		if(movieOnly) 
		{
			mainIFO->CopyAudioVideoAttributes(currIFO->GetIFOBuffer());
			// search TT in VTS (movie)
			int PGCidx;
			int TT = currIFO->SearchMovieTitle(&PGCidx);
			// make movie title playable
			currIFO->MakeMovieTitlePlayable(TT, PGCidx, defAud, defSub, SelectTable[i][0], SelectTable[i][1]);

			// load original main ifo
			char temp_srcPath[256];
			sprintf(temp_srcPath, "%sVIDEO_TS.IFO", srcPath);
			IFO orgmainIFO;
			orgmainIFO.Load(temp_srcPath);

			// select movie
			mainIFO->SelectMovieTitle(TT, i, orgmainIFO.GetIFOBuffer());

			orgmainIFO.Close();
		}

		sprintf(actual_srcPath, "%sVTS_%02i_0.IFO", srcPath, i);
		tsSrcReadSECT		+= GetFileSizeSECT(actual_srcPath); // BUP size
		tsTargetWrittenSECT += GetFileSizeSECT(actual_srcPath); // BUP size

		// write IFO length data and reset title set size counters
		targetWrittenSECT += tsTargetWrittenSECT;
		srcReadSECT += tsSrcReadSECT;

		currIFO->SetLastBUPSector(tsTargetWrittenSECT - 1); // -1 because sector count begins from 0

		tsTargetWrittenSECT = 0;
		tsSrcReadSECT = 0;

		// save modified titleset IFO
		sprintf(actual_targetPath, "%sVTS_%02i_0.IFO", targetPath, i);
		currIFO->Save(actual_targetPath);

	}

	// for movie only mode - rename the transcoded VTS_yy_x to VTS_01_x
	if(movieOnly)
	{
		// rename IFO and BUP
		sprintf(actual_srcPath,    "%sVTS_%02i_0.IFO", targetPath, minVTSnum);
		sprintf(actual_targetPath, "%sVTS_%02i_0.IFO", targetPath, 1);

		ret = MoveFile(actual_srcPath, actual_targetPath);
		_ASSERT(ret != 0);

		sprintf(actual_srcPath,    "%sVTS_%02i_0.BUP", targetPath, minVTSnum);
		sprintf(actual_targetPath, "%sVTS_%02i_0.BUP", targetPath, 1);

		ret = MoveFile(actual_srcPath, actual_targetPath);
		_ASSERT(ret != 0);

		// rename VOBs
		for(int j = 1; j < 10; j++)
		{
			sprintf(actual_srcPath,    "%sVTS_%02i_%1i.VOB", targetPath, minVTSnum, j);
			sprintf(actual_targetPath, "%sVTS_%02i_%1i.VOB", targetPath, 1, j);

			// if file does not exist - break
			if(GetFileSizeSECT(actual_srcPath) == 0) break;
			ret = MoveFile(actual_srcPath, actual_targetPath);
			_ASSERT(ret != 0);
		}
	}

	sprintf(actual_targetPath, "%sVIDEO_TS.IFO", targetPath);

	mainIFO->Save(actual_targetPath);

	return targetWrittenSECT / 512;	
}


int Transfer::SearchEncryption(char *srcPath, char *stopFlag)
{
	Multiplexer *Muxer = new Multiplexer();
	IOLayer ReadIO;

	char actual_srcPath[256];

	// process main (VMG) VOB
	sprintf(actual_srcPath, "%sVIDEO_TS.VOB", srcPath);

	ReadIO.Open((unsigned char *)actual_srcPath, 0);
	int result = Muxer->SearchEncryption(&ReadIO);
	ReadIO.Close();
	if(result == -1) return -2;

	// check for titlesets
	for(int i = 1; i < 100; i++)
	{
		// if does not file exist - break
		if(GetFileSizeSECT(actual_srcPath) == 0) break;

		// process titleset VOBs
		for(int j = 0; j < 10; j++)
		{
			sprintf(actual_srcPath, "%sVTS_%02i_%1i.VOB", srcPath, i, j);

			// if does not file exist - break
			if(GetFileSizeSECT(actual_srcPath) == 0) break;

			ReadIO.Open((unsigned char *)actual_srcPath, 0);
			result = Muxer->SearchEncryption(&ReadIO);
			ReadIO.Close();
			if(result == -1) return -2;
		}
	}

	delete Muxer;

	return 1;	
}

// sets quality option, numbers outside the range 0..4 will be rounded to the next valid option
void Transfer::SetQuality(int quality)
{
	this->quality = quality;
	if(this->quality > 4) this->quality = 4;
	if(this->quality < 0) this->quality = 0;
}

// returns the actual quality option
int  Transfer::GetQuality()					
{
	return quality;
}

// enables (1) or disables (0) movie only mode
void Transfer::SetMovieOnly(bool movieOnly)
{
	this->movieOnly = movieOnly;
	((IFO *)mainIFO)->SetMovieOnly(movieOnly);
	((IFO *)currIFO)->SetMovieOnly(movieOnly);
	((IFO *)openIFO)->SetMovieOnly(movieOnly);

	if(opened)
	{
		((IFO *)openIFO)->Close();

		// reload main IFO for updating movie only mode
		char actual_srcPath[256];
		strcpy(actual_srcPath, srcPath);
		strcat(actual_srcPath, "VIDEO_TS.IFO");
		((IFO *)openIFO)->Load(actual_srcPath);
	}
}

// returns whether movie only mode is enabled (1) or disabled (=custom mode) (0)
int  Transfer::GetMovieOnly()
{
	return movieOnly;
}

// enables (1) or disables (0) decrypting
int Transfer::SetUseDeCSS(bool useDeCSS)
{
	if(((Machinist *)CSS)->isAvailable == 0) return -1;
	this->useDeCSS = useDeCSS;

	return 1;
}

// returns whether decrypting is enabled (1), disabled (0) or not available (-1)
int  Transfer::GetUseDeCSS()
{
	if(((Machinist *)CSS)->isAvailable == 0) return -1;
	return useDeCSS;
}

// enables (1) or disables (0) authorization of drive
int Transfer::SetAuthorizeDrive(bool authorizeDrive)
{
	if(((Machinist *)CSS)->isAvailable == 0) return -1;
	authorize = authorizeDrive;

	return 1;
}

// returns whether authorization is enabled (1), disabled (0) or not available (-1)
int  Transfer::GetAuthorizeDrive()
{
	if(((Machinist *)CSS)->isAvailable == 0) return -1;
	return authorize;
}

// sets the demo mode (1 = demo, 0 = normal)
void Transfer::SetDemoMode(bool demoMode)
{
	this->demoMode = demoMode;
}

// returns the current demo mode
int  Transfer::GetDemoMode()
{
	return demoMode;
}

int Transfer::GetReadKB()
{
	if(globalReadKB != -1) return globalReadKB;
						   return  localReadKB;
}

int Transfer::GetWrittenKB()
{
	if(globalWrittenKB != -1) return globalWrittenKB;
							  return  localWrittenKB;
}

int Transfer::GetNumFrames()
{
	if(globalNumFrames != -1) return globalNumFrames;
							  return  localNumFrames;
}

int Transfer::GetReadLimitKB()
{
	return sourceSizeKB;
}

int Transfer::GetReadBufferOccupancyKB()
{
	return readBufferOccupancy;
}

int Transfer::GetWriteBufferOccupancyKB()
{
	return writeBufferOccupancy;
}

char *Transfer::GetActualFileName()
{
	return actualFileName;
}

// return the file size in number of sectors
int Transfer::GetFileSizeSECT(char *srcPath)
{
	WIN32_FILE_ATTRIBUTE_DATA fileData;

	int ret = GetFileAttributesEx(srcPath, GetFileExInfoStandard, &fileData);

	// if error occurs
	if(ret == 0) return 0;

	return (fileData.nFileSizeLow / 2048);
}

int Transfer::GetElapsedTimeSec()
{
	if(elapsedTimeMSec < 0) return -1;
	return elapsedTimeMSec / 1000;
}

int Transfer::GetEstimatedTimeSec()
{
	if(estimatedTimeMSec < 0) return -1;
	return estimatedTimeMSec / 1000;
}

int Transfer::GetRemainingTimeSec()
{
	if(remainingTimeMSec < 0) return -1;
	return remainingTimeMSec / 1000;
}

int Transfer::Open(char *srcPath)
{
	// init params
	char actual_srcPath[256];
	IFO *openIFO = (IFO *)this->openIFO;

	strcpy(this->srcPath, srcPath);

	// add backslash to the end of pathname
	int len = strlen(this->srcPath);
	if(this->srcPath[len - 1] != '\\')
	{
		this->srcPath[len]	 = '\\';
		this->srcPath[len + 1] = '\0';
	}

	// search for a VIDEO_TS subfolder
	char vtsPath[256];
	strcpy(vtsPath, this->srcPath);
	strcat(vtsPath, "VIDEO_TS");

	WIN32_FIND_DATA fdata;
	HANDLE hFindFile = FindFirstFile(vtsPath, &fdata);
	if(hFindFile != INVALID_HANDLE_VALUE)
	{
		if(strcmpi(fdata.cFileName, "VIDEO_TS") == 0 && (fdata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY)
		{
			strcat(this->srcPath, "VIDEO_TS\\");
		}
		FindClose(hFindFile);
	}

	// set the path to the video files
	strcpy(actual_srcPath, this->srcPath);
	strcat(actual_srcPath, "VIDEO_TS.IFO");
	openIFO->Load(actual_srcPath);

	// check for size of source-directories
	strcpy(actual_srcPath, this->srcPath);
	__int64 sourceSize = EvaluatePath(actual_srcPath);
	// if there are no files or incorrect srcPath 
	if(sourceSize == 0) return -4;

	sourceSizeKB = sourceSize / 1024;
	sourceSizeMB = sourceSizeKB / 1024;

	// search for encryption (if decryption is disabled)
	if(useDeCSS == 0)
	{
		char dummy = 0;
		strcpy(actual_srcPath, this->srcPath);
		int result = SearchEncryption(actual_srcPath, &dummy);
		// if video is scrambled
		if(result == -2) return -2;
	}

	opened = 1;

	return 1;
}

// sets the desired target size in megabytes
int Transfer::SetTargetSizeMB(int targetSize)
{
	targetSizeMB = targetSize;
	return 1;
}

// return the target size in megabytes
int Transfer::GetTargetSizeMB()
{
	return targetSizeMB;
}

// forces the target size to be 99% (force=1), instead of 95% (0) of the target size
int Transfer::ForceTargetSize(bool force)
{
	forceTargetSize = force;

	return forceTargetSize;
}										

// return the size of 
//   - the whole source DVD (movieOnly = false) 
//	 - the main movie (movieOnly = true)
int Transfer::GetSourceSizeMB(bool movieOnly)
{
	if(opened != 1)	return -5;

	if(movieOnly) return GetMovieOnlySizeMB();
	return sourceSizeMB;
}

float Transfer::GetCompressionRatio()
{
	return compressionRatio < 1 ? 1 : compressionRatio;
}

int Transfer::GetNumVTS()
{
	if(opened != 1)	return -5;

	IFO *openIFO = (IFO *)this->openIFO;
	return openIFO->GetNumVTS();
}

int Transfer::GetVTSSize(int VTSIndex)
{
	if(opened != 1) return -5;

	char actual_srcPath[256];

	int fileSize = 0;

	// if real titleset was specified
	if(VTSIndex > 0)
	{
		// get IFO and BUP size
		sprintf(actual_srcPath,    "%sVTS_%02i_0.IFO", srcPath,    VTSIndex);
		fileSize += 2 * GetFileSizeSECT(actual_srcPath);

		// read VOBs sizes
		for(int j = 0; j < 10; j++)
		{
			sprintf(actual_srcPath,    "%sVTS_%02i_%1i.VOB", srcPath,    VTSIndex, j);
			int size = GetFileSizeSECT(actual_srcPath);
			if(size == 0) break;
			fileSize += size;
		}
	}
	// if menu was specified
	else
	{
		// get IFO and BUP size
		sprintf(actual_srcPath,    "%sVIDEO_TS.IFO", srcPath);
		fileSize += 2 * GetFileSizeSECT(actual_srcPath);

		// read VOB sizes
		sprintf(actual_srcPath,    "%sVIDEO_TS.VOB", srcPath);
		fileSize += GetFileSizeSECT(actual_srcPath);
	}

	// return file size in MB
	return fileSize / 512;

}

int Transfer::GetNumVTSLang(int VTSIndex)
{
	if(opened != 1)	return -5;
	if(VTSIndex >= GetNumVTS()) return -1;

	IFO *openIFO = (IFO *)this->openIFO;
	return openIFO->GetNumVTSLang(VTSIndex);
}

int Transfer::GetVTSLangDesc(int VTSIndex, int langIndex, char *buffer, int szBuffer)
{
	if(opened != 1)	return -5;
	if(VTSIndex >= GetNumVTS()) return -1;
	if(langIndex >= GetNumVTSLang(VTSIndex)) return -1;

	IFO *openIFO = (IFO *)this->openIFO;
	return openIFO->GetVTSLangDesc(VTSIndex, langIndex, buffer, szBuffer);
}

int Transfer::GetVTSLangSelected(int VTSIndex, int langIndex)
{
	if(opened != 1)	return -5;
	if(VTSIndex >= GetNumVTS()) return -1;
	if(langIndex >= GetNumVTSLang(VTSIndex)) return -1;

	return SelectTable[VTSIndex][0][langIndex];
}

int Transfer::GetVTSLangDefault(int VTSIndex, int langIndex)
{
	if(opened != 1)	return -5;
	if(VTSIndex >= GetNumVTS()) return -1;
	if(langIndex >= GetNumVTSLang(VTSIndex)) return -1;

	if(defAud == -1) return -9;
	return defAud;
}

int Transfer::SetVTSLangSelected(int VTSIndex, int langIndex, int selected)
{
	if(opened != 1)	return -5;
	if(VTSIndex >= GetNumVTS()) return -1;
	if(langIndex >= GetNumVTSLang(VTSIndex)) return -1;

	SelectTable[VTSIndex][0][langIndex] = selected;
	if(langIndex == defAud && selected == 0) return 2;
	return 1;
}

int Transfer::SetVTSLangDefault(int VTSIndex, int langIndex)
{
	if(opened != 1)	return -5;
	if(VTSIndex >= GetNumVTS()) return -1;
	if(langIndex >= GetNumVTSLang(VTSIndex)) return -1;

	defAud = langIndex;
	if(defAud != -1)
	{
		if(SelectTable[VTSIndex][0][defAud] == 0) return 2;
	}
	return 1;
}

int Transfer::GetNumVTSSub(int VTSIndex)
{
	if(opened != 1)	return -5;
	if(VTSIndex >= GetNumVTS()) return -1;

	IFO *openIFO = (IFO *)this->openIFO;
	return openIFO->GetNumVTSSub(VTSIndex);
}

int Transfer::GetVTSSubDesc(int VTSIndex, int subIndex, char *buffer, int szBuffer)
{
	if(opened != 1)	return -5;
	if(VTSIndex >= GetNumVTS()) return -1;
	if(subIndex >= GetNumVTSSub(VTSIndex)) return -1;

	IFO *openIFO = (IFO *)this->openIFO;
	return openIFO->GetVTSSubDesc(VTSIndex, subIndex, buffer, szBuffer);
}

int Transfer::GetVTSSubSelected(int VTSIndex, int subIndex)
{
	if(opened != 1)	return -5;
	if(VTSIndex >= GetNumVTS()) return -1;
	if(subIndex >= GetNumVTSSub(VTSIndex)) return -1;

	return SelectTable[VTSIndex][1][subIndex];
}

int Transfer::GetVTSSubDefault(int VTSIndex, int subIndex)
{
	if(opened != 1)	return -5;
	if(VTSIndex >= GetNumVTS()) return -1;
	if(subIndex >= GetNumVTSSub(VTSIndex)) return -1;

	if(defSub == -1) return -9;
	return defSub;
}

int Transfer::SetVTSSubSelected(int VTSIndex, int subIndex, int selected)
{
	if(opened != 1)	return -5;
	if(VTSIndex >= GetNumVTS()) return -1;
	if(subIndex >= GetNumVTSSub(VTSIndex)) return -1;

	SelectTable[VTSIndex][1][subIndex] = selected;
	if(subIndex == defSub && selected == 0) return 2;
	return 1;
}

int Transfer::SetVTSSubDefault(int VTSIndex, int subIndex)
{
	if(opened != 1)	return -5;
	if(VTSIndex >= GetNumVTS()) return -1;
	if(subIndex >= GetNumVTSSub(VTSIndex)) return -1;

	defSub = subIndex;
	if(defSub != -1)
	{
		if(SelectTable[VTSIndex][1][defSub] == 0) return 2;
	}
	return 1;
}

int Transfer::GenerateIgnoreTable(int VTSnum, int VOBnum)
{
	int *PGCMarks;
	AVTABLE *AVStreamID;
	int reference;
	IFO *currIFO = (IFO *)this->currIFO;
	STREAM_IGNORETABLE *IgnoreTable = (STREAM_IGNORETABLE *)this->IgnoreTable;

	if(VOBnum == 0) reference = REFERENCE_MENU_VOB;
	else			reference = REFERENCE_TITLE_VOB;

	// get PGC marks and stream ids
	int numPGCs = currIFO->GetPGCMarks(reference, &PGCMarks);
	currIFO->GetAVTable(reference, &AVStreamID);

	int numAudio = currIFO->GetNumLang(reference);
	int numVideo = currIFO->GetNumSub(reference);

	if(IgnoreTable != NULL) 
	{
		int i = -1;
		do
		{
			i++;
			delete IgnoreTable[i].streamID;
			delete IgnoreTable[i].subStreamID;
		}
		while(IgnoreTable[i].nextPGCMark != INT_MAX);
		delete IgnoreTable;
	}
	IgnoreTable = new STREAM_IGNORETABLE[numPGCs];
	this->IgnoreTable = IgnoreTable;

	int c = 0;

	// create ignore table
	for(int i = 0; i < numPGCs; i++)
	{
		// if this PGC was not processed completely or at all
		int proceed = 0;
		if(i == numPGCs - 1) proceed = 1;
		else if(PGCMarks[i + 1] >= 0) proceed = 1; // >= tsTargetWrittenVOB removed, >= 0 instead

		if(proceed == 1)
		{
			if(i < numPGCs - 1)	IgnoreTable[c].nextPGCMark = PGCMarks[i + 1];
			else				IgnoreTable[c].nextPGCMark = INT_MAX;

			IgnoreTable[c].streamID    = new int[8 + 32 * 4];
			IgnoreTable[c].subStreamID = new int[8 + 32 * 4];
			IgnoreTable[c].numStream = 0;

			// process audio
			int indexAudio, numAudio;
			if(VOBnum == 0) 
			{
				indexAudio = 0;
				numAudio = currIFO->GetNumLang(REFERENCE_MENU_VOB);
			}
			else
			{
				indexAudio = currIFO->GetNumLang(REFERENCE_MENU_VOB);
				numAudio = currIFO->GetNumLang(REFERENCE_TITLE_VOB);
			}

			// go through all audio streams
			for(int j = 0; j < numAudio; j++)
			{
				// if audio stream is to be ignored
				if(SelectTable[VTSnum][0][indexAudio + j] == 0 && AVStreamID[i].audioAvailable[j] == 1)
				{
					IgnoreTable[c].streamID[IgnoreTable[c].numStream]    = AVStreamID[i].audioStreamType[j] == STREAM_SUBSTREAM ? 0xBD : AVStreamID[i].audioStreamNum[j];
					IgnoreTable[c].subStreamID[IgnoreTable[c].numStream] = AVStreamID[i].audioStreamType[j] == STREAM_SUBSTREAM ? AVStreamID[i].audioStreamNum[j] : 0;
					IgnoreTable[c].numStream++;
				}
			}

			// process substreams
			int indexVideo, numVideo;
			if(VOBnum == 0) 
			{
				indexVideo = 0;
				numVideo = currIFO->GetNumSub(REFERENCE_MENU_VOB);
			}
			else
			{
				indexVideo = currIFO->GetNumSub(REFERENCE_MENU_VOB);
				numVideo = currIFO->GetNumSub(REFERENCE_TITLE_VOB);
			}

			// go through all audio streams
			for(j = 0; j < numVideo; j++)
			{
				// if audio stream is to be ignored
				if(SelectTable[VTSnum][1][indexVideo + j] == 0 && AVStreamID[i].videoAvailable[j] == 1)
				{
					IgnoreTable[c].streamID[IgnoreTable[c].numStream]    = 0xBD;
					IgnoreTable[c].subStreamID[IgnoreTable[c].numStream] = AVStreamID[i].videoStreamNum[j * 4 + 0];
					IgnoreTable[c].numStream++;
					IgnoreTable[c].streamID[IgnoreTable[c].numStream]    = 0xBD;
					IgnoreTable[c].subStreamID[IgnoreTable[c].numStream] = AVStreamID[i].videoStreamNum[j * 4 + 1];
					IgnoreTable[c].numStream++;
					IgnoreTable[c].streamID[IgnoreTable[c].numStream]    = 0xBD;
					IgnoreTable[c].subStreamID[IgnoreTable[c].numStream] = AVStreamID[i].videoStreamNum[j * 4 + 2];
					IgnoreTable[c].numStream++;
					IgnoreTable[c].streamID[IgnoreTable[c].numStream]    = 0xBD;
					IgnoreTable[c].subStreamID[IgnoreTable[c].numStream] = AVStreamID[i].videoStreamNum[j * 4 + 3];
					IgnoreTable[c].numStream++;
				}
			}

			c++;
		}
	}

	return 1;

}

// sets a pointer to an alert object
// the alert object can be changed at will before and after setting it in transfer
void Transfer::SetAlert(TransferAlert *alert)
{
	this->alert = alert;
	alert->SetTransfer(this);
}

// return the size of a title in MegaBytes (either menu or film)
// sums over split VOBs
int Transfer::GetVOBSizeMB(char *srcPath, int VTSnum, int VOBnum)
{
	return GetVOBSizeSECT(srcPath, VTSnum, VOBnum) / 512;
}

// return the size of a title in sectors (either menu or film)
// sums over split VOBs
int Transfer::GetVOBSizeSECT(char *srcPath, int VTSnum, int VOBnum)
{
	WIN32_FILE_ATTRIBUTE_DATA fileData;
	int sizeSECT = 0;
	int ret;
	char actualPath[256];

	do
	{
		if(VTSnum == 0)	sprintf(actualPath, "%sVIDEO_TS.VOB", srcPath);
		else			sprintf(actualPath, "%sVTS_%02d_%d.VOB", srcPath, VTSnum, VOBnum);

		ret = GetFileAttributesEx(actualPath, GetFileExInfoStandard, &fileData);

		// if successful
		if(ret != 0) sizeSECT += fileData.nFileSizeLow / 2048;

		// next VOB number
		VOBnum++;
	} while(ret != 0 && VTSnum != 0 && VOBnum != 1);

	return sizeSECT;
}

int Transfer::GetMovieOnlyVTSNum()
{
	int movieVTS = -1, movieSize = -1;

	// get the number of the biggest title set
	for(int i = 0; i < GetNumVTS(); i++)
	{
		if(GetVTSSize(i) > movieSize)
		{
			movieSize = GetVTSSize(i);
			movieVTS = i;
		}
	}	

	return movieVTS;
}

int Transfer::GetMovieOnlySizeMB()
{
	int movieVTS = GetMovieOnlyVTSNum();
	return GetVTSSize(movieVTS) - GetVOBSizeMB(srcPath, movieVTS, 0);
}

