/*****************************************************************************
 *TransferAlert.cpp: Callback Class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "StdAfx.h"
#include "Transfer.h"

/*** plain C functions ***/
TransferAlert TransAlert;

extern "C"
{
	void _stdcall SetCallBack(int (_stdcall *pCallBack)(int alert, int param, void *pTransfer, void *pUser), void *pUser)
	{ TransAlert.SetCallBack(pCallBack, pUser); }

	void _stdcall AddAlert(int alert) { TransAlert.AddAlert(alert); }
	void _stdcall SubAlert(int alert) { TransAlert.SubAlert(alert); }
	bool _stdcall GetAlert(int alert) { return TransAlert.GetAlert(alert); }
	int  _stdcall GetAllAlert()       { return TransAlert.GetAllAlert();   }
	void _stdcall SubAllAlert()       { return TransAlert.SubAllAlert();   }

	void _stdcall SetPreviewDC(HDC hDC) { TransAlert.SetPreviewDC(hDC); }
}


TransferAlert::TransferAlert(void)
{
	this->pCallBack = NULL;
	hDC = NULL;
}

TransferAlert::~TransferAlert(void)
{
}

// ***for alert receivers

// for C++ Languages: defines a CallBack to be overriden
// initilaization for non C++ languages
int TransferAlert::CallBack(int alert, int param, void *pTransfer)
{
	if(pCallBack != NULL)
	{
		return pCallBack(alert, param, pTransfer, pUser);
	}

	return -1;
}

// for non C++ Languages: sets a pointer to a callback function
void TransferAlert::SetCallBack(int (_stdcall *pCallBack)(int alert, int param, void *pTransfer, void *pUser), void *pUser)
{
	this->pCallBack = pCallBack;
	this->pUser = pUser;
}

// activates one or more alerts
void TransferAlert::AddAlert(int alert)
{
	flags |= alert;
}

// deactivates one or more alerts
void TransferAlert::SubAlert(int alert)
{
	flags &= ~alert;
}

// checks whether all given alerts are set (returns true if all are set, else false)
bool TransferAlert::GetAlert(int alert)
{
	if((flags | alert) == flags) return true;
	return false;
}

// retrieves the alert flags
int  TransferAlert::GetAllAlert()
{
	return flags;
}

// deactivates all alerts
void TransferAlert::SubAllAlert()
{
	flags = 0;
}

// sets a device context for preview pictures
void TransferAlert::SetPreviewDC(HDC hDC)
{
	this->hDC = hDC;
}

// *** for alert senders

// must be set by transfer object that received this object
void TransferAlert::SetTransfer(void *pTransfer)
{
	this->pTransfer = pTransfer;
}

// sent by alerting objects
int TransferAlert::SendAlert(int alert, int param)
{
	if((alert & flags) != 0)
	{
		return CallBack(alert, param, pTransfer);
	}

	return -1;
}

// gets the device context for preview pictures
HDC TransferAlert::GetPreviewDC()
{
	return hDC;
}

