/*****************************************************************************
 * VLC.cpp: Read and Write VLC Tables
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/


#include "StdAfx.h"
#include "vlc.h"

VLC::VLC(void)
{
}

VLC::~VLC(void)
{
}

// search a VLC in a table
int VLC::ReadVLC()
{
	int czero = 0;
	static const unsigned int mmask[8] = { 0x00, 0x01, 0x03, 0x07, 0x0F, 0x1F, 0x3F, 0x7F };

	if (bitPos == -1) { pBuffer++; bitPos = 7; }

	unsigned int bpos = 16 + bitPos; // formula: 23 - (7 - bitPos);
	unsigned int mask = 1 << bpos; 
	unsigned int buffer = (pBuffer[0] << 16) | (pBuffer[1] << 8) | pBuffer[2];

	// count zeroes
	while(!(buffer & mask)) 
	{
		mask >>= 1;
		czero++; // count zeroes before the first one
	}

	// get number of digits following
	int begin_index = indices[czero];
	int cdigits = table[begin_index].digits;

	// read value
//	int val = (1 << (cdigits - 1)) | ReadBits(cdigits - 1);
	int shift = bpos - czero - cdigits + 1;
	int val = (1 << (cdigits - 1)) | ((buffer & (mmask[cdigits - 1] << shift)) >> shift);

	// search table for value
	int index = begin_index;
	while(table[index].number != val) 
	{
		index++;
		// check for possible errors in VLC code
		_ASSERTX(table[begin_index].zeroes == table[index].zeroes);
		// if number of digits is greater than expected
		if(table[index].digits > cdigits)
		{
			// calculate new digits
			int delta_digits = table[index].digits - cdigits;
			// increase digits counter
			cdigits += delta_digits;
			// read another bit
			val <<= delta_digits;// | ReadBits(delta_digits);
			shift -= delta_digits;
			val |= (buffer & (mmask[delta_digits] << shift)) >> shift;

		}
	}

	bitPos -= czero + cdigits;
	while (bitPos < 0) { pBuffer++; bitPos += 8; }

/*	shift = czero + cdigits;
	bitPos -= shift & 0x7;
	if (bitPos < 0) { pBuffer++; bitPos += 8; }
	pBuffer += shift >> 3;*/

	// return position in table
	return index;
}

// search a VLC in a table
/*int VLC::ReadVLC()
{
	int czero = 0;
	static const unsigned int mask[8] = { 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80 };

	if (bitPos == -1) { pBuffer++; bitPos = 7; }

	// count zeroes
	while((*pBuffer & mask[bitPos--]) == 0) 
	{
		if (bitPos == -1) { pBuffer++; bitPos = 7; }

		czero++; // count zeroes before the first one
	}

	// get number of digits following
	int begin_index = indices[czero];
	int cdigits = table[begin_index].digits;

	// read value
	int val = (1 << (cdigits - 1)) + ReadBits(cdigits - 1);

	// search table for value
	int index = begin_index;
	while(table[index].number != val) 
	{
		index++;
		// check for possible errors in VLC code
		_ASSERTX(table[begin_index].zeroes == table[index].zeroes);
		// if number of digits is greater than expected
		if(table[index].digits > cdigits)
		{
			// calculate new digits
			int delta_digits = table[index].digits - cdigits;
			// increase digits counter
			cdigits += delta_digits;
			// read another bit
			val = (val << delta_digits) + ReadBits(delta_digits);
		}
	}

	// return position in table
	return index;
}
*/

//  *** last thorough tested valid version ***
// search a VLC in a table
/*int VLC::ReadVLC()
{
	int czero = 0;
	int val;
	// count zeroes
	while((val = ReadBit()) == 0) czero++;

	_ASSERTX(val != -1);

	// get number of digits following
	int begin_index = indices[czero];
	int cdigits = table[begin_index].digits;

	// read value
	val = (val << (cdigits - 1)) + ReadBits(cdigits - 1);

	// search table for value
	int index = begin_index;
	while(table[index].number != val) 
	{
		index++;
		// check for possible errors in VLC code
		_ASSERTX(table[begin_index].zeroes == table[index].zeroes);
		// if number of digits is greater than expected
		if(table[index].digits > cdigits)
		{
			// calculate new digits
			int delta_digits = table[index].digits - cdigits;
			// increase digits counter
			cdigits += delta_digits;
			// read another bit
			val = (val << delta_digits) + ReadBits(delta_digits);
		}
	}

	// return position in table
	return index;
}*/

// search a VLC in a simple table
int VLC::ReadSimpleVLC(VLC_SIMPLEENTRY *table)
{
	// init value
	int cdigits = table[0].digits;
	int val = ReadBits(cdigits);

	// search table for value
	int index = 0;
	while(table[index].number != val) 
	{
		index++;
		// if number of digits is greater than expected
		if(table[index].digits > cdigits)
		{
			// calculate new digits
			int delta_digits = table[index].digits - cdigits;
			// increase digits counter
			cdigits += delta_digits;
			// read another bit
			val = (val << delta_digits) + ReadBits(delta_digits);
		}
	}

	// return position in table
	return index;
}

// write a VLC from the table
int VLC::WriteVLC(int tablePos)
{
	int zeroes = table[tablePos].zeroes;

/*	pBuffer += zeroes >> 3;
	bitPos -= zeroes & 0x07;

	if(bitPos < 0)
	{
		bitPos += 8;
		pBuffer++;
	}
*/
//	Seek(zeroes);

	// write VLC
	WriteBits(table[tablePos].number, zeroes + table[tablePos].digits);

	// write VLC return the number of bits written
	return zeroes + table[tablePos].digits;
}

//  *** last thorough tested valid version ***
// write a VLC from the table
/*int VLC::WriteVLC(int tablePos)
{
	// write zeroes
	WriteBits(0, table[tablePos].zeroes);
	// write rest of digits
	WriteBits(table[tablePos].number, table[tablePos].digits);

	// return the number of bits written
	return table[tablePos].zeroes + table[tablePos].digits;
}*/

// write a VLC from a simple table
int VLC::WriteSimpleVLC(VLC_SIMPLEENTRY *table, int tablePos)
{
	// write digits
	WriteBits(table[tablePos].number, table[tablePos].digits);

	// return the number of bits written
	return table[tablePos].digits;
}
