/*****************************************************************************
 * MultiLanguage.cpp: Language Selction Dialog class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "stdafx.h"
#include "ShrinkTo5Gui.h"
#include "MultiLanguage.h"
#include ".\multilanguage.h"


// CMultiLanguage-Dialogfeld


CMultiLanguage::CMultiLanguage(CWnd* pParent /*=NULL*/)
	: CBDialog(CMultiLanguage::IDD, pParent)
{
}

CMultiLanguage::~CMultiLanguage()
{
}

void CMultiLanguage::DoDataExchange(CDataExchange* pDX)
{
	CBDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LANG, mLangSel);
}


BEGIN_MESSAGE_MAP(CMultiLanguage, CBDialog)
	ON_WM_CTLCOLOR()
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
END_MESSAGE_MAP()


// CMultiLanguage-Meldungshandler
/*****************************************************************************
* Create the dialog and fill/load all settings
*****************************************************************************/
BOOL CMultiLanguage::OnInitDialog()
{
	CBDialog::OnInitDialog();

	//Set Bitmap to dialog
	SetBitmapStyle(StyleCenter);
	SetBitmap(IDB_MULTILANG);

	//Make headers bold
	LOGFONT new_logfont; 
	GetDlgItem(IDC_HEADER1)->GetFont()->GetLogFont(&new_logfont); 
	new_logfont.lfWeight = FW_BOLD;
	new_font.CreateFontIndirect(&new_logfont); 
	GetDlgItem(IDC_HEADER1)->SetFont(&new_font);

	//Get the Path
	int Pos = 0;
	long lPos;
	::GetModuleFileName(AfxGetApp()->m_hInstance, EncLangFile.GetBuffer(MAX_PATH), MAX_PATH);
	EncLangFile.ReleaseBuffer();
	lPos = EncLangFile.ReverseFind('\\');
	if (lPos !=-1)
		EncLangFile = EncLangFile.Left(lPos+1);	
	EncLangFile += "language.ini";

	CString strText;
	CString strKey;
	TCHAR chText[1000];

	//Read in all available languages
	if(GetPrivateProfileString(_T("Languages"), _T("count"), _T("#####"), chText, 1000, EncLangFile)){
		int xCount = _wtoi(chText);
		for(int i = 1;i<=xCount;i++){
			strKey.Format(_T("%i"),i);
			if(GetPrivateProfileString(_T("Languages"), strKey, _T("#####"), chText, 1000, EncLangFile))
				mLangSel.AddString(CString(chText));
		}
	}else
		mLangSel.AddString(_T("english"));

	//Set actual language selected
	if(GetPrivateProfileString(_T("Languages"), _T("default"), _T("#####"), chText, 1000, EncLangFile)){
		if(_wtoi(chText)==0)
			mLangSel.SetCurSel(0);
		else
			//mLangSel.SetCurSel(_wtoi(chText)-1);
			mLangSel.SetCurSel(_wtoi(chText)-1);
	}else
		mLangSel.SetCurSel(0);

	return TRUE;  // return TRUE unless you set the focus to a control
}
/*****************************************************************************
* Using to make Controls transparent on the bitmappped dialog
*****************************************************************************/
HBRUSH CMultiLanguage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	HBRUSH hbr = CBDialog::OnCtlColor(pDC, pWnd, nCtlColor);

	if (pWnd->GetDlgCtrlID() == IDC_HEADER1 )
		{      
		pDC->SetTextColor(RGB(102,102,102)); //Textfarbe 
		pDC->SetBkMode(TRANSPARENT);
		return (HBRUSH)GetStockObject(NULL_BRUSH);
		}
	return hbr;
}
/*****************************************************************************
* Save settings to the main dialog and write values to ini file
*****************************************************************************/
void CMultiLanguage::OnBnClickedOk()
{
	CString strKey;
	strKey.Format(_T("%i"),mLangSel.GetCurSel()+1);
	WritePrivateProfileString(_T("Languages"), _T("default"), strKey, EncLangFile);
	mLangSel.GetWindowText(LangID);
	OnOK();
}
