/*****************************************************************************
 * WorkWindow.cpp: Worker Dialog class
 *****************************************************************************
 * Copyright (C) 2004-2005 ShrinkTo5
 *
 * Authors: Ocrana		 <info@shrinkto5.com>
 *          Andrej Grecu <info@shrinkto5.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "stdafx.h"
#include "ShrinkTo5Gui.h"
#include "WorkWindow.h"
#include ".\workwindow.h"


// CWorkWindow-Dialogfeld

IMPLEMENT_DYNAMIC(CWorkWindow, CDialog)
CWorkWindow::CWorkWindow(CWnd* pParent /*=NULL*/)
	: CBDialog(CWorkWindow::IDD, pParent)
{
}

CWorkWindow::~CWorkWindow()
{
}

void CWorkWindow::DoDataExchange(CDataExchange* pDX)
{
	CBDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_FRAME, infFrame);
	DDX_Control(pDX, IDC_FPS, infFPS);
	DDX_Control(pDX, IDC_DATAWRITTEN, infDataOut);
	DDX_Control(pDX, IDC_ELTIME, infElapse);
	DDX_Control(pDX, IDC_ESTTIME, infEstim);
	DDX_Control(pDX, IDC_REMTIME, infRemain);
	DDX_Control(pDX, IDC_BURNPROGRESS, m_burnprogress);
	DDX_Control(pDX, IDC_PREVIEW, cPreview);
}


BEGIN_MESSAGE_MAP(CWorkWindow, CBDialog)
	ON_WM_CTLCOLOR()
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
END_MESSAGE_MAP()


// CWorkWindow-Meldungshandler

BOOL CWorkWindow::OnInitDialog()
{
	CBDialog::OnInitDialog();

	SetBitmapStyle(StyleCenter);
	SetBitmap(IDB_WORKWINDOW);

	//Paint all Header in dark grey/bold
    LOGFONT new_logfont; 	
	GetDlgItem(IDC_HEADER)->GetFont()->GetLogFont(&new_logfont); 
	new_logfont.lfWeight = FW_BOLD;
	new_font.CreateFontIndirect(&new_logfont); 
	GetDlgItem(IDC_HEADER)->SetFont(&new_font);

	//--------------------------------------------------------
	//Calculate Tray Window. Works only if Taskbar is Bottom
	CRect myOwn;
	this->GetWindowRect(&myOwn);

	CRect rectDesktopWithoutTaskbar;   // The desktop area 
	::SystemParametersInfo(SPI_GETWORKAREA, 0, &rectDesktopWithoutTaskbar, 0);

	m_nWndWidth  = myOwn.Width();
    m_nWndHeight = myOwn.Height();

    m_nWndLeft   = rectDesktopWithoutTaskbar.right - m_nWndWidth;
    m_nWndTop    = rectDesktopWithoutTaskbar.bottom - m_nWndHeight;
    m_nWndRight  = m_nWndLeft + m_nWndWidth;
    m_nWndBottom = m_nWndTop + m_nWndHeight;

    SetWindowPos(
        &wndTopMost,
        m_nWndLeft,
        m_nWndBottom - m_nWndHeight, //-5 
        m_nWndWidth, //-10
        m_nWndHeight,
        SWP_SHOWWINDOW
    );
	//--------------------------------------------------------

	return TRUE;  // return TRUE unless you set the focus to a control
	// AUSNAHME: OCX-Eigenschaftenseite muss FALSE zurckgeben.
}

/*****************************************************************************
* Using to make Controls transparent on the bitmappped dialog
*****************************************************************************/
HBRUSH CWorkWindow::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	HBRUSH hbr = CBDialog::OnCtlColor(pDC, pWnd, nCtlColor);

	if (pWnd->GetDlgCtrlID() == IDC_STATICG1 || pWnd->GetDlgCtrlID() == IDC_STATICG2  || pWnd->GetDlgCtrlID() == IDC_STATICG3 || pWnd->GetDlgCtrlID() == IDC_STATICG4)
		{      
		pDC->SetTextColor(RGB(102,102,102)); //Textfarbe 
		pDC->SetBkMode(TRANSPARENT);
		return (HBRUSH)GetStockObject(NULL_BRUSH);
		}
	if (pWnd->GetDlgCtrlID() == IDC_STATICG5 || pWnd->GetDlgCtrlID() == IDC_STATICG6  || pWnd->GetDlgCtrlID() == IDC_FRAME || pWnd->GetDlgCtrlID() == IDC_FPS)
		{      
		pDC->SetTextColor(RGB(102,102,102)); //Textfarbe 
		pDC->SetBkMode(TRANSPARENT);
		return (HBRUSH)GetStockObject(NULL_BRUSH);
		}
	if (pWnd->GetDlgCtrlID() == IDC_DATAWRITTEN || pWnd->GetDlgCtrlID() == IDC_ELTIME  || pWnd->GetDlgCtrlID() == IDC_ESTTIME || pWnd->GetDlgCtrlID() == IDC_REMTIME)
		{      
		pDC->SetTextColor(RGB(102,102,102)); //Textfarbe 
		pDC->SetBkMode(TRANSPARENT);
		return (HBRUSH)GetStockObject(NULL_BRUSH);
		}
		if (pWnd->GetDlgCtrlID() == IDC_HEADER)
		{      
		pDC->SetTextColor(RGB(255,255,255)); //Textfarbe 
		pDC->SetBkMode(TRANSPARENT);
		return (HBRUSH)GetStockObject(NULL_BRUSH);
		}
		
	return hbr;
}

void CWorkWindow::OnBnClickedCancel()
{
	::AfxGetApp()->m_pMainWnd->ShowWindow(SW_RESTORE);
	//opener->ShowWindow(SW_RESTORE);
	this->ShowWindow(SW_HIDE);
}

/*****************************************************************************
 * FillMultiLangGui
 * Set the new Static Text
 *****************************************************************************/
void CWorkWindow::FillMultiLangGui()
{
	this->GetDlgItem(IDC_HEADER)->SetWindowText(GetText(HEADER_4));

	this->GetDlgItem(IDC_STATICG1)->SetWindowText(GetText(STATIC_1));
	this->GetDlgItem(IDC_STATICG2)->SetWindowText(GetText(STATIC_2));
	this->GetDlgItem(IDC_STATICG3)->SetWindowText(GetText(STATIC_3));
	this->GetDlgItem(IDC_STATICG4)->SetWindowText(GetText(STATIC_4));
	this->GetDlgItem(IDC_STATICG5)->SetWindowText(GetText(STATIC_5));
	this->GetDlgItem(IDC_STATICG6)->SetWindowText(GetText(STATIC_6));
	this->GetDlgItem(IDCANCEL)->SetWindowText(GetText(RESTORE_1));
}
